
//**********************************************************************
//
//  MPLREGS.H
//
//  Copyright (c) 2004 National Semiconductor Corporation.
//  All Rights Reserved
//
//  Defines the device registers (MAC and PHY)
//
//**********************************************************************

#ifndef _MPLREGS_H
#define _MPLREGS_H

// Reset intervals
#define RESETINTERVAL_MAC   1000 // interval from MAC reset (in usec)
#define EELOADINTERVAL      1000 // interval from EELOAC (usec)
#define RESETINTERVAL_PHY   1000 // interval from MAC reset (in usec)

// MacPhy PHY IDs
#define MP_PHY_ID1        0x2000
#define MP_PHY_REV        0x5C20 //Prior to MP3
#define ASPEN_PHY_REV     0x5C90
#define PHY_MII_REG_BASE  0x80 

// MacPhy MAC IDs
#define MPI_C_MAC_ID      0x0302 // Macphyter-I Rev-C
#define MPI_D_MAC_ID      0x0403 // Macphyter-I Rev-D
#define MPII_A4_MAC_ID    0x0504 // Macphyter-II Rev-A4
#define MPII_A5_MAC_ID    0x0505 // Macphyter-II Rev A5
#define MPIII_MAC_ID      0x0601 // Macphyter-III

// All packed on byte boundaries
#pragma pack(1)

//    Operational Register Map for MacPhyter2/3
typedef volatile struct _DP8381xOpRegs {

    // Basic MAC (0x00 to 0x5C)
    NS_UINT32   CR;         // Command register 
    NS_UINT32   CFG;        // Cfg and media status 
    NS_UINT32   MEAR;       // Mii, EEprom Access 
    NS_UINT32   MTSCR;      // MAC Test Control 
    NS_UINT32   ISR;        // Interrupt Status 
    NS_UINT32   IMR;        // Interrupt Mask 
    NS_UINT32   IER;        // Interrupt enable 
    NS_UINT32   IHR;        // Interrupt holdoff 
    NS_UINT32   TXDP;       // Tx descriptor ptr, queue 0 
    NS_UINT32   TXCFG;      // Transmit configuration 
    NS_UINT32   pad1;       // Reserved 
    NS_UINT32   GPIOR;      // General I/O pin control 
    NS_UINT32   RXDP;       // Rx descriptor ptr, queue0 
    NS_UINT32   RXCFG;      // Receive configuration 
    NS_UINT32   pad2;       // Reserved 
    NS_UINT32   CCSR;       // PME backdoor 
    NS_UINT32   WCSR;       // Wake-on-LAN command Status 
    NS_UINT32   PCR;        // Pause frame control status 
    NS_UINT32   RFCR;       // Receive filter control 
    NS_UINT32   RFDR;       // Receive filter data 
    NS_UINT32   BRAR;       // Boot rom address 
    NS_UINT32   BRDR;       // Boot rom data 
    NS_UINT32   SRR;        // Silicon revision 
    NS_UINT32   MIBC;       // MIB statistics Control 
    
    // MIB regs (0x60 to 0x7C)
    NS_UINT32   RXE;        // Packets recd with errors 
    NS_UINT32   RXFCS;      // Packets recd with frame check seq err 
    NS_UINT32   RXM;        // Packets missed due to FIFO overruns 
    NS_UINT32   RXFAE;      // Packets recd with frame alignment err 
    NS_UINT32   RXS;        // Packets recd with symbol err 
    NS_UINT32   RXF;        // Packets recd with len > 1518 
    NS_UINT32   TXSQE;      // Packets with loss of collision during 
                            //  transmission 
    NS_UINT32   RXIRL;      // Packets recd with InRange length errs 

    // Internal PHY (0x80 to 0x9C)
    NS_UINT32   BMCR;       // Basic Mode Control Register 
    NS_UINT32   BMSR;       // Basic Mode Status Register 
    NS_UINT32   PHYIDR1;    // PHY Id reg1 
    NS_UINT32   PHYIDR2;    // PHY Id reg2 
    NS_UINT32   ANAR;       // Auto-Neg Adv reg 
    NS_UINT32   ANLPAR;     // Auto-Neg Link partner reg 
    NS_UINT32   ANER;       // Auto-Neg expansion reg 
    NS_UINT32   ANNPTR;     // Auto-Neg next page TX 

    // Priority Queuing (0xA0 to 0xAC)
    NS_UINT32   TXDP1;      // Tx descriptor ptr, queue1 
    NS_UINT32   TXDP2;      // Tx descriptor ptr, queue2 
    NS_UINT32   TXDP3;      // Tx descriptor ptr, queue3 
    NS_UINT32   PQCR;       // Prority Queue Control reg

    // VLAN control (0xB0 to 0xB4)
    NS_UINT32   VRCR;       // VLAN receive control 
    NS_UINT32   VTCR;       // VLAN transmit control 

    NS_UINT32   pad3;       // Reserved 
    NS_UINT32   pad4;       // Reserved 

    // PHY Status (0xC0)
    NS_UINT32   PHYSTS;     // Phy Status Register

    // PHY MII Regs (0xC4 and 0xC8)
    NS_UINT32   MICR;       // Phy MII Interrupt Control
    NS_UINT32   MISR;       // Phy MII Interrupt Status

    // PHY Interal Regs
    NS_UINT32   PAGE;       // 0xCC page select
    union _RegD0 {
        NS_UINT32  FCSCR;   // 0xD0 (Page 0) false carrier sense counter 
        NS_UINT32  FCOCTL;  // 0xD0 (Page 1) FCO control 
    } RegD0;
#define FCSCR  RegD0.FCSCR
#define FCOCTL RegD0.FCOCTL

    union _RegD4 {
        NS_UINT32  RECR;    // 0xD4 (Page 0) receiver error counter 
        NS_UINT32  TMR;     // 0xD4 (Page 1) test mode 
    } RegD4;
#define RECR RegD4.RECR 
#define TMR RegD4.TMR

    union _RegD8 {
        NS_UINT32  PCS;     // 0xD8 (Page 0) PCS sub layer config & status 
        NS_UINT32  BGR;     // 0xD8 (Page 1) band gap reference 
    } RegD8;
#define PCS RegD8.PCS
#define BGR RegD8.BGR

    union _RegDC {
        NS_UINT32  RBR;     // 0xDC (Page 0) RMII & bypass 
        NS_UINT32  CRM;     // 0xDC (Page 1) CRM 
    } RegDC;
#define RBR RegDC.RBR
#define CRM RegDC.CRM

    NS_UINT32  CDCT12;      // 0xE0 (Page 1) CD test control 2 

    union _RegE4 {
        NS_UINT32  PHYCTL;  // 0xE4 (Page 0) phy control 
        NS_UINT32  PMDCSR;  // 0xE4 (Page 1) PMD control/status 
    } RegE4;
#define PHYCTL RegE4.PHYCTL
#define PMDCSR RegE4.PMDCSR

    union _RegE8 {
        NS_UINT32  TENBT;   // 0xE8 (Page 0) 10BaseT status/control 
        NS_UINT32  PGMCGM;  // 0xE8 (Page 1) PGM/CGM control 
    } RegE8;
#define TENBT RegE8.TENBT
#define PGMCGM RegE8.PGMCGM

    union _RegEC {
        NS_UINT32  CDCTL1;  // 0xEC (Page 0) CD test control 1 
        NS_UINT32  DSPTST;  // 0xEC (Page 1) DSP test 
    } RegEC;
#define CDCTL1 RegEC.CDCTL1
#define DSPTST RegEC.DSPTST

    NS_UINT32  EXTCFG;      // 0xF0 (Page 1) extended config 
    NS_UINT32  DSPCFG;      // 0xF4 (Page 1) DSP config 
    NS_UINT32  SDCFG;       // 0xF8 (Page 1) signal detect config. 
    NS_UINT32  TDATA;       // 0xFC (Page 1) test data 

} DP8381xOpRegs;

#pragma pack()  // reset back to the default packing 



// OPERATIONAL REGISTER  BIT DEFINITIONS
// (R/W = read/write, RO = read only, ROSC = read only, self-clearing,
//  WR0 = write, read back 0, !825 = DP83825 only)

//***************************************************************
// CR - command register bit definitions
//***************************************************************
#define TX_EN           0x00000001      // transmitter enable (R/W) 
#define TX_DIS          0x00000002      // transmitter disable (WR0) 
#define RX_EN           0x00000004      // receiver enable (R/W) 
#define RX_DIS          0x00000008      // receiver disable (WR0) 
#define TXRESET         0x00000010      // transmitter reset (WR0) 
#define RXRESET         0x00000020      // receiver reset (WR0) 
                                        // bit 6 (reserved) 
#define SWINT           0x00000080      // software interrupt 
#define SOFTRESET       0x00000100      // soft reset (W read 0  when comp) 
#define TXQUEUE_MASK    0x00001E00      // Tx priority queue mask 
#define TXQUEUE0_EN     0x00000200      // Tx Priority Queue 0 (R/W) 
#define TXQUEUE1_EN     0x00000400      // Tx Priority Queue 1 (R/W) 
#define TXQUEUE2_EN     0x00000800      // Tx Priority Queue 2 (R/W) 
#define TXQUEUE3_EN     0x00001000      // Tx Priority Queue 3 (R/W) 
#define TX_PQUEUE_OFF   9               // Bit Offset for Tx Pri Q setting 


//***************************************************************
// CFG - Configuration and Media Status Register 
//***************************************************************
#define BIG_ENDIAN      0x00000001    // big endian (DMA only) (R/W)
#define TMRTEST         0x00000002    // speeds up 100us internal timer to 4us
#define BROM_DIS        0x00000004    // disable Boot ROM I/F (R/W) 
#define SERR_ON_PAR_ERR 0x00000008    // SERR on PCI parity err (R/W) 
#define ABORT_ON_EXCESSIVE_DEF 0x00000010 // abrt on excessive deferral (R/W) 
#define OOWTMR_AT_PRESTRT 0x00000020  // out-of-window at preamble start (R/W) 
#define SINGLETXBCKOFF  0x00000040    // Single IFG tx Backoff (R/W) 
#define PCIREQ_WEAK     0x00000080    // conservative PCI bus req algo (R/W) 
#define EXTENDEDDESC_EN 0x00000100    // Extended descriptor format en (R/W) 
#define PHY_DIS         0x00000200    // dise phy (deassert mii RXEN) (R/W) 
#define PHY_RST         0x00000400    // reset phy (assert PHYRST_N) (R/W) 
#define BEM_REG         0x00000800    // Big endian reg mode (R/W)-DP83818
#define EXT_PHY         0x00001000    // Enable External Phy Support (R/W) 

// Auto-Neg Select - FM

#define PAUSE_ADV       0x00010000    // pause Advertise (R/W) 
#define PINT_ACEN       0x00020000    // phy int auto clear (R/W) 

// PHY Configuration - FM

#define AUTO_DUP        0x01000000    // Ena Auto Full duplex (R/W) 
#define ANEG_DN         0x08000000    // Auto Neg done 
#define POL_10M         0x10000000    // 10M with reverse polarity detected (RO) 
#define FULLDUP_STS     0x20000000    // full duplex indication from phy (RO) 
#define LINKSPEED100    0x40000000    // 100 Mbps indication from phy (RO) 
#define LINKGOOD_STS    0x80000000    // good link indication from phy (RO) 


//***************************************************************
// MEAR - MII / EEPROM Access Register
//***************************************************************
#define EEPROM_DI       0x00000001  // EEPROM data in (ouR data out) (R/W) 
#define EEPROM_DO       0x00000002  // EEPROM data out (our data in) (RO) 
#define EEPROM_CLK      0x00000004  // EEPROM clock (R/W) 
#define EEPROM_CS       0x00000008  // EEPROM chip select (R/W) 
#define MDIO            0x00000010  // MII management data input/output (R/W) 
#define MDDIR           0x00000020  // MII management data dir, 1 = out (R/W) 
#define MDCLK           0x00000040  // MII management data clock (R/W) 
#define EEPROM_WRITE    0x00000140  // EEPROM Write Command
#define EEPROM_READ     0x00000180  // EEPROM Read Command
#define EEPROM_ERASE    0x000001C0  // EEPROM Erase Command

enum EEPROM_Cmds {
	EE_WriteCmd=(5 << 6), EE_ReadCmd=(6 << 6), EE_EraseCmd=(7 << 6),
};

#define MII_READ        0x6003FFFF
#define MII_WRITE       0x50020000
#define MII_ADDR_OFF    23
#define MII_REG_OFF     18
#define MII_CMD_OFF     16

//***************************************************************
// MTSCR - MAC Test Control Register
//***************************************************************
#define EEBIST_FAIL     0x00000001      // EE Bist fail indication (RO)
#define EEBIST_EN       0x00000002      // Enable EEPROM BIST (R/W)
#define EELOAD_EN       0x00000004      // Enable EEPROM Load (R/W)
#define RBIST_RXRFAIL   0x00000008      // RX FIlter RAM BIST Fail (RO)
#define RBIST_TXFAIL    0x00000010      // TX FIFO Fail (RO)
#define RBIST_RXFAIL    0x00000020      // RX FIFO BIST Fail (RO)
#define RBIST_DONE      0x00000040      // SRAM BIST Done (RO)
#define RBIST_EN        0x00000080      // SRAM BIST Enable (R/W)
#define RBIST_MODE      0x00000100      // SRAM BIST Mode - Reserved NSC (R/W)
#define RBIST_CLKD      0x00000200      // SRAM BIST Clock - Reserved NSC (R/W)
#define RBIST_RST       0x00000400      // SRAM BIST Reset (R/W)
#define RBIST_REL       0x00000800      // SRAM BIST Reliablity Mode (R/W)
#define FORCE_CRS       0x00001000      // Force CRS (R/W)
#define RXFIFO2K        0x00002000      // Select 2KB RX FIFO (R/W)


//***************************************************************
// ISR/IMR - Interrupt bit definitions
//***************************************************************
#define RXOK            0x00000001  // RX OK interrupt 
#define RXDESC          0x00000002  // RX descriptor interrupt 
#define RXERR           0x00000004  // RX packet error 
#define RXEARLY         0x00000008  // RX Early Threshold 
#define RXIDLE          0x00000010  // RX Idle 
#define RXOVERUN        0x00000020  // RX Overrun 
#define TXOK            0x00000040  // TX OK interrupt 
#define TXDESC          0x00000080  // TX descriptor interrupt 
#define TXERR           0x00000100  // TX packet error 
#define TXIDLE          0x00000200  // TX idle 
#define TXUNDERRUN      0x00000400  // TX underrun 
#define MIBSTATS        0x00000800  // MIB statistics service 
#define SW_INT          0x00001000  // software interrupt 
#define WOL_EVENT       0x00002000  // Wake-On-LAN power management event 
#define PHYSTSCHANGE    0x00004000  // phy status change 

#define HIORDER_INT     0x00008000  // one of the following int's occured 
#define RXSTSFIFO_OVRN  0x00010000  // rx status fifo overrun 
#define PCI_TARGABORT   0x00100000  // received PCI target abort 
#define PCI_MSTRABORT   0x00200000  // generated PCI master abort 
#define PCI_SERR        0x00400000  // generated PCI system error 
#define PCI_PARERR      0x00800000  // detected PCI parity error 
#define RXRST_COMPLETE  0x01000000  // receive reset complete 
#define TXRST_COMPLETE  0x02000000  // transmit reset complete 
#define TXDESC0         0x04000000  // Tx Descriptor for priority Q 0 
#define TXDESC1         0x08000000  // Tx Descriptor for priority Q 1 
#define TXDESC2         0x10000000  // Tx Descriptor for priority Q 2 
#define TXDESC3         0x20000000  // Tx Descriptor for priority Q 3 

#define TXDESC_MASK     0x3C000000  // Tx Descriptor INTR mask 
#define TXDESC_OFF      26          // Tx Descriptor INTR offset 

//**************************************************************
// IER - Interrupt Enable Register
//***************************************************************
#define INT_EN              0x00000001  // Intr Ena (0 masks all ints) (R/W) 
#define AUTODISABLEINT_EN   0x00000002  // en auto disable (read of ISR disables
                                        //   all int's until INT_EN set (R/W)
                                        // bits 2-31 (reserved) 


//**************************************************************
// IHR - Interrupt Holdoff Register
//***************************************************************
#define INTHLD_VAL_MASK     0x000000FF  // mask for interrupt holdoff value (R/W) 
#define STARTHLDOFFON1INT   0x00000100  // start interrupt holdoff on 1st int 
                                        // (if clr, hold begins on int en) (R/W)
#define INTHLDTMR_4US       0x00000200  // set holdoff tick to 4us (if clear,
                                        //  tick=100us (R/W) 
                                        // bits 10-15 (reserved) 
#define INTHLDCNCL_RXPKTS_SHIFT 16
#define INTHLDCNCL_RXPKTS_MASK  0x001F0000  // # of rcv packets that will 
                                            //  prematurely terminate intr
                                            //  holdoff (R/W)
#define INTHLDCNCL_TXPKTS_SHIFT 21
#define INTHLDCNDL_TXPKTS_MASK  0x03E00000  // # of rcv packets that will 
                                            //  prematurely terminate intr 
                                            //  holdoff (R/W) 

                                            // bits 26-31 (reserved) 

#define MAX_HOLD_TIME     (0xFF*100)        // Max intr hold timeout value
                                            //  255*100usec 
#define HOLD_TIME_MASK    0xFFFFFF00        // Hold time mask

//***************************************************************
// TXCFG - Transmit Configuration Register
//***************************************************************
#define MXDMA4          0x00100000  // 4  bytes (R/W)   
#define MXDMA8          0x00200000  // 8  bytes (R/W)   
#define MXDMA16         0x00300000  // 16 bytes (R/W)   
#define MXDMA32         0x00400000  // 32 bytes (R/W)   
#define MXDMA64         0x00500000  // 64 bytes (R/W)   
#define MXDMA128        0x00600000  // 128 bytes (R/W)  
#define MXDMA256        0x00700000  // 256 bytes (R/W)  
#define MXDMA512        0x00000000  // 512 bytes (R/W)  

#define TXDMA4          0x00020000  // scale MXDMA table value setting by 4 
#define BOFF_CONT       0x00040000  // enable backoff continuos 
#define BLINDTX_EN      0x00080000  // enable blind transmit (R/W) !825 
#define EXC_COLN_RETRY  0x00800000  // en excessive collision retry (R/W) 
#define AUTOTXPAD_EN    0x10000000  // en Automatic Tx Padding (R/W) 
#define LOOPBACK_EN     0x20000000  // enable MAC loopback mode (R/W) 
#define SQE_IGNORE      0x40000000  // dis SQE (heart beat) test (R/W) 
#define CARRIER_IGNORE  0x80000000  // carrier sense ignore (R/W) 

#define TXDRAIN_MASK    0x0000003F  // transmit drain thresh mask 
#define TXFILL_MASK     0x00003F00  // transmit fill thresh mask 
#define TXFILL_SHIFT    8
#define TXMXDMA_MASK    0x00700000  // max tx DMA mask 
#define TXSET_MASK  (TXDMA4 | TXDRAIN_MASK | TXFILL_MASK | TXMXDMA_MASK)


//***************************************************************
// GPIOR - General Purpose I/O Register
//***************************************************************
#define GP1_OUT         0x00000001  // General purpose Pin 1 Output (R/W)
#define GP2_OUT         0x00000002  // General purpose Pin 2 Output (R/W)
#define GP3_OUT         0x00000004  // General purpose Pin 3 Output (R/W)
#define GP4_OUT         0x00000008  // General purpose Pin 4 Output (R/W)
#define GP1_OE          0x00000010  // General purpose Pin 1 Output Enable (R/W)
#define GP2_OE          0x00000020  // General purpose Pin 2 Output Enable (R/W)
#define GP3_OE          0x00000040  // General purpose Pin 3 Output Enable (R/W)
#define GP4_OE          0x00000080  // General purpose Pin 4 Output Enable (R/W)
#define GP1_IN          0x00000100  // General purpose Pin 1 Input (RO) 
#define GP2_IN          0x00000200  // General purpose Pin 2 Input (RO) 
#define GP3_IN          0x00000400  // General purpose Pin 3 Input (RO) 
#define GP4_IN          0x00000800  // General purpose Pin 4 Input (RO) 


//**************************************************************
// RXCFG - Receive Configuration Register
//***************************************************************
#define RXDRAIN_SHIFT       1
#define RXDRAIN_MASK        0x0000003E  // rx drain threshold mask 

#define RXDMA4              0x00040000  // scale MXDMA table value setting by 4
#define DMA_USECACHESIZ_EN  0x00080000  // rx dma multiples of cacheline size
                                        //   (R/W)

#define RXMXDMA_MASK        0x00700000  // max rx DMA mask 
#define RXSET_MASK          (RXDRAIN_MASK | RXMXDMA_MASK | RXDMA4) 

// max rx DMA burst sizes...use TXCFG MXDMAxxx definitions

#define ACCEPT_INRNG_LENERR 0x04000000  // accept in-range-length error'd
                                        //   pkts (R/W) 
#define ACCEPT_LONGPKT      0x08000000  // accept long packets (R/W) 
#define FULLDUPLEX_EN       0x10000000  // en. full duplex (R/W) 
#define STRIPCRC_EN         0x20000000  // en strip CRC (R/W) 
#define ACCEPT_RUNTS        0x40000000  // accept runts (R/W) 
#define ACCEPT_CRCALIGNERR  0x80000000  // accept CRC and alignment err's (R/W) 

//**************************************************************
// CCSR - PME Backdoor Register
//***************************************************************
#define PMEEN               0x00000100 // PME Enable (R/W) - 
                                       //   - Mirrored from PMCSR 
#define PMESTS              0x00008000 // PME (sticky) Status (R/W) 
                                       //   - Mirrored from PMCSR 

//**************************************************************
// WCSR - Wake Command / Status Register. 
//***************************************************************
#define WAKEON_PHYSTSCHNG   0x00000001  // wake on phy sts change int (R/W) 
#define WAKEON_UNICAST      0x00000002  // wake on any unicast pkt (R/W) 
#define WAKEON_MULTICAST    0x00000004  // wake on multicast pkt (R/W) 
#define WAKEON_BROADCAST    0x00000008  // wake on broadcast pkt (R/W) 
#define WAKEON_ARPPKT       0x00000010  // wake on TCP/IP ARP (R/W) 
#define WAKEON_PATRN0       0x00000020  // wake on pattern #0 match (R/W) 
#define WAKEON_PATRN1       0x00000040  // wake on pattern #1 match (R/W) 
#define WAKEON_PATRN2       0x00000080  // wake on pattern #2 match (R/W) 
#define WAKEON_PATRN3       0x00000100  // wake on pattern #3 match (R/W) 
#define WAKEON_MAGICPKT     0x00000200  // wake on magic pkt (R/W) 
#define MAGICPKT_SECURE_EN  0x00000400  // en magic pkt secure on (R/W) 
#define WAKEON_PATRN4       0x00000800  // wake on pattern #4 match (R/W) 
#define WAKEON_PATRN5       0x00001000  // wake on pattern #5 match (R/W) 
#define WAKEON_PATRN6       0x00002000  // wake on pattern #6 match (R/W) 
#define WAKEON_PATRN7       0x00004000  // wake on pattern #7 match (R/W) 

#define WAKESTS_PATRN4      0x00020000  // pattern4 match wake event (ROSC) 
#define WAKESTS_PATRN5      0x00040000  // pattern5 match wake event (ROSC) 
#define WAKESTS_PATRN6      0x00080000  // pattern6 match wake event (ROSC) 
#define WAKESTS_PATRN7      0x00100000  // pattern7 match wake event (ROSC) 
#define WAKESTS_SEC_HACK    0x00200000  // magic pkt security hack event (ROSC)
#define WAKESTS_PHYSTSCHNG  0x00400000  // phy sts change wake event (ROSC)
#define WAKESTS_UNICAST     0x00800000  // unicast pkt wake event (ROSC) 
#define WAKESTS_MULTICAST   0x01000000  // multicast pkt wake event (ROSC) 
#define WAKESTS_BROADCAST   0x02000000  // broadcast pkt wake event (ROSC) 
#define WAKESTS_ARPPKT      0x04000000  // ARP pkt wake event (ROSC) 
#define WAKESTS_PATRN0      0x08000000  // pattern0 match wake event (ROSC) 
#define WAKESTS_PATRN1      0x10000000  // pattern1 match wake event (ROSC) 
#define WAKESTS_PATRN2      0x20000000  // pattern2 match wake event (ROSC) 
#define WAKESTS_PATRN3      0x40000000  // pattern3 match wake event (ROSC) 
#define WAKESTS_MAGICPKT    0x80000000  // magic pkt wake event (ROSC) 

#define WAKEON_MASK         0x0000EFFF  // Alll wake-on bits

//**************************************************************
// PCR - Pause Control / Status Register.
//***************************************************************
#define PAUSEVAL_MASK           0x0000FFFF  // pause cntr value mask 

#define PS_TX_DP83818           0x00020000  // Manual pause frame load enable 
#define PS_TX                   0x00010000  // Manual pause frame load enable 

#define PS_DA                   0x20000000  // en pause  rcv sent to MAC addr
                                            //  or any pattern match (R/W) 
#define PS_MCAST                0x40000000  // en pause pkt rcv sent to IEEE
                                            //  reserved mcast address (R/W) 
#define RXPAUSE_EN              0x80000000  // en. pause frame reception (R/W) 

// pause threshold low values - rx data fifo  - DP83818+
#define PTHRESHLO_DFIFO_MASK    0x000C0000  // data fifo thresh low mask (R/W) 
#define PTHRESHLO_DFIFO_DIS     0x00000000  // low thresh disabled (R/W) 
#define PTHRESHLO_DFIFO512      0x00040000  // low thresh < 512 bytes (R/W) 
#define PTHRESHLO_DFIFO1024     0x00080000  // low thresh < 1K bytes (R/W) 
#define PTHRESHLO_DFIFO1536     0x000C0000  // low thresh < 1536 bytes (R/W) 

// pause threshold hi values - rx data fifo  - DP83818+
#define PTHRESHHI_DFIFO_MASK    0x00300000  // data fifo thresh hi mask (R/W) 
#define PTHRESHHI_DFIFO_DIS     0x00000000  // hi thresh disabled (R/W) 
#define PTHRESHHI_DFIFO512      0x00100000  // hi thresh >= 512 bytes (R/W) 
#define PTHRESHHI_DFIFO1024     0x00200000  // hi thresh >= 1K bytes (R/W) 
#define PTHRESHHI_DFIFO1536     0x00300000  // hi thresh >= 2K bytes (R/W) 

// pause threshold low values - rx status fifo  - DP83818+
#define PTHRESHLO_SFIFO_MASK    0x00C00000  // stat fifo thresh low mask (R/W) 
#define PTHRESHLO_SFIFO_DIS     0x00000000  // low thresh disabled (R/W) 
#define PTHRESHLO_SFIFO2PKTS    0x00400000  // low thresh < 2 pkts (R/W) 
#define PTHRESHLO_SFIFO4PKTS    0x00800000  // low thresh < 4 pkts (R/W) 
#define PTHRESHLO_SFIFO8PKTS    0x00C00000  // low thresh < 8 pkts (R/W) 

// pause threshold hi values - rx status fifo  - DP83818+
#define PTHRESHHI_SFIFO_MASK    0x03000000  // stat fifo thresh hi mask (R/W) 
#define PTHRESHHI_SFIFO_DIS     0x00000000  // hi thresh disabled (R/W) 
#define PTHRESHHI_SFIFO2PKTS    0x01000000  // hi thresh = 2 or more pkts (R/W) 
#define PTHRESHHI_SFIFO4PKTS    0x02000000  // hi thresh = 4 or more pkts (R/W) 
#define PTHRESHHI_SFIFO8PKTS    0x03000000  // hi thresh = 4 or more pkts (R/W) 



//**************************************************************
// RFCR - Receive Filter Control Register.
//***************************************************************
#define RXFLTRAM_ADDRMASK   0x000007ff

// FM - Need to check
#define MAX_FILTER_ROWS     32
#define MAX_FILTER_COLS     16
#define HASH_TABLE_INDEX    0x200       // beginning of the mcast/ucast hash
#define RCVPATTERNS_ADDR    0x280       // beginning of the receive pattern
                                        //  buffers 

#define ACCEPT_PTRNMATCH4   0x00008000  // en pattern match 0 (R/W) 
#define ACCEPT_PTRNMATCH5   0x00010000  // en pattern match 1 (R/W) 
#define ACCEPT_PTRNMATCH6   0x00020000  // en pattern match 2 (R/W) 
#define ACCEPT_PTRNMATCH7   0x00040000  // en pattern match 3 (R/W) 
#define IGNORE_ADDR_ULBIT   0x00080000  // ignore U/L bit in dest addr (R/W) 
#define UNICASTHASH_EN      0x00100000  // en unicast hash table (R/W) 
#define MCASTHASH_EN        0x00200000  // en multicast hash table (R/W) 
#define ACCEPT_ALLARP       0x00400000  // en reception of all ARP pkts (R/W) 

#define PATTRNMATCH_MASK    0x07800000  // pattern match en mask (R/W) 
#define ACCEPT_PTRNMATCH0   0x00800000  // en pattern match 0 (R/W) 
#define ACCEPT_PTRNMATCH1   0x01000000  // en pattern match 1 (R/W) 
#define ACCEPT_PTRNMATCH2   0x02000000  // en pattern match 2 (R/W) 
#define ACCEPT_PTRNMATCH3   0x04000000  // en pattern match 3 (R/W) 
#define ACCEPT_PERFECTMATCH 0x08000000  // en perfect match (R/W) 
#define ACCEPT_ALLUNICAST   0x10000000  // en reception of all unicast (R/W) 
#define ACCEPT_ALLMCAST     0x20000000  // en reception of all multicast (R/W) 
#define ACCEPT_ALLBCAST     0x40000000  // en reception of all broadcast (R/W) 
#define RXFLTR_EN           0x80000000  // ena recevie filtering.  
                                        //  Must be cleared to
                                        //   set other filter bits. (R/W) 

//**************************************************************
// RFDR - Receive Filter Data Register.
//***************************************************************
#define RXFLTRDATA_MASK     0x0000FFFF  // Recieve filter data mask (R/W) 
#define RXFLTRMSKBITS_MASK  0x00030000  // recive filter byte mask bits (R/W) 
#define RXFLTRMSKBIT_SHIFT  16          // required shift to point to the 
                                        //  mask bits 


//**************************************************************
// MIBC - MIB Control Register
//***************************************************************
#define MIBCNTRS_WRN    0x00000001  // Warning Test Indicator  (RO) 
#define MIBCNTRS_FREEZE 0x00000002  // freeze mib counters (WR0) 
#define MIBCNTRS_CLR    0x00000004  // clear all mib counters (W/R0) 
#define MIBCNTRS_STRB   0x00000008  // Strobe MIB counter (R/W) - Test Only 


//**************************************************************
// PQCR    - Priority Queuing Control Register
//***************************************************************
#define TXPRIOQ_EN  0x00000001  // en tx priority queuing (R/W) 
#define ROTPRIO_EN  0x00000002  // en rotating priority (i.e. 01230) If clear,
                                //  fixed priority (3 highest, 0 lowest) (R/W) 
#define Q0_QUOTA_SHIFT 8  // shift for Q0 quota (when ROTPRIO_EN set)
#define Q1_QUOTA_SHIFT 14 // shift for Q1 quota (when ROTPRIO_EN set)
#define Q2_QUOTA_SHIFT 20 // shift for Q2 quota (when ROTPRIO_EN set)
#define Q3_QUOTA_SHIFT 26 // shift for Q3 quota (when ROTPRIO_EN set)

//**************************************************************
// VRCR    -  VLAN Rx Control Reg
//***************************************************************
#define VTDEN 0x00000001  // VLAN tag detection enable
#define VTREN 0x00000002  // VLAN tag removal enable
#define DVTF  0x00000004  // Discard VLAN tagged frame
#define DUTF  0x00000008  // Discard untagged frame

//**************************************************************
// VTCR    -  VLAN Tx Control Reg
//***************************************************************
#define VGTI   0x00000001  // VLAN Global tag insertion
#define VPPTI  0x00000002  // VLAN per pkt tag insertion
#define VPAD68 0x00000004  // Tag VLAN frames to 68 bytes

#define VTCI_MASK  0x0000FFFF // tag mask
#define VTCI_SHIFT 16        // shift for tag contol field

//**************************************************************
// PHY Registers
//***************************************************************

// MII Base register offsets
#define MII_BMCR        0x00
#define MII_BMSR        0x01
#define MII_PHYIDR1     0x02
#define MII_PHYIDR2     0x03
#define MII_ANAR        0x04
#define MII_ANLPAR      0x05
#define MII_ANER        0x06
#define MII_ANNPTR      0x07

// BMCR - Basic Mode Control Register Definitions
#define PHY_RESET       0x8000     // Initiate soft reset of PHY
#define PHY_LOOPBACK    0x4000     // Put PHY in loopback mode
#define FORCE_SPEED_100 0x2000     // Force mode - 100Mbps
#define FORCE_SPEED_10  0x0000     // Force mode - 10Mbps
#define AUTONEG_ENABLE  0x1000     // Enable Auto-Neg
#define PHY_POWERDOWN   0x0800     // Power down the PHY
#define ISOLATE_PORT    0x0400     // Isolate this port from the MII
#define AUTONEG_RESTART 0x0200     // Restart Auto-Neg
#define FORCE_MODE_FD   0x0100     // Force mode - Full duplex
#define FORCE_MODE_HD   0x0000     // Force mode - Half duplex
#define COLL_TEST       0x0080     // Start Collision test

// BMSR - Basic Mode Status Register Definitions
#define AUTONEG_COMP    0x0020     // Auto nego process done
#define LINK_STATUS     0x0004     // Link Up
#define JABBER_DET      0x0002     // Jabber detected

// ANAR Definitions.
#define NP              0x8000     // Next Page Indication
#define ADV_RF          0x2000     // Advertise remote fault detection
#define ADV_ASM_DIR     0x0800     // Advertise Asymmetrical Pause
#define ADV_PAUSE       0x0400     // Advertise Symmetrical Pause
#define ADV_100BASET4   0x0200     // 100BASE-T4 Support
#define ADV_100BASET_FD 0x0100     // 100BASE-TX Full Duplex Support
#define ADV_100BASET_HD 0x0080     // 100BASE-TX Half Duplex Support
#define ADV_10BASET_FD  0x0040     // 10BASE-TX Full Duplex Support
#define ADV_10BASET_HD  0x0020     // 10BASE-TX Half Duplex Support
#define PSB             0x0001     // Protocol Selection - 802.3u

// PHYSTS Definitions - Some uncommon bits are not populated
#define POL_STS         0x1000     // Set when Inverted polarity is detected
#define INT_STS         0x0080     // Set when an internal interrupt is pending
#define RF_STS          0x0040     // Set when remote fault condition is 
                                   //  detected
#define JABBER_STS       0x0020    // Set when Jabber detected on line
#define AUTONEG_DONE_STS 0x0010    // Set when Auto-Neg is complete
#define LOOPBACK_STS     0x0008    // Set when PHY is in loopback mode
#define DUPLEX_FULL_STS  0x0004    // Set for Full duplex, reset for half
#define SPEED_10_STS     0x0002    // Set for 10Mbps, reset for 100Mbps
#define LINK_STS         0x0001    // Set when valid link is established

// MICR Definitions
#define PHY_INT          0x0002    // Propagate PHY intr to MAC
#define TINT             0x0001    // Indicate PHY to generate test intr

// MISR Definitions
#define MSK_LINK         0x4000    // Mask Link interrupt
#define MSK_ANC          0x0800    // Mask Auto-Neg done interrupt

#endif // _MPLREGS_H
