
        include defs.asm

;<<<===========================================================================
; ReadTickCounter
;
;  Reads the Timer tick count register which decrements by 2 from 65536
;  to 0 every 1/36.414 of a second.  Each 2 decrements of the count
;  represents 838 nsec's.
;
;  Inputs    :  None
;  Outputs   :  AX = timer tick count (decrease by 2)
;  Returns   :  None
;  Preserved :  None
;  Called by :  others
;===========================================================================>>>
                
ReadTickCounter proc    near

        mov     al, 06h
        out     43h, al                 ;Command 8254 to latch T0's count

        ;Read the Timer Tick Counter
        ;---------------------------
        in      al, 40h                 ;Read LSB first
        mov     ah, al
        in      al, 40h                 ;Read MSB
        xchg    al, ah                  ;AX = timer tick count (decrease by 2)
        ret

ReadTickCounter endp

;<<<===========================================================================
; Wait419nsec
;
;  Wait for 419 nsec (1 tick = 419 nsec).
;
;  Inputs    :  None
;  Outputs   :  None
;  Returns   :  None
;  Preserved :  None
;  Called by :  others
;===========================================================================>>>
                
Wait419nsec     proc    near

        push    ax
        push    cx

        ;Read the begin timer tick count
        ;-------------------------------
        call    ReadTickCounter
        mov     cx, ax                  ;AX = timer tick count (decrease by 2)
Wait419nsecLoop:
        ;Read the last timer tick count
        ;------------------------------
        call    ReadTickCounter

        sub     cx, ax
        cmp     cx, 0                   ;1 tick = 419 nsec
        jbe     Wait419nsecLoop

        pop     cx
        pop     ax
        ret

Wait419nsec     endp


;<<<===========================================================================
; Delay2usec
;
;  Wait for 2 usec.
;
;  Inputs    :  None
;  Outputs   :  None
;  Returns   :  None
;  Preserved :  None
;  Called by :  others
;===========================================================================>>>
                
Delay2usec       proc    near

        push    cx
        mov     cx, 5                ;about 2 usec (5 * 419 nsec)
Delay2usecLoop:
        call    Wait419nsec
        loop    Delay2usecLoop
        pop     cx
        ret

Delay2usec       endp


;<<<===========================================================================
; Delay20usec
;
;  Wait for 20 msec.
;
;  Inputs    :  None
;  Outputs   :  None
;  Returns   :  None
;  Preserved :  None
;  Called by :  others
;===========================================================================>>>
                
Delay20usec       proc    near

        push    cx
        mov     cx, 50                ;about 20 usec (50 * 419 nsec)
Delay20usecLoop:
        call    Wait419nsec
        loop    Delay20usecLoop
        pop     cx
        ret

Delay20usec       endp

