// Include files

#include <stdio.h>
#include <stdlib.h>
#include <conio.h> 
#include <dos.h>
#include "GLOBAL.H"
#include "NICSTUFF.H"
#include "FUNCTION.H"
#include "PCIBIOS.H"


// Function prototypes

void DisplayCLHelp(void);
void getAddress( int argc, char *argv[], int * );
void getNextNIC(void);
void getID( int argc, char *argv[], int * );
int Master(void);
int Slave(void);
int Loopback(char);
void displayRegs(char);
void testISR();
UINT16 flip_16 (UINT16 us);


// External Function Prototypes

extern int initNIC();
extern int  NICLoopback(int);
extern void DisplayNICRegs(void);
extern BYTE sendPing(void);
extern BYTE receivePing(void);
extern void interrupt far McPhyISR();

// Global Variables

char  cQuit=0,cTransmit=0,cReceive=0,cLoopback=0,cReturn=0,cMaster=0;
// unsigned short IRQ;
unsigned short RegBase=0;
int currentBus;
int currentDev;
int currentFunc;
unsigned short batchMode=0;
unsigned long PciCfgSpace[ 12 ]; 
BYTE PacketSize=60,Last=0;
long Iterations=0;
int NICsInstalled = 0;

extern MACAddr_t MACAddr;
extern RegisterTable_t PciTable[ PCI_TABLE_SIZE ];

/************************************************
*       Function:   DisplayCLHelp
*	Purpose:	Displays Command Line options
*	Arguments:  none
*	Returns:    nothing
*************************************************/
void DisplayCLHelp()
{
    printf("\n");
    printf("\nUSAGE:  DIAG [B] <test> [option]\n");
    printf("\nDIAG B <test> [option] - for use of DIAG in batch mode.\n");
    printf("\tG    Go to Next NIC.\n");
    printf("\tE    Initialize EEPROM with Default Values.\n");
    printf("\tC[M/V/D] [macAddress] Change MAC/subVendor/subDevice ID.\n");
    printf("\tM [# of packets]   Master mode.\n");
	printf("\tS    Slave mode.\n");
    printf("\tL[M/P/X]   Mac/Phy/eXternal loopback test\n");
    printf("\tT [# of packets]   Transmit test packet(s)\n");
	printf("\tR    Receive test packet(s)\n");
	printf("\tD[N/P/E] Display NIC/PCI/EEPROM Registers.\n");
    printf("\tP[M/L/S [macAddress]] Pwr Mgmt WakeOn MgkPkt/Link/Send MgkPkt.\n");
    printf("\tQ    QUIT utility.\n");

    printf( "\n\nEnter selection. \n" );

}

/************************************************
*	Function:   Master
*	Purpose:	Inputs how many times to perform
*				Ping test as master, and then
*				executes test that many times.
*				If cTransfer is 1, only transfers
*				are executed.
*	Arguments:  none
*	Returns:    1 if FAIL, 0 if Succees
*************************************************/
int Master(void)
{
	long x;

	if (initNIC())
	{
		return 1;
	}

	// Wait for the link being stable
	delay(1);

	if (!batchMode && (Iterations == 0))
	{
		printf( "\n\nNumber of Repetitions: " );
		scanf( "%ld", &Iterations );
	}

	if (Iterations)
	{
		for (x = 0; x < Iterations; x++)
		{
			if (x == (Iterations-1))
			{
				cQuit = 1;
				if (!sendPing())
				{
					return 1;
				}
				if (!cTransmit)
				{
					if (!(Last = receivePing()))
					{
						return 1;
					}
				}
			}
			else
			{
				if (cTransmit)
				{
					if (!sendPing())
					{
						return 1;
					}
				}
				else
				{
					if (!sendPing())
					{
						return 1;
					}
					if (!receivePing())
					{
						return 1;
					}
				}
			}
		}
	}
	else while (1)
	{ 
		if (!sendPing())
		{
			return 1;
		}
        if (kbhit())
		{
			getch();
			cQuit = 1;
			return 0;
		}
	}

	if (!cTransmit)
	{
		if (Last == 0x07)
		{
			printf("\n\nPing completed successfully.\n");
		}
		else
		{
			printf("\n\nPing failed.\n");
			return 1;
		}
	}
	
	cQuit = 0;

	return 0;
}

/************************************************
*	Function:   Slave
*	Purpose:	Performs the Ping test as a slave.
*				Waits for a transfer from the master,
*				and then sends back received packets
*				until it receives a packet ending
*				with a byte of 0x07.
*				If cReceive is 1, all received packets
*				are displayed.
*	Arguments:  none
*	Returns:    nothing
*************************************************/
int Slave(void)
{						
	if (initNIC())
	{
		return 1;
	}

	// Wait for the link being stable
	delay(1);

	while (!cQuit)
	{
		if (!(Last = receivePing()))
		{
			return 1;
		}
	}   	
    
    if (!cReceive)
	{
		if (Last == 0x07)
		{
			printf("\n\nPing completed successfully.\n");
		}
		else
		{
			printf("\n\nPing failed.\n");
		}
	}
		
	return 0;
}							


/************************************************
*	Function:   Loopback
*	Purpose:	Performs the selected loopback test
*	Arguments:  Mode - Loopback mode
*					M	MAC Loopback
*					P	PHY Loopback
*					X	External Loopback
*	Returns:    1 if FAIL, 0 if Succees
*************************************************/
int Loopback(char Mode)
{
	int lbMode;

	clearscreen();
	cLoopback = 1;

	switch (Mode)
	{
        case 'M':
			printf("MAC Loopback selected...\n");
			lbMode = 1;
			break;
        case 'P':
			printf("PHY Loopback selected...\n");
			lbMode = 2;
			break;			
        default:
			printf("External Loopback selected...\n");
			lbMode = 3;
			break;
	}
	
	if (NICLoopback(lbMode))
	{
		return 1;
	}
	
	return 0;
}

/************************************************
*	Function:   getAddress
*	Purpose:    display current address & prompt for new address
*	Arguments:  none
*	Returns:    nothing
*************************************************/
void getAddress( int argc, char *argv[], int *nicAddress )
{
	unsigned i;
	unsigned char inputString[ 25 ], res;

	// The trick is to route the MACaddress to the appropriate bins:
	// If in batch mode the address should start at ?
	if( batchMode || (argc == 8) )
	{
		// First check if there is a macAddress:
		if( argc < 8 )
		{
			batchMode = 0;
		}
		else // otherwise try to load the MacAddress:
		{
			for( i=0; i<6; i++ )
				sscanf
					(
					argv[ i + (batchMode? 3:2 )],
					"%02X",
					inputString+(i*2)
					);
		}
	}

	//Prompt for a new address:
    while( 1 )
    {
		// In case there is no MacAddress:
		if( batchMode || (argc == 8) )
		{
			break;
		}

		printf( "\nType in the MAC address with format: 'XX XX XX XX XX XX' \n" );
		scanf( "%02X %02X %02X %02X %02X %02X",
				inputString,
				inputString+2,
				inputString+4,
				inputString+6,
				inputString+8,
				inputString+10
				);

		printf( "%02X %02X %02X %02X %02X %02X\n",
				*inputString,
				*(inputString+2),
				*(inputString+4),
				*(inputString+6),
				*(inputString+8),
				*(inputString+10)
				);

		printf( "\nCorrect? (Y/N): " );
		scanf( "%s", &res );
		if( toupper( res ) == 'N' )
		{
			continue;
		}
		else
		{
			break;
		}
    }
    
    nicAddress[ 0 ] = (UINT16)*( inputString+2 );
    nicAddress[ 0 ] = (nicAddress[ 0 ] << 8) + (UINT16)*( inputString );

    nicAddress[ 1 ] = (UINT16)*( inputString+6 );
    nicAddress[ 1 ] = (nicAddress[ 1 ] << 8) + (UINT16)*( inputString+4 );

    nicAddress[ 2 ] = (UINT16)*( inputString+10 );
    nicAddress[ 2 ] = (nicAddress[ 2 ] << 8) + (UINT16)*( inputString+8 );
     
    return;
}

void getNextNIC( )
{
	unsigned int i, j, index;
	unsigned baseIOaddresses[4];
	int bus[4];
	int dev[4];
	int func[4];
	UINT16 nicAddress[ 3 ];
	unsigned vendorId = 0x100B, value; 
	unsigned deviceId = 0x20;
	unsigned long dvalue;


	// Clear Screen so we can display the list of NIC's
	clearscreen();

	// Tell the user that we are searching for up to 4 NIC's in the system:
	printf( "\n\nSearching for up to 4 NIC's in the system...." );

	// Look for as many as 4 NIC's
	for( i=0; i<4; i++ )
	{
		//PciBIOScalls to find the NIC:
		// Find the card on the PCI Bus:
		if( PciBiosFindDevice( vendorId, deviceId, i, &bus[i], &dev[i], &func[i] ) )
		{
			if( i == 0 )
			{
				printf( "\n\nMacPhyter Not Found on PCI Bus!" );
				return;
			}
			else break;
		}

	    PciBiosReadConfigWord( bus[i], dev[i], func[i], PciCfgBaseAddr0, &RegBase );

	    if( RegBase & 0x01 )
		{
			RegBase = RegBase & ~0x03;
		}
	
		// Save the BaseI/O address of each NIC found:
		baseIOaddresses[i] = RegBase;

		// Display each NIC found
		printf( "\n\n    %d. I/O Base: %04X\n", i, RegBase );

		ReadMACAddress(  nicAddress );
    
		printf( "       MAC Address: " );

		for( j=0; j<3; j++ )
		{
			MACAddr[ j*2+1 ] = ( unsigned char )( nicAddress[ j ] >> 8 );
			MACAddr[ j*2 ] = ( unsigned char )( nicAddress[ j ] );
			printf( "%02X %02X ", MACAddr[ j*2 ], MACAddr[ j*2+1 ] );
		}
	}

	NICsInstalled = i;

	// Prompt the user to enter the number next to the desired NIC:
	printf( "\n\n\nEnter the number next to the NIC you wish to select: \n" );
    scanf( "%d", &index );

	if( (index<0) || (index>=i) )
	{
		printf( "\nInvalid, but I'll use 0 anyway!\n" );
		index=0;
	}

	// Based on the selection change the BASE I/O address.
	RegBase = baseIOaddresses[index];

	currentBus = bus[index];
	currentDev = dev[index];
	currentFunc = func[index];

    for( j=0x0; j<PCI_TABLE_SIZE; j++ )
	{
		PciCfgSpace[ j ] = 0;
		PciBiosReadConfigWord( currentBus, currentDev, currentFunc, PciTable[ j ].regOffset, &(WORD)PciCfgSpace[ j ]  );
		PciBiosReadConfigWord( currentBus, currentDev, currentFunc, PciTable[ j ].regOffset + 2, &value );
		dvalue = (UINT32)value;
		dvalue <<= 16;
		PciCfgSpace[ j ] |= dvalue;
	}

	// Tell the User you are switching control over to the Selected NIC.
	printf( "\nNow switching DIAG to: %d. I/O Base: %04X\n ", index, RegBase );

	ReadMACAddress(  nicAddress );

	printf( "\t\t\t  MAC Address: " );

	for( j=0; j<3; j++ )
	{
		MACAddr[ j*2+1 ] = ( unsigned char )( nicAddress[ j ] >> 8 );
		MACAddr[ j*2 ] = ( unsigned char )( nicAddress[ j ] );
		printf( "%02X %02X ", MACAddr[ j*2 ], MACAddr[ j*2+1 ] ); 
	}
}

void getID( int argc, char *argv[], int *ID )
{
	unsigned inputString[ 25 ];
	unsigned char res;

	// The trick is to route the MACaddress to the appropriate bins:
	// If in batch mode the address should start at ?
	if( batchMode || (argc == 3) )
	{
		// First check if there is an ID:
		if( argc < 3 ) batchMode = 0;
		else // otherwise try to load the ID:
		{
			sscanf(argv[ (batchMode? 3:2 ) ], "%04X", inputString);
		}
	}

	//Prompt for a new ID:
	while( 1 )
	{
		// In case there is no MacAddress:
		if( batchMode || (argc == 3) ) break;

		printf( "\nType in the subsystem Vendor or Device ID format: 'XXXX' \n" );
		scanf( "%04X", inputString );

		printf( "%04X \n", *inputString );

		printf( "\nCorrect? (Y/N): " );
		scanf( "%s", &res );
		if( toupper( res ) == 'N' ) continue;
		else break;
	}

	*ID = flip_16(*inputString);

	return;
}

/* flip_16 - flip a 16 bit value  */
UINT16 flip_16 (UINT16 us)
{
	unsigned value = 0;
	int i;

	for (i = 0; i < 16; i++)
	{
		if (us & (1 << i))
		{
			value |= (0x8000 >> i);
		}
	}

	return value;
}



/************************************************
*	Function:   main
*	Purpose:	Start the ball rolling
*	Arguments:  command line options
*	Returns:    nothing
*************************************************/
void main( int argc, char *argv[] )
{   
	char ctemp1, ctemp2, inputString[ 25 ];
	int nicAddr[ 3 ], ssId;
	bool bShowInfo;

	if( argc > 1 )
	{
		ctemp1 = toupper(argv[1][0]);
		ctemp2 = toupper(argv[1][1]);
	}
	else
	{
		ctemp1 = 'H';
	}

	// Check for batchMode parameters:
	if (ctemp1 == 'B')
	{
		batchMode = 1;
		if( argc > 2 )
		{
			ctemp1 = toupper(argv[2][0]);
			ctemp2 = toupper(argv[2][1]);
		}
		else
		{
			ctemp1 = 'H';
		}
	}

	// Display a welcome screen:
	if (!batchMode)
	{
		clearscreen();
		printf( "\nWELCOME to DIAG.EXE version: %d.%d", MAJOR_REV, MINOR_REV );
		printf( "\n" );
	}

	if (findNIC(batchMode? FALSE:TRUE))
	{
		exit(1);
	}

	while (1) 
	{
		cTransmit = 0;
		cMaster = 0;
		cReceive = 0;
		cLoopback = 0;
		cReturn = 0;
		cQuit = 0;

		switch (ctemp1)
		{
			case 'C':   
				if (ctemp2 == 'D' || ctemp2 == 'V')
				{
					getID( argc, argv, &ssId );
					if (writeID( ssId, ctemp2 ))
					{
						exit(1);
					}
					bShowInfo = FALSE;
				}
				else if (ctemp2 =='M')
				{
					getAddress(argc, argv, nicAddr);
					if (writeMacAddress(nicAddr))
					{
						exit(1);
					}
					bShowInfo = TRUE;
				}

				// Call findNIC routine again to update PCI register contents and
				// display the new address
				if (findNIC(bShowInfo))
				{
					exit(1);
				}
				break;
			case 'E':
				if (writeEeDefaults())
				{
					exit(1);
				}
				// Call findNIC routine again to update PCI register contents
				if (findNIC(bShowInfo))
				{
					exit(1);
				}
				break;
			case 'G':
				getNextNIC( );
				break;
			case 'T':
				cTransmit = 1;
			case 'M':
				cMaster = 1;
				if (argc > 2)
				{
					Iterations = atol( argv[ batchMode? 3:2 ] );
					printf("Iterations = %ld\n", Iterations);
				}
				if (Master())
				{
					exit(1);
				}
				Iterations = 0;
				break;
			case 'R':
				cReceive = 1;
			case 'S':
				if (Slave())
				{
					exit(1);
				}
				break;
			case 'L':
				if (Loopback(ctemp2))
				{
					exit(1);
				}
				break;
			case 'D':
                displayRegs(ctemp2);
				HOLD
				break;
			case 'P':
				if (initNIC())
				{
					exit(1);
				}
				if (ctemp2 == 'M')
				{
					printf( "\nSetting Magic Pkt Mode.\n" );
					if (pwrMgtMode( 0x200 ))
					{
						exit(1);
					}
				}
				if (ctemp2 == 'S')
				{
					getAddress( argc, argv, nicAddr );
					if (sendMgkPkt( (BYTE *)nicAddr) )
					{
						exit(1);
					}
					HOLD
				}
				if (ctemp2 == 'L')
				{
					printf( "\nSetting Link Change Mode.\n" );
					if (pwrMgtMode( 0x1 ))
					{
						exit(1);
					}
				}
				break;
			case 'F':
				if (initNIC())
				{
					exit(1);
				}
				if( ctemp2 == '1' )
				{
					setMedia( 0x00000000 );
				}
				if( ctemp2 == '2' )
				{
					setMedia( 0x00004000 );
				}
				if( ctemp2 == '3' )
				{
					setMedia( 0x00008000 );
				}
				if( ctemp2 == '4' )
				{
					setMedia( 0x0000C000 );
				}
				break;
			case 'Q':
				exit(0);
				break;
			default:
				break;
		}

		if (batchMode && ctemp1 != 'G')
		{
			break;
		}

		DisplayCLHelp();
		scanf( "%s", &inputString );
		ctemp1 = toupper( inputString[ 0 ] );
		ctemp2 = toupper( inputString[ 1 ] );
	}

	exit(0);
}

void testISR()
{
	void (interrupt far *old_handler)();
	unsigned intno;

	old_handler = _dos_getvect( intno );
	
	//intno = PciBiosFindCardInt( );

	_disable();
	_dos_setvect( intno, mcPhyISR );
	_enable();
		
}		

void displayRegs( char Mode )
{
	switch (Mode)
	{
		case 'P':
			printf("\nPCI Config Registers:\n");
			DisplayPciRegs();
			break;
		case 'E':
			printf("\nEEPROM Contents:\n");
			DisplayEEPROM();
			break;			
		default:
			printf("\nOperational Registers:\n");
			DisplayNICRegs();
			break;
	}
}

