
/**********************************************************************
*									
* NICSTUFF.H								
*									
* This file contains various definitions for values used by		
* the NICSTUFF.C module.						
*									
***********************************************************************/

#define PCI_TABLE_SIZE  12

// Definitions for the NIC core register set.

// Page 0 - Writes  	
#define		CR		         	0x00	// Command Register
#define 	CFG    				0x04	// Page 0 - Reads
#define 	MEAR    			0x08	// Page 0 - Reads: EEPROM ACCESS REG

#define EEread          0x0180          /* .. read */

	// Bit Definitions for MEAR:
	#define 	EECS    		0x8		// 	
	#define 	EECLK    		0x4		// 	
	#define 	EEDO    		0x2		// 	
    #define 	EEDI    		0x1		// 	

#define 	PCITEST_CNTRL    	0x0C	// Page 0 - Reads
	// PCI TEST CNTRL Register:
	#define 	RBIST_RST		0x400		
	#define 	RBIST_CLKD		0x200		
	#define 	RBIST_MODE		0x100		
	#define 	RBIST_EN		0x80		
	#define 	RBIST_ACT		0x40		
	#define 	RBIST_RXFAIL	0x20		
	#define 	RBIST_TXFAIL	0x10		
	#define 	RBIST_RXFFAIL	0x8		
	#define 	EELOAD_EN		0x4		
	#define 	EEBIST_EN		0x2		
	#define 	EEBIST_FAIL		0x1
	
#define 	INT_STATUS 		    0x10
#define 	INT_MASK	        0x14
#define 	INT_ENABLE	        0x18
#define 	TX_CONFIG	        0x24
        #define TxCSI           0x80000000      /* carrier sense ignore */
        #define TxHBI           0x40000000      /* heartbeat ignore */
        #define TxMLB           0x20000000      /* MAC loopback */
        #define TxATP           0x10000000      /* automatic transmit padding */
        #define TxIFG           0x0C000000      /* (&) interframe gap time */
        #define TxECRETRY       0x00800000
        #define TxMXDMA         0x00700000      /* (64 bytes) tx max cycles per DMA burst */

        #define DMA4B           0x00100000
        #define DMA8B           0x00200000
        #define DMA16B          0x00300000
        #define DMA32B          0x00400000
        #define DMA64B          0x00500000
        #define DMA128B         0x00600000
        #define DMA256B         0x00700000
        #define DMA512B         0x00000000

        #define TXDRN512B       0x00000010
        #define TXDRN1024B      0x00000020
        #define TXDRN1472B      0x0000002e
        #define TXDRN1536B      0x00000030

        #define TXFIL32B        0x00000100
        #define TXFIL64B        0x00000200
        #define TXFIL96B        0x00000300
        #define TXFIL160B       0x00000500
        #define TXFIL288B       0x00000900
        #define TXFIL544B       0x00001100
#define 	TX_DP	            0x20
#define 	RX_DP            	0x30
#define 	RX_CONFIG	        0x34
        #define RxAEP           0x80000000      /* rx accept errored packets */
        #define RxARP           0x40000000      /* rx accept runt packets */
        #define RxATP           0x10000000      /* rx accept xmit packets (fdx) */
        #define RxALP           0x08000000      /* rx accept long packets */
        #define RxMXDMA         0x00700000      /* (&) rx max cycles per DMA burst */

        #define RXDRN8B         0x00000002
        #define RXDRN16B        0x00000004
        #define RXDRN24B        0x00000006
        #define RXDRN32B        0x00000008
        #define RXDRN40B        0x0000000a
        #define RXDRN48B        0x0000000c
        #define RXDRN56B        0x0000000e
        #define RXDRN64B        0x00000010
        #define RXDRN128B       0x00000020
        #define RXDRN248B       0x0000003e
#define     CCSR                0x3C
#define     WCSR                0x40
#define 	RX_FILTER_CTRL   	0x48
#define 	RX_FILTER_DATA   	0x4C
#define		SRR					0x58
#define 	MIB_CTRL         	0x5C

#define 	PHY_BMCR	        0x80

        /* Bit definition for PHY BMCR register */
        #define BMCR_RESET          0x8000
        #define BMCR_LOOPBACK       0x4000
        #define BMCR_100MBPS        0x2000
        #define BMCR_AUTONEG        0x1000
        #define BMCR_POWER_DOWN     0x0800
        #define BMCR_ISOLATE        0x0400
        #define BMCR_RESTART_ANEG   0x0200
        #define BMCR_FULL_DUPLEX    0x0100
        #define BMCR_COLLISION_TEST 0x0080

#define ANAR			0x90
#define ANLPAR			0x94

#define         PHY_PHYSTS              0xC0
#define         PHY_MICR                0xC4
#define         PHY_MISR                0xC8

#define PHY_PAGESEL      0xCC

// Page 0
#define PHY_FCSCR        0xD0
#define PHY_RECR         0xD4
#define PHY_PCSR         0xD8
#define PHY_RBR          0xDC
#define PHY_CONTROL      0xE4
#define PHY_10BTSCR      0xE8
#define PHY_CDCTRL1      0xEC

// Page 1
#define PHY_FCOCTRL      0xD0
#define PHY_TMR          0xD4
#define PHY_CDCTRL2      0xE0
#define PHY_PMDCSR       0xE4
#define PHY_PGMCGM       0xE8
#define PHY_CCDSPCFG     0xF4
#define PHY_CCSDCFG      0xF8
#define PHY_TSTDAT       0xFC

#define RX_FILTER_CTRL_RFEN     0x80000000
#define RX_FILTER_CTRL_AAB      0x40000000
#define RX_FILTER_CTRL_AAM      0x20000000
#define RX_FILTER_CTRL_AAU      0x10000000
#define RX_FILTER_CTRL_APM      0x08000000
#define RX_FILTER_CTRL_APAT     0x07800000
#define RX_FILTER_CTRL_AARP     0x00400000
#define RX_FILTER_CTRL_MHEN     0x00200000
#define RX_FILTER_CTRL_UHEN     0x00100000
#define RX_FILTER_CTRL_ULM      0x00080000
#define RX_FILTER_CTRL_RFADDR   0x000003FF
#define RX_FILTER_MASK          ( HAL_RX_FILTER_CTRL_RFEN | \
                                      HAL_RX_FILTER_CTRL_AAM  | \
                                      HAL_RX_FILTER_CTRL_AAU  | \
                                      HAL_RX_FILTER_CTRL_AAB  | \
                                      HAL_RX_FILTER_CTRL_APM    )



#define COMMAND_RXENABLE    0x00000004
#define COMMAND_TXENABLE    0x00000001

/* Bit definition for Receive Filter/Match control register */
#define MIB_CTRL_MIBS           0x00000008
#define MIB_CTRL_ACLR           0x00000004
#define MIB_CTRL_FRZ            0x00000002
#define MIB_CTRL_WRN            0x00000001

#define INT_TXRCMP              0x02000000
#define INT_RXRCMP              0x01000000
#define INT_DPERR               0x00800000
#define INT_SSERR               0x00400000
#define INT_RMABT               0x00200000
#define INT_RTABT               0x00100000
#define INT_RXSOVR              0x00010000
#define INT_HIBERR              0x00008000
#define INT_PHYINT              0x00004000
#define INT_PME                 0x00002000
#define INT_SWI                 0x00001000
#define INT_MIB                 0x00000800
#define INT_TXURN               0x00000400
#define INT_TXIDLE              0x00000200
#define INT_TXERR               0x00000100
#define INT_TXDESC              0x00000080
#define INT_TXOK                0x00000040
#define INT_RXORN               0x00000020
#define INT_RXIDLE              0x00000010
#define INT_RXEARLY             0x00000008
#define INT_RXERR               0x00000004
#define INT_RXDESC              0x00000002
#define INT_RXOK                0x00000001

#define INT_PCI_ERRORS          (INT_DPERR | INT_SSERR |\
                                     INT_RMABT | INT_RTABT)

#define INT_TXINTR              (INT_TXURN | INT_TXIDLE |\
                                     INT_TXERR | INT_TXOK)

#define INT_RXINTR              (INT_RXORN   | INT_RXIDLE |\
                                     INT_RXEARLY | INT_RXERR  |\
                                     INT_RXDESC  | INT_RXOK)



#define		CLDA0				0x01	// Current Local DMA Address 0
#define		CLDA1				0x02	// Current Local DMA Address 1
#define		TSR		        	0x04    // Transmit Status Register
#define		NCR             	0x05    // Number of Collisions Register
#define		FIFO            	0x06    // FIFO
#define		CRDA0           	0x08    // Current Remote DMA Address 0
#define		CRDA1           	0x09    // Current Remote DMA Address 1
#define		RSR			     	0x0C    // Receive Status Register
#define		CNTR0		       	0x0D    // Tally Counter 0 (Frame Alignment)
#define		CNTR1		      	0x0E    // Tally Counter 1 (CRC Errors)
#define		CNTR2			  	0x0F    // Tally Counter 2 (Missed Packet)

// Page 1
#define		PAR0				0x01	// Physical Address Register 0
#define		PAR1				0x02	// Physical Address Register 1
#define		PAR2				0x03	// Physical Address Register 2
#define		PAR3				0x04	// Physical Address Register 3
#define		PAR4				0x05	// Physical Address Register 4
#define		PAR5				0x06	// Physical Address Register 5
#define		CURR         		0x07    // Current Page Register
#define		MAR0				0x08	// Multicast Address Register 0
#define		MAR1				0x09	// Multicast Address Register 1
#define		MAR2				0x0A	// Multicast Address Register 2
#define		MAR3				0x0B	// Multicast Address Register 3
#define		MAR4				0x0C	// Multicast Address Register 4
#define		MAR5				0x0D	// Multicast Address Register 5
#define		MAR6				0x0E	// Multicast Address Register 6
#define		MAR7				0x0F	// Multicast Address Register 7

// Page 2 - Diagnostic
#define		RNPP				0x03	// Remote Next Packet Pointer
#define		LNPP				0x05	// Local Next Packet Pointer
#define		UAC					0x06	// Upper Address Counter
#define		LAC					0x07	// Lower Address Counter

// Miscellaneous
#define		XFERPORT			0x10
//#define	RESET				0x1E  
#define 	RESET       		0x00000130

// Bit definitions for Command Register.		
#define		CR_STP				0x01	// Stop	
#define		CR_STA				0x02	// Start
#define		CR_TXP				0x04	// Transmit Packet
#define		CR_RD0				0x08	// Remote DMA Read
#define		CR_RD1				0x10	// Remote DMA Write
#define		CR_SEND				0x18	// Send Packet
#define		CR_RD2				0x20	// Abort/Complete Remote DMA
#define		CR_PS0				0x40	// Register Page 1
#define		CR_PS1				0x80	// Register Page 2

// remote DMA operations
#define		CR_READ				CR_RD0+CR_STA				// 0x0A
#define		CR_WRITE			CR_RD1+CR_STA				// 0x12
#define		CR_SENDPACKET		CR_SEND+CR_STA				// 0x1A

#define		CR_ABORT			CR_STP+CR_RD2				// 0x21
#define		CR_TRANSMIT			CR_STA+CR_RD2+CR_TXP		// 0x26
#define		CR_PAGE0			CR_STA+CR_RD2				// 0x22
#define		CR_P1STOP			CR_STP+CR_RD2+CR_PS0		// 0x61
#define		CR_PAGE1			CR_STA+CR_RD2+CR_PS0		// 0x62
#define		CR_P2STOP			CR_STP+CR_RD2+CR_PS1		// 0xA1
#define		CR_PAGE2			CR_STA+CR_RD2+CR_PS1		// 0xA2

// Bit definitions for Interrupt Status Register
#define 	ISR_PRX				0x01	// Packet Received
#define 	ISR_PTX				0x02	// Packet Transmitted
#define 	ISR_RXE				0x04	// Receive Error
#define 	ISR_TXE				0x08	// Transmit Error
#define 	ISR_OVW				0x10	// Overwrite Warning
#define 	ISR_CNT				0x20	// Counter Overflow
#define 	ISR_RDC				0x40	// Remote DMA Complete

#define		ISR_RST				0x80	// Reset Status
#define		ISR_CLEAR			0xFF	// Clear ISR

// Bit definitions for Interrupt Mask Register
#define		IMR_PRXE			0x01	// Packet Received Int Enable
#define 	IMR_PTXE			0x02	// Packet Transmitted Int Enable
#define 	IMR_RXEE			0x04	// Receive Error Int Enable
#define 	IMR_TXEE			0x08	// Transmit Error Int Enable
#define 	IMR_OVWE			0x10	// Overwrite Warning Int Enable
#define 	IMR_CNTE			0x20	// Counter Overflow Int Enable
#define 	IMR_RDCE			0x40	// Remote DMA Complete Int Enable

#define		IMR_DEFAULT			0x15	// Defalut for IMR  

// Bit definitions for Data Configuration Register
#define		DCR_WTS				0x01	// Word Transfer Select
#define		DCR_BOS				0x02	// Byte Order Select
#define		DCR_LAS				0x04	// Long Address Select
#define		DCR_NORM			0x08	// Loopback Select
#define		DCR_ARM				0x10	// Auto-Initialize Select
#define		DCR_FT0				0x20	// FIFO Threshold Select 0
#define		DCR_FT1				0x40	// FIFO Threshold Select 1

#define		DCR_LOOPBYTE		0x40	// Loopback
#define		DCR_LOOPWORD		0x41
#define		DCR_FT1BYTE			0x48	// 8 Byte FIFO Thresh.
#define		DCR_FT1WORD			0x49	// 4 Word FIFO Thresh.

// Bit definitions for Transmit Configuration Register
#define 	TCR_CRC				0x01	// Inhibit CRC
#define		TCR_LB0				0x02	// Encoded Loopback Control 0
#define		TCR_LB1				0x04	// Encoded Loopback Control 1
#define 	TCR_ATD				0x08    // Auto Transmit Disable
#define 	TCR_OFST			0x10	// Collision Offset Enable

#define		TCR_CLEAR			0x00	// Clear TCR 

// Bit Definitions for Transmit Status Register
#define		TSR_PTX				0x01	// Packet Transmitted
#define		TSR_COL				0x04	// Transmit Collided
#define		TSR_ABT				0x08	// Transmit Aborted
#define		TSR_CRS				0x10	// Carrier Sense Lost
#define		TSR_FU				0x20	// FIFO Underrun
#define		TSR_CDH				0x40	// CD Heartbeat
#define		TSR_OWC				0x80	// Out of Window Collision

// Bit definitions for Receive Configuration Register
#define		RCR_SEP				0x01    // Save Errored Packets
#define		RCR_AR				0x02    // Accept Runt Packets
#define		RCR_AB				0x04    // Accept Broadcast Packets
#define		RCR_AM				0x08	// Accept Multicast Packets
#define		RCR_PRO				0x10    // Promiscuous Physical
#define		RCR_MON				0x20	// Monitor Mode 

#define		RCR_DEFAULT			0x1F	// Set all but Monitor Mode

// Bit definitions fro Receive Status Register
#define		RSR_PRX				0x01	// Packet received intact
#define		RSR_CRC				0x01	// CRC Error
#define		RSR_FAE				0x01	// Frame Alignment Error
#define		RSR_FO				0x01	// FIFO Overrun
#define		RSR_MPA				0x01	// Missed Packet
#define		RSR_PHY				0x01	// Physical/Multicast Address
#define		RSR_DIS				0x01	// Receiver Disabled
#define		RSR_DFR				0x01	// Deferring

// Send Packet definitions
#define		SENDPKTHI			0x0F	// Definitions for Send Packet Command
#define		SENDPKTLO			0x00

// Definitions for the adapter operating mode
#define 	PIO				0
#define 	SHARED			1

// Definitions for Transfer Mode
#define		TRANSMIT		0
#define		RECEIVE			1
#define		LOOPBACK		2
#define		PINGSIZE		64

// Definitions for NIC datawidth mode

#define 	WORDWIDE		0
#define 	BYTEWIDE		1

// Definition for NIC transmit page

#define		TRANSMITPAGE	0x40
#define		TRANSMITADDR	0x4000


// Definition for offset of Ethernet address in NIC memory space 
// as well as length

#define		ADDRESSHIGH		0x00
#define		ADDRESSLOW		0x00
#define		ADDRESSLENGTH	0x0C

#define PMESTS				0x00008000L
#define PMEEN				0x00000100L

/* 
 * The following structure is defined as per the descriptor structure
 * required for the MacPhyter3v controller.
 */
#define MAX_ETHER_PACKET_SIZE 1520

typedef unsigned long UINT32;

typedef BYTE etherPacket_t[ MAX_ETHER_PACKET_SIZE ];

/* 
 * The following structure defines the command/Status definitions for the
 * transmit and receive descriptors, as defined by MacPhyter3V.
 */
typedef union CommandStatus
{
    UINT32 cmdStatus;
    struct rxStatus 
    {
        UINT32  descriptorSize:12;        /* Modified by MacPhyter3V         */
        UINT32  reserved:4;
        UINT32  Collision:1;              /* Modified by MacPhyter3V         */
        UINT32  LoopBackPacket:1;         /* Modified by MacPhyter3V         */
        UINT32  FrameAlignmentError:1;    /* Modified by MacPhyter3V         */
        UINT32  CRCError:1;               /* Modified by MacPhyter3V         */
        UINT32  InvalidSymbolError:1;     /* Modified by MacPhyter3V         */
        UINT32  RuntPacketReceived:1;     /* Modified by MacPhyter3V         */
        UINT32  TooLongPacketReceived:1;  /* Modified by MacPhyter3V         */
        UINT32  DestinationClass:2;       /* Modified by MacPhyter3V         */
        UINT32  ReceiveOverrun:1;         /* Modified by MacPhyter3V         */
        UINT32  ReceiveAborted:1;         /* Modified by MacPhyter3V         */
        UINT32  ReceivedOkay:1;           /* Modified by MacPhyter3V         */
        UINT32  ReceiveIncludeCRC:1;      /* Modified by MacPhyter3V         */
        UINT32  ReceiveIntr:1;            /* Modified by HAL, if interrupts are 
                                          * required after receiving this
                                          * descriptor */
        UINT32  MoreDescriptors:1;        /* Modified by MacPhyter3V         */
        UINT32  ReceiveOwn:1;             /* Modified by MacPhyter3V/HAL     */
    } rxStatus;
    struct txStatus 
    {
        UINT32  descriptorSize:12;        /* Modified by HAL                 */
        UINT32  reserved:4;
        UINT32  CollisionCount:4;         /* Modified by MacPhyter3v         */
        UINT32  ExcessiveCollisions:1;    /* Modified by MacPhyter3v         */
        UINT32  OutofWindowCollision:1;   /* Modified by MacPhyter3v         */
        UINT32  ExcessiveDeferral:1;      /* Modified by MacPhyter3v         */
        UINT32  TransmitDeferred:1;       /* Modified by MacPhyter3v         */
        UINT32  CarrierSenseLost:1;       /* Modified by MacPhyter3v         */
        UINT32  TransmitUnderrun:1;       /* Modified by MacPhyter3v         */
        UINT32  TransmitAborted:1;        /* Modified by MacPhyter3v         */
        UINT32  TransmitOkay:1;           /* Modified by MacPhyter3v         */
        UINT32  TransmitCRC:1;            /* Modified by HAL                 */
        UINT32  TransmitIntr:1;           /* Modified by HAL                 */
        UINT32  MoreDescriptors:1;        /* Modified by HAL                 */
        UINT32  TransmitOwn:1;            /* Modified by HAL/MacPhyter3V     */
    } txStatus;
} TxRxCmdStatus_t, *pTxRxCmdStatus;

typedef struct HALSpace
{
    UINT32          linkPhyAddrDesc; /* Physical Address of next descriptor */
    TxRxCmdStatus_t cmdStatus;    /* Command/Status bit definition       */
    UINT32          physPointer;  /* Physical Address of the buffer      */
} HALSpace_t, *pHALSpace_t;

/*
 * The following structure is used to define a single physically contiguous 
 * memory fragment.  This structure is known and used both by the HAL and by
 * the OSM. 
 */
typedef struct HALFragment
{
    struct HALFragment   *next;     /* Used for Linked List handling    */
    UINT32      bufLogicalAddress;  /* Logical Address of the buffer    */
    UINT32      bufPhysicalAddress; /* Physical Address of the buffer   */
    UINT32      bufTotalLen;        /* Total Length of the buffer in bytes */
    UINT32      bufValidLen;        /* Valid Length of the buffer in bytes */
    UINT32      phyAddrHALSpace;    /* Physical Address of HALSpace     */
    UINT32      uniqueHandle;       /* UniqueHandle for OSM use         */
    HALSpace_t  HALSpace;           /* Private space for use by HAL     */
} HALFragment_t, *pHALFragment_t;

typedef struct RegisterTable
{
    BYTE        *regName;     /* Used for Linked List handling    */
    WORD        regOffset;
} RegisterTable_t, *pRegisterTable_t;

typedef unsigned char MACAddr_t[ 6 ];
typedef MACAddr_t *pMACAddr_t;

#define RX_DESCRIPTOR_POOL_SIZE 20
typedef struct TxRxDescriptorPool
{
	HALSpace_t TxDescriptor;
	HALSpace_t RxDescriptor[ RX_DESCRIPTOR_POOL_SIZE ];
	etherPacket_t RxData[ RX_DESCRIPTOR_POOL_SIZE ];
} TxRxDescriptorPool_t, *pTxRxDescriptorPool_t;
