/***********************************************************************
*									
* NICSTUFF.C								
*									
* This is a collection of routines that are intended to make it easy	
* to communicate with any piece of hardware that is based on a
* National Semiconductor Ethernet chip with the MacPhyter core. 
*
* This includes all routines necessary to setup the card, send	
* packets, receive packets, perform loopback test and
* others.
*
***********************************************************************/
// Include files

#include <dos.h>
#include <stdio.h>
#include <conio.h>
#include <memory.h>
#include <ctype.h>
#include "GLOBAL.H"
#include "NICSTUFF.H"
#include "PCIBIOS.H"
#include "FUNCTION.H"

// External Function Prototypes

// Global Variables
extern unsigned short RegBase;
extern int currentBus;
extern int currentDev;
extern int currentFunc;
//extern unsigned short IRQ;
extern unsigned long PciCfgSpace[ PCI_TABLE_SIZE ]; 
extern BYTE  PacketSize;
extern char  cQuit, cTransmit, cLoopback, cReturn, cReceive, cMaster;

// Declare the Tx and Rx descriptor as members of data structure and use /Zp4
// compiler option to align both descriptors on 32-bit boundaries
TxRxDescriptorPool_t descriptorPool;
int RxDescriptorIndex = 0;          
              
unsigned long siliconRev;
long PktsTransmitted = 0;
long PktsRcvd = 0;

BYTE Destination[]={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
BYTE PingPacket[128];
BYTE Header[4];

#define REGISTER_TABLE_SIZE 39
RegisterTable_t RegisterTable[ REGISTER_TABLE_SIZE ] =
        {
        "CR     ", 0,    "CFG    ", 4,    "MEAR   ", 8,    "PTSCR  ", 0xC,
        "ISR    ", 0x10, "IMR    ", 0x14, "IER    ", 0x18, "TxDP   ", 0x20,
        "TxCFG  ", 0x24, "RxDP   ", 0x30, "RxCFG  ", 0x34, "WCSR   ", 0x40,
        "PCR    ", 0x44, "RFCR   ", 0x48, "RFDR   ", 0x4C, "BRAR   ", 0x50,
        "BRDR   ", 0x54, "SRR    ", 0x58, "MIBC   ", 0x5C, "RxErPkt", 0x60,
        "RxFcsEr", 0x64, "RxMsdEr", 0x68, "RxFaErr", 0x6C, "RxSymEr", 0x70,
        "RxFrLng", 0x74, "TxSqeEr", 0x78,

        "BMCR   ", 0x80, "BMSR   ", 0x84, "PHYIDR1", 0x88, "PHYIDR2", 0x8C,
        "ANAR   ", 0x90, "ANLPAR ", 0x94, "ANER   ", 0x98, "ANNPTR ", 0x9C,
        "PHYSTS ", 0xC0, "FCSCR  ", 0xD0, "RECR   ", 0xD4, "PHYCR  ", 0xE4,
        "10BTSCR", 0xE8 
        };

RegisterTable_t PciTable[ PCI_TABLE_SIZE ] =
        {
        "CFGID  ", 0x00, "CFGCS  ", 0x04, "CFGRID ", 0x08, "CFGLAT ", 0x0C,
        "CFGIOA ", 0x10, "CFGMA  ", 0x14, "CFGSID ", 0x2C, "CFGROM ", 0x30,
        "CAPPTR ", 0x34, "CFGINT ", 0x3C, "PMCAP  ", 0x40, "PMCS   ", 0x44
        };

BYTE Packet[ MAX_ETHER_PACKET_SIZE ];

MACAddr_t MACAddr = { '0', '0', '0', '0', '0', '0' };

//
// Phy Configuration data (base CFG Register contents) for user selectable
// Media Type/Speed.
//
unsigned long phyConfigData[6] = { 
		0x0005e000,		// Auto
		0x00044000,		// 100Half
		0x00040000,		// 10Half
		0x0005a000,		// 100Full
		0x00058000,		// 10Full
		0x00052000		// Auto10
		};
unsigned long phyConfig;

#define PINT_ACEN 0x00020000
/*
unsigned long anarCfg[ 4 ] = { 0x0060,
							   0x00a0,
							   0x0180,
							   0x01e0 };
unsigned long autoNeg[ 16 ] = { 0x00,
								0x03,
								0x07, 0x07,
								0x01, 0x01, 0x01, 0x01,
								0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05, 0x05 };
unsigned long forceCfg[ 4 ] = { 0x0000,
								0x2000,
								0x0100,
								0x2100 };*/

/************************************************
*       Function:   findNIC
*       Purpose:    Find an instance of MacPhyter in the system.
*       Returns:    1 if FAIL, 0 if Succees
*************************************************/
int findNIC(bool bShowInfo)
{
	char inputString[ 25 ], ctemp1;
	UINT16 nicAddress[ 3 ];
	unsigned vendorId = 0x100B, value; 
	unsigned long dvalue;
	unsigned deviceId = 0x20;
	int i;

	if( RegBase == 0)
	{
		// Find the card on the PCI Bus:
		if( PciBiosFindDevice( vendorId, deviceId, 0, &currentBus, &currentDev, &currentFunc ) )
		{
			printf( "\nMacPhyter Demo Card Not Found on PCI Bus!" );
			return( 1 );
		}

	    PciBiosReadConfigWord( currentBus, currentDev, currentFunc, PciCfgBaseAddr0, &RegBase );

	    if( RegBase & 0x01 )
		{
			RegBase = RegBase & ~0x03;
		}
	}

    for( i=0x0; i<PCI_TABLE_SIZE; i++ )
	{
		PciCfgSpace[ i ] = 0;
		PciBiosReadConfigWord( currentBus, currentDev, currentFunc, PciTable[ i ].regOffset, &(WORD)PciCfgSpace[ i ]  );
		PciBiosReadConfigWord( currentBus, currentDev, currentFunc, PciTable[ i ].regOffset + 2, &value );
		dvalue = (UINT32)value;
		dvalue <<= 16;
		PciCfgSpace[ i ] |= dvalue;
	}

    // PciBiosReadConfigWord( currentBus, currentDev, currentFunc, 0x3C, &IRQ );
    // IRQ &= 0xFF;
    // if( !IRQ ) printf( "\n !!WARNING: NO IRQ GRANTED!\n" );
    
    if (bShowInfo)
	{
		printf( "\nMacPhyter PCI Adapter Found at I/O Base: %04X\n", RegBase );
		ReadMACAddress(  nicAddress );    
		printf( "\tMAC Address: " );
		for( i=0; i<3; i++ )
		{
			MACAddr[ i*2+1 ] = ( unsigned char )( nicAddress[ i ] >> 8 );
			MACAddr[ i*2 ] = ( unsigned char )( nicAddress[ i ] );
			printf( "%02X %02X ", MACAddr[ i*2 ], MACAddr[ i*2+1 ] );
		}
		printf( "\n\n" );
	}
    
    // WARN for Odd Settings:
    PciBiosReadConfigWord( currentBus, currentDev, currentFunc, 0x4, &value );
    if ((value & 7) < 7)
	{
		if( (value & 1) == 0 )
		{
			printf( "\n !!WARNING: I/O SPACE NOT GRANTED!" );
		}
		if( (value & 2) == 0 )
		{
			printf( "\n !!WARNING: MEMORY SPACE NOT GRANTED!" );
		}
		if( (value & 4) == 0 )
		{
			printf( "\n !!WARNING: BUS MASTERING NOT GRANTED!\n ENABLING PCI BUS MASTERING NOW..." );
			value |= 4;
			PciBiosWriteConfigWord( currentBus, currentDev, currentFunc, 0x4, value );			
		}
		printf( "\n\n" );
	}

    PciBiosReadConfigWord( currentBus, currentDev, currentFunc, 0x46, &value );
    if (value & 1) 
	{
		printf( "\n WARNING: PWR MGT ENABLED!\n\n" );

		// Read the EEPROM contents to see if it's enabled by default:
		ReadEEWord( RegBase + MEAR, 0x03, &value );
		if( value & 0x1000 )
		{
			printf( " WARNING: PWR MGT ENABLE BIT EEPROM DEFAULT!" );
			printf( "\n\n Would you like to disable default PwrMgt Enable Bit? (Y/N)" );

			// Prompt user to change default bit in EEPROM:
			scanf( "%s", &inputString );
			ctemp1 = toupper( inputString[ 0 ] );

			if (ctemp1 == 'Y')
			{
				// Since value contains EEPROM[3]
				value &= 0xEFFF;

				writeEeEnable();
				WriteEEWord( RegBase + MEAR, 0x03, value );
				writeEeDisable();

				// Now Recalculate the CheckSum and write the new value in:
				if (doEeCheckSum())
				{
					return 1;
				}
			}

			printf("\n");
		}
	}
    
    return 0;
}

/************************************************
*	Function:   DisplayPciRegs
*	Purpose:	dump PCI Cfg registers
*	Returns:    void
*************************************************/
void DisplayPciRegs()
{
	int i = 0;

	for (i = 0; i < PCI_TABLE_SIZE; i++)
	{
		printf
		( 
		"\n%02X:%s= %08lX  ",
		PciTable[ i ].regOffset,
		PciTable[ i ].regName,
		PciCfgSpace[ i ] 
		);

		i++;

		if (i < PCI_TABLE_SIZE)
		{
			printf
			(
			"%02X:%s= %08lX  ",
			PciTable[ i ].regOffset,
			PciTable[ i ].regName,
			PciCfgSpace[ i ] 
			);
		}

		i++;

		if (i < PCI_TABLE_SIZE)
		{
			printf
			(
			"%02X:%s= %08lX  ",
			PciTable[ i ].regOffset,
			PciTable[ i ].regName,
			PciCfgSpace[ i ] 
			);
		}
	}
        	
	printf( "\n\n" );
}

/************************************************
*       Function:       DisplayNICRegs
* 	Purpose:	Displays the contents of the ST-NIC registers
*	Input:		none
*   _output:		none
*************************************************/
void DisplayNICRegs()
{
	int i = 0;

	for (i = 0; i < REGISTER_TABLE_SIZE; i++)
	{
		printf
		(
		"\n%s: %08lX",
		RegisterTable[ i ].regName,
		inpd( RegBase + RegisterTable[ i ].regOffset )
		);

		i++;

		if (i < REGISTER_TABLE_SIZE)
		{
			printf
			(
			"   %s: %08lX",
			RegisterTable[ i ].regName,
			inpd( RegBase + RegisterTable[ i ].regOffset )
			);
		}

		i++;

		if (i < REGISTER_TABLE_SIZE)
		{
			printf
			(
			"   %s: %08lX",
			RegisterTable[ i ].regName,
			inpd( RegBase + RegisterTable[ i ].regOffset )
			);
		}
	}

	printf( "\n\n" );
}
	
/************************************************
*	Function:   resetNIC
*	Purpose:	Performs a hardware reset of the NIC.
*	Arguments:  Port - the RESET port
*	Returns:    1 if FAIL, 0 if Succees
*************************************************/
#define mibACLR         0x00000004      /* clear all counters */

int resetNIC( unsigned long value )
{
	int i;
    unsigned long dword;
    
    outpd( RegBase + INT_ENABLE, 0 );
    
    outpd( RegBase + INT_MASK, 0 );
    
    outpd( RegBase + RX_FILTER_CTRL, 0 );
    
    outpd( RegBase + MIB_CTRL, mibACLR  );
    
    outpd( RegBase, value );

    i = 0;
    do
	{
		msleep(10);
		dword = inpd( RegBase + CR );

    } while ( ( dword & RESET ) && i++ < 10 );
    
	outpd( RegBase + CFG, phyConfig | PINT_ACEN );

	if (i == 10)
	{
		return 1;
	}

	msleep(100);

	return 0;
}

/************************************************
*	Function:   initNIC
*	Purpose:	Performs preliminary initialization
*				of NIC chip.  Performs hardware reset
*				and then sets up registers, based
*				on given arguments
*	Arguments:  UseSend - set if program to use Send Packet
*				RCRMask - sets packet types to receive
*	Returns:    1 if FAIL, 0 if Succees
*************************************************/
int initNIC()
{
    unsigned long phyStatus;
    unsigned long dword;
	int i;

	// Init the number of packets transmitted:
	PktsTransmitted = 0;
	PktsRcvd = 0;

	RxDescriptorIndex = 0;

	siliconRev = inpd( RegBase + SRR );

	// Reset the NIC
	if (resetNIC( RESET ))
	{
		return 1;
	}

	outpd( RegBase + PCITEST_CNTRL, EELOAD_EN );     // Download eeprom defaults
	//delay( 1 );
	msleep(500);

	// Reset PHY and adjust PHY registers
	phyConfig = phyConfigData[0]; // Auto-negotiation enabled
	EuphyterPhyReset();
    EuphyterPhyAdjust();

	
    //outpd( RegBase + PHY_BMCR, BMCR_RESET );    // Reset the PHY
	//delay( 1 );
	//msleep(500);

	CorrectPhyCoefficient();
    
    outpd(RegBase + CCSR, PMESTS);
    outpd(RegBase + WCSR, 0x0000);

	// Config the transmit register.    
	phyStatus = inpd(RegBase + PHY_PHYSTS);
    dword = DMA32B  | TXFIL64B  | TXDRN1536B;
    dword |= 0x00040000; // Enable the original backoff algorithm.
	if ((phyStatus & 0x00000005) == 0x00000005)
	{
		dword |= TxCSI | TxHBI | TxATP;
	}
	else
	{
		dword |= TxATP;
	}
	// for diag program, this bit must be enabled, although windows drivers not
	dword |= TxECRETRY;
	outpd(RegBase + TX_CONFIG, dword);	
	//delay( 1 );
	msleep(100);

	// Config the receive register.    
	dword = DMA64B | RXDRN32B;
	if ((phyStatus & 0x00000005) == 0x00000005)
	{
		dword |= RxATP;
	}
	outpd( RegBase + RX_CONFIG, dword);
	//delay( 1 );
	msleep(100);

	// Receive Filter:
    outpd( RegBase + RX_FILTER_CTRL, RX_FILTER_CTRL_RFEN |\
                           RX_FILTER_CTRL_AAB  |\
                           RX_FILTER_CTRL_AAM  |\
                           RX_FILTER_CTRL_APM );

    for( i=0; i<RX_DESCRIPTOR_POOL_SIZE; i++ )
	{
		/* Clear the ScratchSpace before starting to use it */
		memset( &descriptorPool.RxDescriptor[ i ], '\0', sizeof( HALSpace_t ) );

		//Physical dataBuffer Pointer:
		descriptorPool.RxDescriptor[ i ].physPointer = 
    		DosLogicalToPhysical( (void *)descriptorPool.RxData[ i ] );

		//Physical nextDescriptor Pointer:
		if( i+1 < RX_DESCRIPTOR_POOL_SIZE )
		{
    		descriptorPool.RxDescriptor[ i ].linkPhyAddrDesc = 
    			DosLogicalToPhysical( &descriptorPool.RxDescriptor[ i+1 ] );
		}
		else
		{
    		descriptorPool.RxDescriptor[ i ].linkPhyAddrDesc = 
    			DosLogicalToPhysical( &descriptorPool.RxDescriptor[ 0 ] );
		}

		descriptorPool.RxDescriptor[ i ].cmdStatus.rxStatus.descriptorSize = sizeof( etherPacket_t );
    }
	
	// Clear the MIB counters:
    outpd( RegBase + MIB_CTRL, MIB_CTRL_ACLR );

	// Set the Interrupt Mask register:
    outpd( RegBase + INT_MASK,  INT_DPERR |
                		INT_SSERR         |
                		INT_RMABT         |
                		INT_RTABT         |
                		INT_RXSOVR        |
                		INT_TXERR         |
                		INT_TXOK          |
                		INT_RXERR         |
                		INT_RXOK          |
                		INT_RXORN );


    // Write RX DP:
    outpd( RegBase + RX_DP, DosLogicalToPhysical( &descriptorPool.RxDescriptor[ 0 ] ) );

    // Enable Rcv:
    outpd( RegBase, COMMAND_RXENABLE );

	return 0;
}


/************************************************
*	Function:NICLoopback
*	Purpose:	Performs loopback testing on the NIC
*				controller.  The mode is passed to 
*				determine what type of loopback is to 
*				be performed:
*					1:	Internal MAC loopback
*					2:	ENDEC loopback
*					3:	External Transceiver loopback
*	Arguments:  Mode - the mode of loopback to perform
*	Returns:    1 if FAIL, 0 if Succees
*************************************************/
int NICLoopback( int Mode )
{
	int y = 0;
    unsigned long tempReg = 0;

    if (initNIC())
	{
		return 1;
	}

	// If in the MAC loopback mode:
	if (Mode == 1)
	{
		// To Enable loopback: 1st disable Xmit/Rcv:
        outpd( RegBase + CR, 0x08);
        //delay( 1 );
		msleep(500);
        outpd( RegBase + CR, 0x02);
        //delay( 1 );
		msleep(500);
	
		// Next: set the MLB bit in the TxCFG register:
		tempReg = inpd( RegBase + TX_CONFIG );
		tempReg |= 0x20000000;
		outpd( RegBase + TX_CONFIG, tempReg );
	}
	    
    // If in the PHY loopback mode:
    if (Mode == 2)
	{
		// To Enable loopback: 1st disable Xmit/Rcv:
        outpd( RegBase + CR, 0x08);
        //delay( 1 );
		msleep(500);
        outpd( RegBase + CR, 0x02);
        //delay( 1 );
		msleep(500);

        // Next: set the Loopback bit in the PHY_BMCR register:
        tempReg = inpd( RegBase + PHY_BMCR );
        tempReg |= 0x04000;
        outpd( RegBase + PHY_BMCR, tempReg );
        //delay( 1 );
		msleep(800);

        // Next set the ATX bit in RxCFG
        tempReg = inpd( RegBase + RX_CONFIG );
        tempReg |= 0x10000000;
        outpd( RegBase + RX_CONFIG, tempReg );
	}

    // If in the External loopback mode:
    if (Mode == 3)
	{
		// To Enable loopback: 1st disable Xmit/Rcv:
		outpd( RegBase + CR, 0x08);
		//delay( 1 );
		msleep(500);
		outpd( RegBase + CR, 0x02);
		//delay( 1 );
		msleep(500);

		// Next set the ATX bit in RxCFG, so we can receive our data.
		tempReg = inpd( RegBase + RX_CONFIG );
		tempReg |= 0x10000000;
		outpd( RegBase + RX_CONFIG, tempReg );
    }

    // rewrite the Rx descriptor:
	outpd( RegBase + RX_DP, DosLogicalToPhysical( &descriptorPool.RxDescriptor[ 0 ] ) );

    // Enable Rcv:
    outpd( RegBase, COMMAND_RXENABLE );

	msleep(100);

	if (!sendPing())
	{
		printf("Loopback--Send Ping failed\n");
		return 1;
	}
	else if (!receivePing())
	{
		printf("Loopback--Receive Ping Failed\n");
		return 1;
	}

	for (y = 0; y < 60; y++)
	{
		if (PingPacket[ y ] != Packet[ y ])
		{
			printf("\nLoopback failed!--unequal ping packet size failed\n");
			return 1;
		}
	}

	printf("\nLoopback Successful!\n");
	
	return 0;
}

/************************************************
*	Function:   createPacket
*	Purpose:	Creates a packet to be used to test
*				the loopback features of the NIC.
*				If operating in Word-wide mode, every 
*				other byte is loaded.
*	Arguments:  none
*	Returns:    nothing
*************************************************/
void createPacket(void)
{
	int x;

	// Setup Ping Packet	
	for (x = 0; x < 6; x++)
	{
		if (cReturn && !cLoopback)
		{
			PingPacket[x] = Destination[x];
		}
		else
		{
			PingPacket[x] = 0xFF;
		}
	}
	for (x = 0; x < 6; x++)
	{
		PingPacket[x+6] = MACAddr[x];
	}
	PingPacket[12] = 0x0F;
	PingPacket[13] = 0xF0;
	for (x = 1; x < 24; x++)
	{
		PingPacket[x+13] = x;
		PingPacket[60-x] = x;
	}
	if (cLoopback)
	{
		for(x = 37; x < PacketSize; x++)
		{
			PingPacket[x] = x-13;
		}
	}
}

/************************************************
*	Function:   getHeader
*	Purpose:	Reads the Header from the received
*				packet.
*	Arguments:  Address - Start of packet in receive buffer
*	Returns:    nothing
*************************************************/
WORD getHeader(etherPacket_t * Address)
{   
	int x;
	WORD Length;

    //NICtoPC(Header,4,Address);
    
    // Determine length of received packet    
	Length = Header[3];    
    Length << 8;
    Length |= Header[2];
	printf("...a total of %d data bytes received.\n",Length);
    
    printf("\nHeader:        ");
	for (x = 0; x < 4; x++)
	{
		printf("%02X ",Header[x]);
	}

    return (Length);
}


/************************************************
*	Function:   displayPacket
*	Purpose:	Displays the received or transmitted
*				packet
*	Arguments:  none
*	Returns:    The value of the last byte in the packet
*************************************************/
BYTE displayPacket( unsigned long Length, int Transfer)
{   
	int x;
	BYTE temp;
			
	if (Transfer == TRANSMIT)
	{
		for(x=0; x<(int)Length; x++)
		{
			Packet[x] = PingPacket[x];
		}
	}
		
	printf("\nSource:        ");
	for (x = 0; x < 6; x++)
	{
		printf("%02X ",Packet[x+6]);
	}	    
	
	printf("      Destination:   ");
	for (x = 0; x < 6; x++)
	{
		printf("%02X ",Packet[x]);
	}
	
	printf("\nLength/Type:   ");
	for (x = 0; x < 2; x++)
	{
		printf("%02X ",Packet[x+12]);
	}
	
	if (Transfer == RECEIVE)
	{
		printf("\n\nReceived Packet:\n");
	}
	else
	{
		printf("\n\nTransmitted Packet:\n");
	}

	for (x = 14; x < (int)(Length-4); x++)
	{
    	printf("%02X   ",Packet[x]);	
    	temp = Packet[x];
    }

	if ( Transfer == RECEIVE )
	{
		printf("\n\nReceived %ld packets.\n", ++PktsRcvd );
	}
	else
	{
		printf("\n\nTransmitted %ld packets.\n", ++PktsTransmitted );
	}

	return(temp);
}	

int sendMgkPkt( BYTE *destAddr )
{
	int x, y;

        // Setup Magic Packet header:
	for (x = 0; x < 6; x++)
	{
		PingPacket[x] = (BYTE)destAddr[x];
	}
	
	for (x = 0; x < 6; x++)
	{
		PingPacket[x+6] = MACAddr[x];
	}

	PingPacket[12] = 0x0F;
	PingPacket[13] = 0xF0;

	for (y = 0; y < 6; y++)
	{
		PingPacket[14+y] = 0xFF;
	}

	// data for packet consists of 16 destAddr:
	for (y = 0; y <= 15; y++)
	{
		for (x = 0; x < 6; x++)
		{
			PingPacket[(6*y)+20+x] = (BYTE)destAddr[x];
		}
	}

    if (MacSend( PingPacket, 120 ))
	{
		printf( "\nTransmit Aborted. " );
		return 1;
	}

	// Now display a copy of the Magic Packet Sent:
    displayPacket( 127, TRANSMIT );

	return 0;
}


/************************************************
*	Function:   sendPing
*	Purpose:	Creates a packet to be used for 
*				the Ping test.  Once the packet
*				is assembled, it is transmitted.
*				If operating in Word-wide mode, every 
*				other byte is loaded.
*	Arguments:  none
*	Returns:    value of last data byte transmitted.
*************************************************/
BYTE sendPing(void)
{
	int x;
    
	createPacket();							// Setup Ping Packet

	if (cReturn)
	{
		for (x=12; x<60; x++)
		{
			PingPacket[x]=Packet[x];
		}
	}
	
	if (cTransmit)
	{
		PingPacket[59] = 0xFF;
	}

	if (cQuit)
	{
		PingPacket[59] = 0x07;
	}

	if (MacSend( PingPacket, PINGSIZE-4 ))
	{
		printf( "\nTransmit Aborted. " );
		return 0;
	}

	return displayPacket(PINGSIZE, TRANSMIT);
}

/************************************************
*	Function:   receivePing
*	Purpose:	Waits for receive interrupt to be
*				set, and then processes the incoming
*				packet.  During the first iteration,
*				the destination address is setup.
*	Arguments:  none
*	Returns:    value of last data byte received
*************************************************/
BYTE receivePing(void)
{	
	int count;
	unsigned i = 0;
	BYTE temp;
	unsigned long Length;
	unsigned long isrReg;

	isrReg = inpd( RegBase + INT_STATUS );

	// Check for Received data and check the buffer:
	if (!( isrReg & INT_RXINTR ) &&
		( !descriptorPool.RxDescriptor[ RxDescriptorIndex ].cmdStatus.rxStatus.ReceiveOwn ))
	{
		printf("\n\nWaiting to receive a packet...\n");

		isrReg = inpd( RegBase + INT_STATUS );
		while (!( isrReg & INT_RXINTR )) 
		{
			if (kbhit())
			{
				getch();
				cQuit = 1;
				return( '\0' );
			}

			msleep(50);
			isrReg = inpd( RegBase + INT_STATUS );
		}
	}


	if (cLoopback)
	{
		count = 1;
	}

    while( descriptorPool.RxDescriptor[ RxDescriptorIndex ].cmdStatus.rxStatus.ReceiveOwn )
    {
/*		//setReg(ISR,ISR_PRX);			// Clear Received bit in ISR
        // Length = getHeader( &descriptorPool.RxData[ 0 ] );
        if( !descriptorPool.RxDescriptor[ RxDescriptorIndex ].cmdStatus.rxStatus.ReceiveOwn )
		{
			break;
		}*/

        Length = descriptorPool.RxDescriptor[ RxDescriptorIndex ].cmdStatus.rxStatus.descriptorSize;

		// In case we get a bad length:
		if( Length > MAX_ETHER_PACKET_SIZE )
		{
			Length = MAX_ETHER_PACKET_SIZE;
		}

		//Place the Packet into Packet array
        for (i = 0; i < Length; i++)
		{
			Packet[i] = descriptorPool.RxData[RxDescriptorIndex][i];
		}
				    
        temp = displayPacket( Length, RECEIVE );
  				  
		// Retrieve the source address and use it as our destination address for ping test
        for (i = 0; i < 6; i++)
		{
			Destination[i] = Packet[i+6];
		}

        cReturn = 1; // Indicate that packet was received

		// If it's loopback or Master/Slave test, call sendPing
        if (cLoopback || (!cMaster && !cReceive))
		{
			if (cLoopback && (count == RX_DESCRIPTOR_POOL_SIZE))
			{
				cQuit = 1;
				break;
			}

			if (!sendPing())
			{
				return 1;
			}

			count++;
		}

        if (temp == 0x07)
		{
			cQuit = 1;
			break;
		}
   		
		// Do the clean up on the descriptor:
        descriptorPool.RxDescriptor[ RxDescriptorIndex ].cmdStatus.cmdStatus = 0;
        descriptorPool.RxDescriptor[ RxDescriptorIndex ].cmdStatus.rxStatus.descriptorSize
                = sizeof( etherPacket_t );

		outpd( RegBase, COMMAND_RXENABLE );

		RxDescriptorIndex++;
        if (RxDescriptorIndex == RX_DESCRIPTOR_POOL_SIZE)
		{
			RxDescriptorIndex = 0;
		}
	}

/*	// reset the descriptor head and receive enable:
    if (RxDescriptorIndex == RX_DESCRIPTOR_POOL_SIZE)
	{
		RxDescriptorIndex = 0;
		outpd( RegBase + RX_DP, DosLogicalToPhysical( &descriptorPool.RxDescriptor[ 0 ] ) );

		// Enable Rcv:
		outpd( RegBase, COMMAND_RXENABLE );
	}*/

    return( temp );
}


/*************************************************
*	Function:   MacSend
*	Purpose:	Transfers a packet from the NIC onto
*				the network.  The Buffer is first DMA'd
*				to the NIC using PCtoNIC.  The appropriate
*				registers are then set and a transmit 
*				command is issued.
*	Arguments:  Buffer - Pointer to the data to transfer
*				Length - The length of the packet
*	Returns:    nothing
*************************************************/
int MacSend(BYTE *Buffer, WORD Length)
{
	int i = 100;
	int retVal = 0;	
	unsigned long cmdReg;
	unsigned long isrReg;
    
    // zero out the memory. 
    memset( &descriptorPool.TxDescriptor, '\0', sizeof( HALSpace_t ) );

    printf("\n\n\nTransmitting Packet...");
    _disable();                                                     // Disable interrupts

    // Initialize the descriptor:
    descriptorPool.TxDescriptor.physPointer = DosLogicalToPhysical( Buffer );
    descriptorPool.TxDescriptor.cmdStatus.txStatus.descriptorSize = Length;
    descriptorPool.TxDescriptor.cmdStatus.txStatus.TransmitOwn = 1;

	// Wait for the MacPhyter to finish transmitting:
	_enable();
    while( i-- )
	{
		if( !inpd( RegBase+TX_DP ) )
		{
			break;
		}
	}

	// If MacPhyter never frees up then quit and exit:
	if ( !i )
	{
		printf( "\n\nUnable to post TxMits, MacPhyter TimeOut!\n" );
		return 1;
	}

	// Otherwise post the descriptor to the chip:
	outpd( RegBase+TX_DP, DosLogicalToPhysical( &descriptorPool.TxDescriptor ) );

	// Set TX ENABLE bit:
	//if( !cLoopback )
	{
		cmdReg = inpd( RegBase );
		cmdReg |= COMMAND_TXENABLE;
		outpd( RegBase, cmdReg );
	}

	// Wait for processing:
	//delay( 1 );
	msleep(100);

	isrReg = inpd( RegBase + INT_STATUS );

	if ( isrReg & ( INT_TXOK | INT_TXERR) ) 
	{
		if( isrReg & INT_TXOK )
		{
			printf("\n...Packet Transmitted OK.\n");
		}
		else
		{
			printf("\n...Transmit Failed %08lX.\n", isrReg );
			retVal = 1;
		}
	}
	else
	{
		printf("\n...Packet Transmission Idle! ISR: %08lX\n", isrReg );
		retVal = 1;
	}

	return retVal;
}               

void interrupt far mcPhyISR( )
{
	unsigned long isrReg;

	isrReg = inpd( RegBase + INT_STATUS );
	
	printf( "ISR register has following value:  %08lX", isrReg );
}

/* This Function places the MacPhyter into a variety of test modes: */
int pwrMgtMode( int wakeOption )
{
	// Read WCSR then clear (if any) internal Wake Event & Reset WCSR
	if (inpd( RegBase + WCSR) & 0x1) // If PHY interrupt
	{
		if (inpd( RegBase + PHY_MISR) & 0x4000)	// If there was Link change
		{
			// Clear and reset MISR
			outpd( RegBase + PHY_MISR, 0x0);
		}

		// Reset WSCR
		outpd( RegBase + WCSR, 0x00000000);
		msleep(100);
	}

	// Read WCSR to see if it contains any Wake Packet
	if (inpd( RegBase + WCSR) & 0x200) // If there was Magic Packet
	{
		// Reset Receiver
		outpd( RegBase + CR, 0x0);
		msleep(100);

		// Give RXDP a valid receiv descriptor pointer
		outpd( RegBase + RX_DP, DosLogicalToPhysical( &descriptorPool.RxDescriptor[ 0 ] ) );

		// Receiver Enable;
		outpd( RegBase + CR, 0x4);
	}

    // Clear any PMCSR activity and ready for Wake Event:
    PciBiosWriteConfigWord( currentBus, currentDev, currentFunc, 0x44, 0x8100 );
    PciBiosWriteConfigWord( currentBus, currentDev, currentFunc, 0x46, 0x0000 );

    // Enable Promiscuous Reception:
    outpd( RegBase + RX_FILTER_CTRL, 0xF0000000 );

    // Set the WCSR with the Wake Option:
    outpd( RegBase + WCSR, (unsigned long)wakeOption );

    // If Waking on link Change: setup micr and misr:
    if (wakeOption == 1)
	{
		// First Set the IMR to enable Phy AND PME Interrupts:
		outpd( RegBase + INT_MASK, 0x6000 );

		// Then Setup the PHY to generate ints on link change
		outpd( RegBase + PHY_MICR, 0x2 );
		outpd( RegBase + PHY_MISR, 0x4000 );
	}

    // Clear any PMCSR activity and ready for Wake Event:
    PciBiosWriteConfigWord( currentBus, currentDev, currentFunc, 0x44, 0x0103 );

    // Return All's Well
    return 0;
}

// The following is the input for the mediaSpeeds:
#define M10_HALF         0x00000000
#define M100_HALF        0x00004000
#define M10_FULL         0x00008000
#define M100_FULL        0x0000C000

void setMedia( unsigned long mediaSpeed )
{
	#define PHY_RST 0x00000400
	unsigned long currentCfg, newCfg;

	// Read the media speed:
	currentCfg = inpd( RegBase + CFG );

	// Update the Cfg register with the desired seting and PHY reset:
	newCfg = currentCfg & 0xFFFF1FFF;
	newCfg |= mediaSpeed;
	outpd( RegBase + CFG, newCfg | PHY_RST );

	// Wait for Phy Reset and then clear the PHY_RST bit:
	delay( 1 );
	outpd( RegBase + CFG, newCfg );
}

void EuphyterPhyReset()
{
    UINT32 dword;

	// Reset the PHY    
	outpd( RegBase + CFG, phyConfig | PINT_ACEN | PHY_RST );
	msleep(100);
    outpd( RegBase + CFG, phyConfig | PINT_ACEN );

	//
    // Wait for the phy to come out of reset.
    //
    for ( dword = BMCR_RESET; dword & BMCR_RESET; )
	{
		dword = inpd( RegBase + PHY_BMCR );
    }

	msleep(100);
}

void EuphyterPhyAdjust()
{
    unsigned long dword;

    //
    // Configure the PHY test registers for optimal performance
    //
    if ( siliconRev == 0x0200 )
	{
		outpd( RegBase + PHY_PAGESEL, 0x0001 );
		outpd( RegBase + PHY_PMDCSR,  0x0802 );
		outpd( RegBase + PHY_FCOCTRL, 0x0010 );
		outpd( RegBase + PHY_CCSDCFG, 0x0333 );
		outpd( RegBase + PHY_PGMCGM,  0x0860 );
		outpd( RegBase + PHY_TMR,     0x2100 );
		outpd( RegBase + PHY_CDCTRL2, 0x4f48 );
		outpd( RegBase + PHY_PAGESEL,      0 );
		dword = inpd( RegBase + PHY_10BTSCR );
		outpd( RegBase + PHY_10BTSCR, dword | 0x04 );
    }
    else if ( ( siliconRev & 0xff00 ) == 0x0300 )
	{
		outpd( RegBase + PHY_PAGESEL,  0x0001 );
		outpd( RegBase + PHY_PMDCSR,   0x189c );
		outpd( RegBase + PHY_TSTDAT,        0 );
		outpd( RegBase + PHY_CCDSPCFG, 0x5040 );
		outpd( RegBase + PHY_CCSDCFG,  0x008c );
		outpd( RegBase + PHY_PAGESEL,       0 );
    }
	else if ( ( siliconRev & 0xff00 ) == 0x0400 ||
                ( siliconRev & 0xff00 ) == 0x0500 )
	{
		outpd( RegBase + PHY_PAGESEL,  0x0001 );
		outpd( RegBase + PHY_PMDCSR,   0x189c );
		outpd( RegBase + PHY_PAGESEL,       0 );
    }

	msleep(10);
}

#define SPEED100 0x40000000

void CorrectPhyCoefficient()
{
    unsigned long dword;

	dword = inpd( RegBase + CFG );
    if ( (dword & SPEED100) && // Check if speed is 100 Mb/s
         ( siliconRev & 0xff00 ) == 0x0300 ||
         ( siliconRev & 0xff00 ) == 0x0400)
	{
		outpd( RegBase + PHY_PAGESEL, 0x0001 );
		dword = inpd( RegBase + PHY_CCDSPCFG );
        dword &= 0x0fff;
		outpd( RegBase + PHY_CCDSPCFG, dword | 0x1000 );

		msleep(100);

		dword = inpd( RegBase + PHY_TSTDAT );
		dword &= 0xff;
        if ( dword < 0x80 || dword > 0xD7 )
		{
			outpd( RegBase + PHY_TSTDAT, 0x00E8 );
			dword = inpd( RegBase + PHY_CCDSPCFG );
			outpd( RegBase + PHY_CCDSPCFG, dword | 0x0020 );
        }
		outpd( RegBase + PHY_PAGESEL, 0 );
    }

	msleep(10);
}