/*
 * DIAG.C - A window graphical diag program for DP83815
 */
#include "winlib.h"
#include "wgraphic.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <dos.h>
#include <time.h>
#include "GLOBAL.H"
#include "NICSTUFF.H"
#include "FUNCTION.H"


#if !defined(MAXPATH)
   #define MAXPATH 80
#endif


#define LEFTCLK MOUSE_LEFTCLK
#define CMD_SAVE  USER_EVENT+0

/* Error macro used by menu functions */
#define   MN_ERROR(result) { menu_error = 1;  return (result); }


#define numChoice 12
#define maxLen 30

#define M10_HALF			0x00000000
#define M100_HALF			0x00004000
#define M10_FULL			0x00008000
#define M100_FULL			0x0000C000
#define M_AUTO				0x0000E000

#define NO_LINK				ERROR
#define HAS_LINK			0x80000000
#define SPEED100			0x40000000
#define	FULL_DUPLEX			0x20000000

/*************************************************************************
*  function protypes
*/

int findNIC();
int init_test=1;
BYTE destAddr[]={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};


char  cReceiveOnly=0, cQuit=0,cTransmit=0,cReceive=0,cLoopback=0,cReturn=0,cHelp=1, cMaster=0;
char  cM100_Full=0, cM100_Half=0, cM10_Full=0, cM10_Half=0;
char  cRepeat = 0, cCount=0, cXmitFailed=0, cKeyHit=0;

WORD  SharedAddress=0;
unsigned short RegBase=0;
unsigned short batchMode=0;
unsigned long PciCfgSpace[ 12 ]; 
BYTE  MemMode=PIO;
BYTE  DataWidth=BYTEWIDE;
BYTE  NextPacket=0x47;
BYTE  PacketSize=60,Last=0;

void    diag_menu(Mouse_State *mstate);
void    minimize(Mouse_State *mstate);
void    maximize(Mouse_State *mstate);
void    restore(Mouse_State *mstate);
void    do_menu(Mouse_State *mstate);    /* Do menu when clicked by mouse */
void    set_box_colors(void);            /* Set defaults for pop-up boxes */


void	sys_help(char *help_id);        /* Display a centered help window */
void    sys_error(char *message);           /* Print system error message */
void    sys_message(int center, char *message); /* Display system message */
void    sys_front(Mouse_State *mstate);       /* Move window to the front */
void    e_view(void);						  /* View existing NIC config */
void    e_setup(void);								  /* Setup NIC config */
void    e_diagnostic(void);              /* Run Diagnostic for NIC config */
void    e_quit(void);                   /* Quit to Dos */
Window  new_diag(struct info *np);            /* Create a new diag window */
Window  close_diag(Window diag_wd);                /* Close a diag window */
Window  next_diag(Window start);   /* Next notepad in display sequence */

int     ask_save(Window diag_wd);    /* Last chance to save modified window */
void    e_write(Window diag_wd);        /* write to a file */
void    e_move(Window diag_wd);           /* move viewport */
void    e_size(Window diag_wd);        /* re-size viewport */
void    e_help(void);              /* show notepad help */
void    e_menu(void);         /* menu */
void    e_input(BYTE *destAddr);         /* input field for data entry*/
int     e_inputNum(void);         /* input field for repetition*/
int writeEeCfg( unsigned long newCfg );

void    save_default_size(Window diag_wd);
void    sys_color_change(void);       /* Set colors to be used */
void	sys_mouseon(void);
void	sys_mouseoff(void);

void diagquit();
void ViewCurrentCfgWd(void);
void MacLoopbackWd(void);
void ChangeMacAddrWd(void);
void DisplayEEpromWd(void);
void RunEEpromTestWd(void);
void InitEEpromWd(void);
void PhyLoopbackWd(void);
void selMaster(void);
void LoopbackWd(void);
void WakeOnLanWd(void);
void WakeOnLinkWd(void);
void MagicPacketWd(void);
void DisplayNicWd(void);	
void DisplayPCIWd(void);
void RecTestWd(void);
void selSlave(void);
void XmitTestWd(void);
void ExtLoopbackWd(void);

void SelAutoDetect(void);
void SelHighFull(void);
void SelHighHalf(void);
void SelLowFull(void);
void SelLowHalf(void);
void selBootRomEnable(void);
void selBootRomDisabled(void);
void selFlowCtrl_NONE(void);
void selFlowCtrl_RX_ONLY(void);
void selFlowCtrl_TX_ONLY(void);
void selFlowCtrl_TX_RX(void);
void selWakeOnLanEnabled(void);	
void selWakeOnLanDisabled(void);
void selWakeOnLinkEnabled(void);
void selWakeOnLinkDisabled(void);
void setMedia( unsigned long mediaSpeed );


void diag_sel();
void diag_cursup();
void diag_cursdown();
void diag_help();

Menu    menu_root(void);                     /* Build root menu (level 1) */
void    menu_selection(void);               /* Convert menu item to a key */
Menu    menu_vert(char *menu_title, int col, int row, char **names, char **helpid,
                unsigned *codes, void (*call[])(void));  /* make sub-menu */

int make_menu(void);                        /* Create pull-down menu */
int diag_changed(Window wd, int id, int check, int x, int y,
                        int w, int h);     /* Notify when window content changed */

struct _colors {
   int  help, top_wd, mid_wd, bot_wd, system, menu, menu_bar, menu_key, form, field;
   int  message, error, title, border, box, box_title, button, button_focus;
};

struct diagnostic_defaults {
   struct _colors colors;
   struct _colors mono_colors;
   struct _colors bw_colors;
};

struct diag_list {
   struct diag_list *next;
   Window diag;
};


struct info {
    int   color;               /* Window color */
    int   title_color;         /* Color of title */
    int   border_color;        /* Color of border */
    char  col, row;            /* Window viewport coordinates */
    char  vwidth, vheight;     /* Window viewport dimensions */
    int   wwidth, wheight;     /* Window dimensions */
	int   border;			   /* Window boarder */	
};

struct    info  np_info =              /* Default size, color & position */
          {COLOR(WHITE+BRIGHT,BLUE),   /* wd color */
           COLOR(WHITE+BRIGHT,BLUE),   /* title color */
           COLOR(WHITE+BRIGHT,BLUE),   /* border color */
           8, 4,                       /* screen column and row */
           60, 12,                     /* viewport width & height */
           80, 100,                    /* window width & height */
		   0};
	
struct    info  top_wd_info =           /* Default size, color & position */
          {COLOR(YELLOW+BRIGHT,BLUE),   /* wd color */
           COLOR(YELLOW+BRIGHT,BLUE),   /* title color */
           COLOR(YELLOW+BRIGHT,BLUE),   /* border color */
           4, 1,                        /* screen column and row */
           70, 1,                       /* viewport width & height */
           70, 1,                       /* window width & height */
		   1};							/* No border */
	
struct    info  mid_wd_info =           /* Default size, color & position */
          {COLOR(YELLOW+BRIGHT,CYAN),    /* wd color */
           COLOR(WHITE+BRIGHT,CYAN),    /* title color */
           COLOR(WHITE+BRIGHT,CYAN),    /* border color */
           10, 6,                       /* screen column and row */
           60, 10,                      /* viewport width & height */
           60, 10,                      /* window width & height */
		   0};						    /* Yes border */

struct    info  mid_xl_wd_info =           /* Default size, color & position */
          {COLOR(YELLOW+BRIGHT,CYAN),    /* wd color */
           COLOR(WHITE+BRIGHT,CYAN),    /* title color */
           COLOR(WHITE+BRIGHT,CYAN),    /* border color */
           10, 4,                       /* screen column and row */
           60, 17,                      /* viewport width & height */
           60, 17,                      /* window width & height */
		   0};						    /* Yes border */

struct    info  bot_wd_info =           /* Default size, color & position */
          {COLOR(WHITE+BRIGHT,GREEN),   /* wd color */
           COLOR(WHITE+BRIGHT,GREEN),   /* title color */
           COLOR(WHITE+BRIGHT,GREEN),   /* border color */
           5, 23,                       /* screen column and row */
           70, 1,                       /* viewport width & height */
           70, 1,                       /* window width & height */
		   1};							/* No border */

struct    info  error_info =           /* Default size, color & position */
          {COLOR(WHITE+BRIGHT,RED),    /* wd color */
           COLOR(WHITE+BRIGHT,CYAN),    /* title color */
           COLOR(WHITE+BRIGHT,CYAN),    /* border color */
           10, 6,                       /* screen column and row */
           60, 10,                      /* viewport width & height */
           60, 10,                      /* window width & height */
		   0};						    /* Yes border */

struct diag_data {
    char  *dataname;           /* data */
    int   modified;            /* Text has been changed */
    int   pen_color;           /* Color of text */
    int   title_color;         /* Color of window title */
    int   border_color;        /* Color of border */
    int   default_x, default_y;  /* Data for restore */
    int   default_w, default_h;
};

/*************************************************************************
*  global definition
*/

Window    diag;                          /* Currently active diag */
struct    _colors   colors;                 /* Structure holding colors */
Window    sys=NULL;                         /* System message window    */
int       help_status;                         /* 0 if help file loaded */
int       menu_error;      /* 1 if an error occurs while building menus */
int       color_display = 0;                  /* Display supports color */
int       mono_display = 0;               /* Monochrome display adapter */
int       display_type;                  /* Type of video adapter found */
int       graphics_mode = 0;                       /* Use graphics mode */
int       display_lines = 0;            /* Number of lines in text mode */
int       display_main_menu = 1;       /* Leave main menu on the screen */
int       mouse_on = 1;                        /* Mouse pointer visible */
int       main_menu_request = 0;         /* Main menu selected by mouse */
char      cur_dir[MAXPATH] = "\0";       /* Default directory for files */

/*--- Menu variables ---*/
Menu    root=NULL, pwrmanagement=NULL, loopback= NULL, 
		mn_SetupCfg=NULL, mn_DataRate=NULL, mn_BootRom=NULL,
		mn_FlowCtrl=NULL, mn_WOLan=NULL, mn_WOLink=NULL,
		mn_NetDiag=NULL,
		mn_LanWakeSignal=NULL,
		view=NULL, setup=NULL, diagnostic=NULL;

/*--- Define sub-menu temination keys ---*/
unsigned term_vertical[] = {_LARROW, _RARROW, _ESC, 0};


void menu_selection(void)      /*--- Called on some item selections ---*/
{                           /*--- Convert a menu selection to a key ---*/
    Menu   menu_selected;
    unsigned key;

    d_change(MAUTOCLOSE, ENABLE);           /* Close all menus on return */
    menu_selected = m_lastmenu();   /* Get menu containing selected item */
    key = m_lastkey(menu_selected);                      /* Get key code */
    k_ungetkey(key);       /* Make notepad command key the next key read */
}


/*-------------------------- Main Menu -------------------------------- */
char  *main_name[] = {
	"   ~View Current Configuration ",
	"   ~Set Up New Configuration   ",
	"   ~Loopback Test              ",
	"   Po~wer Management Test      ",
	"   ~Transmit Test Packet(s)    ",
	"   ~Receive Test Packet(s)     ",
	"   Run ~EEPROM Tests           ",
	"   ~Help                       ",
	"   E~xit To Dos                ",
    NULL};
unsigned main_menu_code[] = {
		'V',
		'S',
		'L',
		'W',
		'T',
		'R',
		'E',
		'H',
		ALT_F4};
void (*main_menu_fn[])(void) = {
	ViewCurrentCfgWd,
	0,
	0,
	0,
	XmitTestWd,
	RecTestWd,
	RunEEpromTestWd,
	e_help,
	menu_selection};

char  *main_help[] = {
	"ViewCurrentCfgWd",
	"SetupNewCfgWd",
	"LoopbackWd",
	"PowerMangWd",
	"XmitTestWd",
	"RecTestWd",
	"RunEEpromTestWd",
	"diag",
	"quit"};

/* ------------------------ Setup New Configuration Menu ------------------------ */
char  *setupcfg_name[]={

    "    ~Auto Detect             ",
    "    100 ~FULL DUPLEX         ",
    "    100 ~HALF DUPLEX         ",
    "    10  F~ULL DUPLEX         ",
    "    10  HA~LF DUPLEX         ",
    NULL};

void (*setupcfg_menu_fn[])(void) = {
	SelAutoDetect,
	SelHighFull,
	SelHighHalf,
	SelLowFull,
	SelLowHalf};
	
unsigned setupcfg_menu_code[] = {
	'A',
	'F',
	'H',
	'U',
	'L'};
char  *setupcfg_help[] = {
    "Selectmediumspeed",
    "Selectmediumspeed",
    "Selectmediumspeed",
    "Selectmediumspeed",
    "Selectmediumspeed"};
	
/* --------------------- Select Data Rate Menu ----------------------------- */
char  *selDataRate_name[] = {                                    /* Setup menu */
    "    Auto Detect             ",
    "    100 Full Duplex         ",
    "    100 Half Duplex         ",
    "    10  Full Duplex         ",
    "    10  Half Duplex         ",
    NULL};
void (*selDataRate_menu_fn[])(void) = {
	SelAutoDetect,
	SelHighFull,
	SelHighHalf,
	SelLowFull,
	SelLowHalf};
	
unsigned selDataRate_menu_code[] = {
	0,
	0,
	0,
	0,
	0};
char  *selDataRate_help[] = {
    "Selectmediumspeed",
    "Selectmediumspeed",
    "Selectmediumspeed",
    "Selectmediumspeed",
    "Selectmediumspeed"};
	

/* ----------------------- Select Boot Rom Menu ----------------------------- */
char  *selBootRom_name[] = {                                    /* Setup menu */
    "  Boot ROM Enabled          ",
    "  Boot ROM Disabled         ",
    NULL};
void (*selBootRom_menu_fn[])(void) = {
	selBootRomEnable,
	selBootRomDisabled};
	
unsigned selBootRom_menu_code[] = {
	0,
	0};
char  *selBootRom_help[] = {
    "SelectBootRom",
	"SelectBootRom"};
	

/* ------------------- Select Flow Control Menu ----------------------------- */
char  *selFlowCtrl_name[] = {                                    /* Setup menu */
    "  Tx Disable, Rx Disable    ",
    "  Tx Disable, Rx Enable     ",
    "  Tx Enable,  Rx Disable    ",
    "  Tx Enable,  Rx Enable     ",
    NULL};
void (*selFlowCtrl_menu_fn[])(void) = {
	selFlowCtrl_NONE,
	selFlowCtrl_RX_ONLY,
	selFlowCtrl_TX_ONLY,
	selFlowCtrl_TX_RX};
	
unsigned selFlowCtrl_menu_code[] = {
	0,
	0,
	0,
	0};
char  *selFlowCtrl_help[] = {
    "SelectFlowControl",
    "SelectFlowControl",
    "SelectFlowControl",
    "SelectFlowControl"};

/* -------------------- Select Wake On Lan Menu ------------------------------- */
char  *selWakeOnLan_name[] = {                                    /* Setup menu */
    "  Wake On LAN Enabled       ",
    "  Wake On LAN Disabled      ",
    NULL};
void (*selWakeOnLan_menu_fn[])(void) = {
	selWakeOnLanEnabled,
	selWakeOnLanDisabled };
	
unsigned selWakeOnLan_menu_code[] = {
	0,
	0};
char  *selWakeOnLan_help[] = {
    "SelectWakeOnLan",
	"SelectWakeOnLan"};

/* -------------------- Select Wake On Link Menu ------------------------------ */
char  *selWakeOnLink_name[] = {                                    /* Setup menu */
    "  Wake On Link Enabled      ",
    "  Wake On Link Disabled     ",
    NULL};
void (*selWakeOnLink_menu_fn[])(void) = {
	selWakeOnLinkEnabled,
	selWakeOnLinkDisabled };
	
unsigned selWakeOnLink_menu_code[] = {
	0,
	0};
char  *selWakeOnLink_help[] = {
    "SelectWakeOnLink",
	"SelectWakeOnLink"};

	
/* -------------------- Network Diag Menu ------------------------------ */
char  *NetDiag_name[] = {                                    /* Setup menu */
    "  ~Master                 ",
    "  ~Slave                  ",
    NULL};
void (*NetDiag_menu_fn[])(void) = {
	selMaster,
	selSlave};
	
unsigned NetDiag_menu_code[] = {
	'M',
	'S'};
char  *NetDiag_help[] = {
    "SelectMaster",
	"SelectSlave"};


	
/* -------------------------- PowerMang Menu ------------------------------- */
char  *powermang_name[] = {                                    /* Setup menu */
    "    ~Wake On LAN              ",
    "    Wake On L~INK             ",
    "    ~Magic Packets            ",
    NULL
    };
void (*powermang_menu_fn[])(void) = {
	WakeOnLanWd,
	WakeOnLinkWd,
	MagicPacketWd,
	menu_selection};
	
unsigned powermang_menu_code[] = {
	'W',
	'I',
	'M'};
char  *powermang_help[] = {
    "WakeOnLanWd",
    "WakeOnLinkWd",
	"MagicPacketWd"};
	
	
/* ---------------------------- LoopBack  Menu ---------------------------- */
char  *loopback_name[] = {                                    /* Setup menu */
	"    ~MAC Loopback Test        ",
	"    ~PHY Loopback Test        ",
    NULL
    };
void (*loopback_menu_fn[])(void) = {
	MacLoopbackWd,
	PhyLoopbackWd,
	menu_selection};
	
unsigned loopback_menu_code[] = {
	'M',
	'P'};
char  *loopback_help[] = {
    "MacLoopbackWd",
		"PhyLoopbackWd"};
	
/* ---------------------------- Setup Menu ----------------------------- */
char  *setup_name[] = {                                    /* Setup menu */
    " ~View Current Configuration ",
    " ~This is Set Up New Configuration   ",
    " ~Run Diagnostics            ",
    " ~Exit To Dos                ",
    NULL
    };
unsigned setup_menu_code[] = {0, 0, 0, ALT_F4};
void (*setup_menu_fn[])(void) = {e_view, e_setup, e_diagnostic, menu_selection};
char  *setup_help[] = {
    "view", "setup", "diagnostic", "quit"};


/* ---------------------------- diagnostic Menu  ---------------------------- */
	char  *diagnostic_name[] = {                           /* diagnostic menu */
    " ~View Current Configuration ",
    " ~Set Up New Configuration   ",
    " ~This is Run Diagnostics            ",
    " ~Exit To Dos                ",
    NULL
    };
unsigned diagnostic_menu_code[] = {0, 0, 0, ALT_F4};
void (*diagnostic_menu_fn[])(void) = {e_view, e_setup, e_diagnostic, menu_selection};
char  *diagnostic_help[] = {
    "view", "setup", "diagnostic", "quit"};



int DiagChoice[numChoice];
char DiagChoiceName[numChoice][maxLen]={
	"    Initialize ~EEPROM     ",
	"   ~Change MAC Address     ",
	"      ~Master mode         ",
	"      ~Slave mode          ",
	"    M~AC Loopback Test     ",
	"    P~HY Loopback Test     ",
	" E~xternal Loopback Test   ",
	" ~Transmit Test Packet(s)  ",
	"  ~Receive Test Packet(s)  ",
	"  Display ~NIC Registers   ",
	" ~Display EEPROM Contests  ",
	" Display ~PCI Config space "};

char strTitle[]="DIAG.EXE";
Control ctl[numChoice+3];

struct    diag_list *diags = NULL;       /* List of open diag windows */

struct    diagnostic_defaults diag_default = {     /* Default colors etc   */
         {COLOR(WHITE+BRIGHT,GREEN),    /* help window color */
          COLOR(YELLOW+BRIGHT,BLUE),     /* top window color */
          COLOR(YELLOW+BRIGHT,CYAN),     /* mid window color */
          COLOR(WHITE+BRIGHT,GREEN),     /* bot window color */
          COLOR(BLUE+BRIGHT, WHITE),    /* system color */
          COLOR(BLUE, WHITE),           /* menu color */
          COLOR(WHITE, RED),            /* menu bar */
          COLOR(RED, WHITE),            /* menu hot key */
          COLOR(BLUE+BRIGHT, WHITE),    /* data entry field prompt */
          COLOR(WHITE+BRIGHT, WHITE),   /* data entry field */
          COLOR(BLACK, CYAN),           /* message color */
          COLOR(WHITE+BRIGHT, RED),     /* error message color */
          COLOR(WHITE+BRIGHT,BLUE),     /* diag title color */
          COLOR(WHITE+BRIGHT,BLUE),     /* diag border color */
          COLOR(BLUE, WHITE),           /* popup box color */
          COLOR(YELLOW+BRIGHT, WHITE),  /* menu title in popup box */
          COLOR(WHITE+BRIGHT, BLUE),    /* push button color */
          COLOR(WHITE+BRIGHT, CYAN)     /* push button with focus */
          }, {    /* monochrome display */
          0x70, 0x70, 0x70, 0x70, 0x70, 0x07,       
			  /* help, top window, mid window, bot window, system, menu */
          0x70, 0x0f, 0x07, 0x70,       /* menu bar, key, form, field  */
          0x07, 0x0f, 0x70, 0x70,       /* message, error, title, border */
          0x07, 0x0f, 0x70, 0xf0        /* box, box title, button, focus */
          }, {    /* color display with black and white monitor */
          0x70, 0x70, 0x70, 0x70, 0x70, 0x07,       
			  /* help, top window, mid window, bot window, system, menu */
          0x70, 0x0f, 0x07, 0x70,       /* menu bar, key, form, field  */
          0x07, 0x0f, 0x70, 0x70,       /* message, error, title, border */
          0x07, 0x0f, 0x70, 0x7f        /* box, box title, button, focus */
          }
};  /* end struct */

/*************************************************************************
*  set_box_colors()
*/
void set_box_colors(void)               /* Set defaults for pop-up boxes */
{
   d_change(WCOLOR, colors.box);
   d_change(WBRDCOLOR, colors.box);
   d_change(MTTLCOLOR, colors.box_title);
   d_change(BUTTONCOLOR, colors.button);
   d_change(BSELECTCOLOR, colors.button);
   d_change(BFOCUSCOLOR, colors.button_focus);
   d_change(BSFOCUSCOLOR, colors.button_focus);
   d_change(GRAYCOLOR, COLOR(BLACK+BRIGHT, BLACK));
}


/*************************************************************************
*  e_next()
*/
void  e_next(Window wd)        /*--- Move the next notepad to the top ---*/
{
   Window w;
   w = next_diag(wd);
   if (w != diag) w_back(diag);
   if (w != NULL) w_front(w);
}

/*************************************************************************
*  e_write()
*/
void  e_write(Window diag_wd)
                                      /*--- Write window text to file ---*/
{
   struct  diag_data *data;
   if (diag_wd == NULL) return;
   data = w_user_pointer(diag_wd, NULL);           /*--- get name of file --*/
//   sys_write(diag, data->filename);   /*--- Write window text to file ---*/
   data->modified = 0;
}

/*************************************************************************
*  e_view()
*/
void  e_view(void)                         
{
   Window diag_wd;
   int col, row;
   if (diag != NULL) {
      col = w_info(diag, ISCREENCOL)+2;
      row = w_info(diag, ISCREENROW)+1;
      if (col+np_info.vwidth > (int)ex_crt_columns-1 ||
         row+np_info.vheight > (int)ex_crt_rows-2) {
         col = 1;
         row = 2;
         }
      }
   else {col = 8; row = 4;}
   np_info.col = (char)col;
   np_info.row = (char)row;
   diag_wd = new_diag(&np_info);
   if (diag_wd != NULL) {
      diag = diag_wd;
      }
}

/*************************************************************************
*  e_setup()
*/
void  e_setup(void)                         
{
   /******/
}

/*************************************************************************
*  e_diagnostic()
*/
void  e_diagnostic(void)                         
{
   /******/
}

void diag_menu(Mouse_State *mstate)     /* create menu for menu button */
{
   Menu mm;
   Menu_Item item;
   int idx;
   Window wd = mstate->window;
   static char *names[7] = {"~Restore ", "~Move    ", "~Size    ",
                            "Mi~nimize", "Ma~ximize", "~Next    ",
                            "~Close   "};
   mm = m_create(NULL, w_info(wd, ISCREENCOL), w_info(wd, ISCREENROW), 8, 7);
   if (mm == NULL) return;
   for (idx = 0; idx < 7; idx++) {
      item = m_additem(mm, names[idx]);
      if (item == NULL) {m_free(mm); return;}
      m_itemclose(item,1);
      }
   if (m_open(mm) == _ESC) {m_free(mm); return;}
   idx = m_lastindex(mm);
   switch (idx) {
      case 0 : restore(mstate); break;
      case 1 : e_move(wd); break;
      case 2 : e_size(wd); break;
      case 3 : minimize(mstate); break;
      case 4 : maximize(mstate); break;
      case 5 : e_next(wd); break;
      case 6 :
         diag = close_diag(wd);
         break;
      }
   m_free(mm);
}

/*************************************************************************
*  e_move()
*/
void  e_move(Window diag_wd)                  /*--- Move window viewport ---*/
{
   int      cols, rows;
   unsigned key;
   int      wcol, wrow, vheight, vwidth;
   Mouse_State mstate;

   sys_message(1,"Press arrow keys to move window and Esc(ape) to end move.");
   w_cursoff();
   do {
      wcol = w_info(diag_wd, ISCREENCOL);
      wrow = w_info(diag_wd, ISCREENROW);
      vheight = w_info(diag_wd, IVIEWHEIGHT);
      vwidth = w_info(diag_wd, IVIEWWIDTH);
      cols = 0; rows = 0;
      key = s_getkey(&mstate);
      switch (key) {
         case _RARROW  :
            if ((wcol + vwidth) <= (int)ex_crt_columns-2) cols = 1;
            break;
         case _LARROW  :
            if (wcol > 1) cols = -1;
            break;
         case _DNARROW :
            if ((wrow + vheight) <= (int)ex_crt_rows-2) rows = 1;
            break;
         case _UPARROW :
            if (wrow > 1) rows = -1;
            break;
         case _ENTER:
         case _ESC: break;
         case MOUSE_LEFTCLK:
         case MOUSE_RIGHTCLK:
         case MOUSE_CENTRCLK:
            s_ungetkey(&mstate);
            key = _ESC;
            break;
         }
      if (rows != 0 || cols != 0) w_moveto(diag_wd, wcol+cols, wrow+rows);
      } while (key != _ESC && key != _ENTER);
   sys_message(0, NULL);
}

/*************************************************************************
*  e_size()
*/
void  e_size(Window diag_wd)               /*--- Re-size window viewport ---*/
{
   int    cols, rows;
   unsigned key;
   int    wcol, wrow, vheight, vwidth;
   Mouse_State mstate;

   sys_message(1,"Press arrow keys to re-size viewport and Esc(ape) to end operation.");
   w_cursoff();
   do {
      wcol = w_info(diag_wd, ISCREENCOL);
      wrow = w_info(diag_wd, ISCREENROW);
      vheight = w_info(diag_wd, IVIEWHEIGHT);
      vwidth = w_info(diag_wd, IVIEWWIDTH);
      cols = 0; rows = 0;
      key = s_getkey(&mstate);
      switch (key) {
         case _RARROW  :
            if (wcol+vwidth <= (int)ex_crt_columns-2) {
               cols = 1;
               if (w_info(diag_wd, ICOLS) <= vwidth)
                  if (w_size(diag_wd, vwidth+1, w_info(diag_wd, IROWS)) != 0)
                     cols = 0;
               }
            break;
         case _LARROW  :
            if (vwidth > 1) cols = -1;
            break;
         case _DNARROW :
            if (wrow + vheight < (int)ex_crt_rows-2) {
               rows = 1;
               if (w_info(diag_wd, IROWS) <= vheight)
                  if (w_size(diag_wd, w_info(diag_wd, ICOLS), vheight+1) != 0)
                     rows = 0;
               }
            break;
         case _UPARROW :
            if (vheight > 1) rows = -1;
            break;
         case _ENTER:
         case _ESC: break;
         case MOUSE_LEFTCLK:
         case MOUSE_RIGHTCLK:
         case MOUSE_CENTRCLK:
            s_ungetkey(&mstate);
            key = _ESC;
            break;
         }
      if (rows != 0 || cols != 0) w_viewsize(diag_wd, vwidth+cols, vheight+rows);
      } while (key != _ESC && key != _ENTER);
   sys_message(0, NULL);
}


/*************************************************************************
*  e_input()
*/
void  e_input(BYTE *destAddr)    /*--- Field for input data ---*/
{

	Window box;
	Field  fd[6];
	char *prompt[]={"DA ",
		            " ",
					" ",
					" ",
					" ",
					" "
	};

	char *template[]={
		"__",
		"__",
		"__",
		"__",
		"__",
		"__"
	};

	int col[]={1, 6, 9, 12, 15, 18};
	int row[]={0, 0, 0, 0, 0, 0};


	int key;
	BYTE inputString[25];
	char buffer[18];
	int x, y, i=0;

	d_change(WUMCOLOR, colors.box_title);
	d_change(WBRDCOLOR, colors.box);
	d_change(WBRDTYPE, BRD_SSSS);
	d_change(WCOLOR, colors.box);
   
	box = w_open(30, 13, 40, 1);
	if (box == NULL) return;
	w_umessage(box, " Set MAC Destination Address ");
	for (i =0; i< 6; i++){
		fd[i]=f_create(prompt[i], template[i]);
		f_append(box, col[i], row[i], fd[i]);
		f_setbuffer(fd[i], "00");
	}

	f_return(fd[5], ENABLE);
	key=f_batch(box);

	if (key != _ESC && key != -1 && (unsigned)key != MOUSE_LEFTCLK) {


	for (x=0; x < 6; x++)
	{
		f_getstring(fd[x], buffer+(x*2));
	}

	for (x=0; x < 6; x++)
	{
	  sscanf(buffer+(x*2), "%02X", inputString+(x*2));
	}

	for (x=0; x< 6; x++)
	{
		destAddr[x]=*(inputString+x*2);
	}


      f_free(fd);
      }
   else f_free(fd);
	w_close(box);
}

/*************************************************************************
*  e_inputNum()
*/
int  e_inputNum(void)    /*--- Field for input data ---*/
{

	Window box;
	Field  fd;

	int key;
	int numPacket=0;
	d_change(WUMCOLOR, colors.box_title);
	d_change(WBRDCOLOR, colors.box);
	d_change(WBRDTYPE, BRD_SSSS);
	d_change(WCOLOR, colors.box);
   
	box = w_open(30, 13, 40, 1);
	if (box == NULL) return(numPacket);
	w_umessage(box, " Input Number of Packets ");

	fd=f_create("Enter Number of Packets ", "__");
	f_setint(fd, numPacket, ENABLE, DISABLE, DISABLE);
	f_process(box,0,0,fd);


	if (key != _ESC && key != -1 && (unsigned)key != MOUSE_LEFTCLK) {
	numPacket=f_getint(fd);
    f_free(fd);
      }
   else f_free(fd);

	w_close(box);
	return numPacket;
}
					  
					  
/*************************************************************************
*  e_help()
*/
void  e_help(void)    /*--- Display a help commands ---*/
{
   sys_help("diag");
}

/*************************************************************************
*  e_menu()
*/
void e_menu(void)                          
{
   menu_error   = 0;
   d_change(MAUTOCLOSE, ENABLE);
   if (m_getwindow(root) == NULL) m_open(root);
   else m_process(root);
}

/*************************************************************************
*  do_menu()
*/
void do_menu(Mouse_State *mstate)
{
   if (m_getwindow(root) == NULL) m_open(root);
   else m_process(root);

}
   
/*************************************************************************
*  new_diag()
*/
Window new_diag(struct info *np)      /* Open initial diag window */
{
   Window diag_wd;
   struct diag_data *data;
   struct diag_list *list;

   /*--- Open initial notepad window ---*/

   if (np->border){
   d_change(WBRDTYPE, BRD_NONE);           /* Change border type default */
   }
   else{
   d_change(WBRDTYPE, BRD_DDDD);           /* Change border type default */
   }
   d_change(WCOLOR, np->color);                /* Change color of window */
   d_change(WBRDCOLOR, np->border_color);         /* Change border color */
   d_change(WUMCOLOR, np->title_color);    /* Change notepad title color */
   d_change(WDISPLAY, DISABLE);
   if ((diag_wd = w_open(np->col, np->row, np->vwidth, np->vheight))==NULL)
      {d_change(WDISPLAY, ENABLE); return NULL;}

   /*--- Change cursor size to reflect insert mode ---*/
   if (mono_display) w_cursize(diag_wd, 2, 7);
   else w_cursize(diag_wd, 5, 12);
   if (w_size(diag_wd, np->wwidth, np->wheight) == -1) { /*Change window size*/
      d_change(WDISPLAY, ENABLE);
      w_close(diag_wd);
      return NULL;
      }
   data = malloc(sizeof(struct diag_data));
   list = malloc(sizeof(struct diag_list));
   if (data == NULL || list == NULL)
      {d_change(WDISPLAY, ENABLE); w_close(diag_wd); return NULL;}
   data->modified = 0;
   data->default_x = np->col;
   data->default_y = np->row;
   data->default_w = np->vwidth;
   data->default_h = np->vheight;
   data->pen_color = np->color;
   data->title_color = np->title_color;
   data->border_color = np->border_color;
   w_user_pointer(diag_wd, data);
   w_note_change(diag_wd, diag_changed);
   list->diag = diag_wd;
   list->next = diags;                     /* add to list of notepads */
   diags = list;
   d_change(WDISPLAY, ENABLE);
   w_on(diag_wd);
   return(diag_wd);                              /* Return Window data type */
}

/*************************************************************************
*  next_diag()
*/
Window next_diag(Window start)   
{
   Window w, diag_wd = NULL;
   struct diag_list *list;
   w = w_next(start);
   while (w != NULL) {
      if (w_get_parent(w) == NULL) {   /* a sub-window is not a notepad */
         list = diags;                  /* is this window a notepad? */
         while (list != NULL && list->diag != w) list = list->next;
         if (list != NULL) {
            diag_wd = list->diag;
            w = NULL;
            }
         }
      if (w != NULL) w = w_next(w);
      }
   return diag_wd;
}

/*************************************************************************
*  close_diag()
*/
Window  close_diag(Window diag_wd)            
{
   struct diag_data *data;
   struct diag_list *list, **last;

   if (diag_wd == NULL) return next_diag(NULL);
   data = w_user_pointer(diag_wd, NULL);
   if (data->modified) {          /*--- last chance to save changes ---*/
      if ((unsigned)ask_save(diag_wd) == CMD_CANCEL) return next_diag(NULL);
      }
   if (data != NULL) {
      free(data);
      }

	   last = &diags;                        /* Remove from notepad list */
   list = *last;
   while (list != NULL && list->diag != diag_wd) {
      last = &list->next;
      list = *last;
      }
   if (list != NULL) {
      *last = list->next;
      free(list);
      }
   w_close(diag_wd);
   return (next_diag(NULL));                   
}

/*************************************************************************
*  diag_changed()
*/
int diag_changed(Window wd, int id, int check, int x, int y, int w, int h)
{           
	if (check) {
      switch (id) {
         case _CHG_MOVE:
            if (w_info(wd, ISCREENROW)+y < 2) return -1;
         case _CHG_SIZE:
            if (w_info(wd, ISCREENROW)+w_info(wd, IVIEWHEIGHT)+y
                >= (int)ex_crt_rows-1) return -1;
            break;
         case _CHG_SIZEAT:    /* move by (x,y) size by (w,h) */
            if (w_info(wd, ISCREENROW)+y < 2) return -1;
            if (w_info(wd, ISCREENROW)+w_info(wd, IVIEWHEIGHT)+y+h
                >= (int)ex_crt_rows-1) return -1;
            break;
         default: w = x; x=w;   /* eliminate argument not used warning */
         }
      return 0;
      }
   if (id == _CHG_FRONT) {diag = wd;}
   else if (id == _CHG_BACK) {diag = next_diag(NULL);}
   return 0;
}

/*************************************************************************
*  sys_mouseon()
*/
void sys_mouseon(void)
{                           /* enable the display of the mouse pointer */
   if (mouse_on) return;
   s_showpointer();
   mouse_on = 1;
   }

/*************************************************************************
*  sys_mouseroff()
*/
void sys_mouseoff(void)
{                  /* turn off the mouse pointer during keyboard input */
   if (mouse_on) {
      s_hidepointer();
      mouse_on = 0;
      }
   }

/*************************************************************************
*  sys_front()
*/
void sys_front(Mouse_State *mstate)          
{                                /*  If already in front, position cursor */
   if (mstate->window == diag)
      w_poscurs(mstate->window, mstate->col, mstate->row);
   else if (mstate->window != NULL) w_front(mstate->window);
}

/*************************************************************************
*  sys_help()
*/
void sys_help(char *help_id)                /*--- display help window ---*/
{
   Window    help;
   Defaults  def;
   Mouse_State mstate;
   unsigned  key;
   int       col, row, height, width, view_row, chg, rows;
   char      *null_item = "null";

   if (help_status == -1) {
       sys_error("Warning: Help file not loaded");
       return;
      }
   /*--- Change defaults and open, but do not display, help window ---*/
   def = d_get();                              /* Save current defaults */
   d_change(WDISPLAY, DISABLE);
   d_change(WBRDTYPE, BRD_DSDS);
   d_change(WCOLOR, colors.help);
   d_change(WBRDCOLOR, colors.help);
   d_change(WUMCOLOR, colors.help);
   d_change(WLMCOLOR, colors.help);
   if (help_id == NULL) help_id = null_item;
   if ((help = h_open(help_id, 1, 1, 0, 0)) == NULL) {
       sys_error("Error: Can't open help window (missing information ?)");
       return;
      }

   /*--- Get help window size ---*/
   width = w_info(help, IVIEWWIDTH);
   height = w_info(help, IVIEWHEIGHT);

   /*--- Center help window on screen and display window ---*/
   col = ((ex_crt_columns - width) / 2);
   row = ((ex_crt_rows - height) / 2);
   w_moveto(help, col, row);
   if (w_info(help, IROWS) > height) s_scrollbar(help, 1, NULL);
   w_on(help);
   rows = w_info(help, IROWS);
   do {
      key = s_getkey(&mstate);
      view_row = w_info(help, IVIEWROW);
      switch(key) {
         case  _PGUP   :
            if (view_row > height) w_viewscroll(help, SCROLL_UP, height);
            else if (view_row > 0) w_viewscroll(help, SCROLL_UP, view_row);
            break;
         case  _PGDN   :
            chg = rows - view_row - height;
            if (chg > height) w_viewscroll(help, SCROLL_DOWN, height);
            else if (chg > 0) w_viewscroll(help, SCROLL_DOWN, chg);
            break;
         case  _HOME   : w_viewscroll(help, SCROLL_UP, view_row); break;
         }
      } while (key == _PGUP || key == _PGDN || key == _HOME
              || key == MOUSE_MOVE);
   if (key != _ESC) s_ungetkey(&mstate);
   w_close(help);                                   /* Close help window */
   d_put(def);                              /* Restore original defaults */
}



/*************************************************************************
*  sys_error()
*/
void   sys_error(char *message)             /* Print system error message */
{
   Mouse_State mstate;

   sys_message(1, message);                             /* Print message */
   while (s_getkey(&mstate) == MOUSE_MOVE);      /* Pause until keypress */
   s_ungetkey(&mstate);                /* Make pressed key next key read */
   sys_message(0, NULL);                     /* Blank out message window */
}

/*************************************************************************
*  sys_color_change()
*/
void  sys_color_change(void)     /*--- Change colors used by software ---*/
{
   /*--- Set colors for the current video mode ---*/
   if (mono_display) {
      colors = diag_default.mono_colors;
      d_colorscheme(MONOCHROME_COLORS);
      }
   else if (color_display) colors = diag_default.colors;
   else {
      d_colorscheme(BLACK);
      colors = diag_default.bw_colors;
      }
   np_info.color = colors.top_wd;
   np_info.border_color = colors.border;
   np_info.title_color = colors.title;
}



/*************************************************************************
*  sys_message()
*/
void  sys_message(int center, char *message)   /* Display system message */
{
   int   old_type;

   /*--- Open window for system message (if necessary) ---*/
   if (sys == NULL) {
      old_type = d_change(WBRDTYPE, BRD_NONE); /*Change border type default*/
      if ((sys = w_open(0, ex_crt_rows-1, ex_crt_columns, 1)) == NULL) {
      /* Open borderless window */
         w_closeall();
         s_close();
         puts("MN_ERROR: Can't open message window.");
         exit(0);
         }
      d_change(WBRDTYPE, old_type);         /* Restore border type default */
      }

   /*--- Display system message ---*/
   if (message != NULL && *message == 'E') w_pencolor(sys, colors.error);
   else w_pencolor(sys, colors.message);
   w_clear(sys);
   if (message != NULL) {
      if (center) w_puts_centered(sys, 0, message);
      else w_putsat(sys, 0, 0, message);
      if (*message == 'E') putchar(0x0007);      /* Beep on error message */
      }
   w_front(sys);                      /* make sure the message is visible */
   return;
}

/*************************************************************************
*  make_menu()
*/
int make_menu(void)
{
	Window wd;
   /*-- If menus exist, delete (to change colors etc) ---*/
   if (root != NULL) m_free(root);
   if (mn_SetupCfg != NULL) m_free(mn_SetupCfg);
   if (mn_DataRate != NULL) m_free(mn_DataRate);
   if (mn_BootRom != NULL) m_free(mn_BootRom);
   if (mn_FlowCtrl != NULL) m_free(mn_FlowCtrl);
   if (mn_WOLan != NULL) m_free(mn_WOLan);
   if (mn_WOLink != NULL) m_free(mn_WOLink);
   if (loopback != NULL) m_free(loopback);
   if (pwrmanagement != NULL) m_free(pwrmanagement);
   if (view != NULL) m_free(view);
   if (setup != NULL) m_free(setup);
   if (diagnostic != NULL) m_free(diagnostic);
   if (mn_NetDiag != NULL) m_free(mn_NetDiag);

   /*--- Change menu default values ---*/
   d_change(MAUTOCLOSE, DISABLE);
   d_change(MCOLOR, colors.menu);
   d_change(MBRDCOLOR, colors.menu);
   d_change(MITEMCOLOR, colors.menu);
   d_change(MTTLCOLOR, colors.menu);
   d_change(MBARCOLOR, colors.menu_bar);
   d_change(MKEYCOLOR, colors.menu_key);
   root = menu_vert(" Main Menu ", 20, 6, main_name, main_help, main_menu_code, main_menu_fn);
   mn_SetupCfg = menu_vert(" Setup New Configuration ", 26, 10, setupcfg_name, setupcfg_help, 
	   setupcfg_menu_code, setupcfg_menu_fn);
   loopback = menu_vert(" Loopback Testing Menu ", 26, 10,loopback_name, loopback_help, 
	   loopback_menu_code, loopback_menu_fn);
   pwrmanagement = menu_vert(" Power Management Menu ", 26, 11, powermang_name, powermang_help, 
	   powermang_menu_code, powermang_menu_fn);

   /*--- Assign sub-menus to root menu items ---*/
   m_itemsub(m_getitem(root,1), mn_SetupCfg);
   m_itemsub(m_getitem(root,2), loopback);
   m_itemsub(m_getitem(root,3), pwrmanagement);
	m_itemclose(m_getitem(mn_SetupCfg,0), ENABLE);
	m_itemclose(m_getitem(mn_SetupCfg,1), ENABLE);
	m_itemclose(m_getitem(mn_SetupCfg,2), ENABLE);
	m_itemclose(m_getitem(mn_SetupCfg,3), ENABLE);
	m_itemclose(m_getitem(mn_SetupCfg,4), ENABLE);

	m_itemclose(m_getitem(pwrmanagement,0), ENABLE);
	m_itemclose(m_getitem(pwrmanagement,1), ENABLE);
	m_itemclose(m_getitem(pwrmanagement,2), ENABLE);

   /*--- Start root menu ---*/
   m_open(root);
   
m_free(root);
display_main_menu=0;

   return 0;
}

/*************************************************************************
*  menu_root()
*/
Menu  menu_root(void)                           /*--- Build root menu ---*/
{
   int  i, border;
   int  col=20, row=5, width=30, height=13;
   Menu root;
   Menu_Item item;

   border = d_change(MBRDTYPE, (int)BRD_DSDS);
   /*--- Create new menu structure ---*/
   if ((root = m_create(" Main Menu ", col, row, width, height)) == NULL)
       MN_ERROR(NULL)
   /*--- Add items to newly created menu structure ---*/
   for (i = 0; main_name[i] != NULL; i++) {
       if ((item = m_additem(root, main_name[i])) == NULL) MN_ERROR(NULL)
       /*--- Change menu item attributes ---*/
       m_itemhelpid(item, main_help[i]);  /* Assign help identifier */
      }
   d_change(MBRDTYPE, border);
   return root;
}



/*************************************************************************
*  menu_vert()
*/
Menu  menu_vert(char *menu_title, int col, int row, char **names, char **helpid,
                unsigned *codes, void (*call[])(void))
{
   /*---  Create a vertical menu at col, row. ---*/
   Menu mn;
   Menu_Item item;
   char **name;
   void (**fn)(void);
   int  height = 0, width = 0;
   /*--- Determine menu height and width ---*/
   
   name = names;
   while (*name != NULL) {
      if ((int)strlen(*name) > width) width = strlen(*name);
      height++;
      name++;
      }
   /*--- Create new menu structure ---*/
   if ((mn = m_create(menu_title, col, row, width, height)) == NULL) MN_ERROR(NULL)
   /*--- Add items to newly created menu structure ---*/
   name = names;
   fn = call;
   while (*name != NULL) {
      if ((item = m_additem(mn, *name)) == NULL) MN_ERROR(NULL)
      /*--- Change menu item attributes ---*/
      if (fn != NULL) {
         if (*fn != NULL) m_itemcall(item, *fn);  /* Assign function call */
         fn++;
         }
      if (helpid != NULL && *helpid != NULL)
         m_itemhelpid(item, *helpid++);         /* Assign help identifier */
      if (codes != NULL) {
         if (*codes != 0) m_itemcode(item, *codes);
         codes++;
         }
      name++;
      }
   return mn;
}


/*************************************************************************
*  save_default_size()
*/
void save_default_size(Window diag_wd)
{                         /*--- Save window size for use by restore ---*/
   struct diag_data *data = w_user_pointer(diag_wd, NULL);
   int vwidth = w_info(diag_wd, IVIEWWIDTH);
   int vheight = w_info(diag_wd, IVIEWHEIGHT);
   if ((vwidth > 1 && vwidth < (int)ex_crt_columns-2)
      || (vheight > 1 && vheight < (int)ex_crt_rows-4)) {
      data->default_w = vwidth;
      data->default_h = vheight;
      data->default_x = w_info(diag_wd, ISCREENCOL);
      data->default_y = w_info(diag_wd, ISCREENROW);
      }
   }


/*************************************************************************
*  maximize()
*/
void maximize(Mouse_State *mstate) 
{
   Window wd;
   int enable;
   wd = mstate->window;
   save_default_size(wd);
   /* Maximize the window and change the buttons to restore & minimize */
   enable = w_display_enable(DISABLE);
   w_moveto(wd, 1, 2);
   if (w_info(wd, ICOLS) < (int)ex_crt_columns-2)
      w_size(wd, ex_crt_columns-2, w_info(wd, IROWS));
   if (w_info(wd, IROWS) < (int)ex_crt_rows-4)
      w_size(wd, w_info(wd, ICOLS), ex_crt_rows-4);
   w_viewsize(wd, ex_crt_columns-2, ex_crt_rows-4);
   s_button(mstate->window, TITLE_MAXIMIZE_BUTTON, 0x04, -1, LEFTCLK, restore);
   s_button(mstate->window, TITLE_MINIMIZE_BUTTON, 0, -1, 0, NULL);
   w_display_enable(enable);
   }

/*************************************************************************
*  minimize()
*/
void minimize(Mouse_State *mstate)   
{
   /* Minimize the window and change the buttons to restore & none */
   save_default_size(mstate->window);
   w_viewsize(mstate->window, 1, 1);
   s_button(mstate->window, TITLE_MAXIMIZE_BUTTON, 0x04, -1, LEFTCLK, restore);
   s_button(mstate->window, TITLE_MINIMIZE_BUTTON, 0, -1, 0, NULL);
}


/*************************************************************************
*  restore()
*/
void restore(Mouse_State *mstate)   /* Restore to size and location of */
{                                   /* the last minimize or maximize   */
   Window wd = mstate->window;
   struct diag_data *data;
   int enable;
   /* Restore the window and change the buttons to maximize & minimize */
   enable = w_display_enable(DISABLE);
   w_viewsize(wd, 1, 1);
   data = w_user_pointer(wd, NULL);
   w_moveto(wd, data->default_x, data->default_y);
   w_viewsize(wd, data->default_w, data->default_h);
   w_display_enable(enable);
   s_button(mstate->window, TITLE_MAXIMIZE_BUTTON, 0x1e, -1, LEFTCLK, maximize);
   s_button(mstate->window , TITLE_MINIMIZE_BUTTON, 0x1f, -1, LEFTCLK, minimize);
}

/*************************************************************************
*  ask_save()
*/
int ask_save(Window diag_wd)        /* Ask whether to write window contents */
{
   Window box, n;
   Defaults df;
   Control button[3];
   int idx, col, width = 1000, state, oldstate;
   unsigned key;
   Mouse_State mstate;
   static char *button_names[3] = {"   Save  ", " Abandon ", " Cancel "};
   static char *diag = "DIAG Util";
   static int code[3] = {CMD_SAVE, CMD_OK, CMD_CANCEL};
   static char *msg = " has been changed";
   struct diag_data *data;

   if (diag_wd == NULL) return -1;
   data = w_user_pointer(diag_wd, NULL);
   if (data->modified == 0) return CMD_OK;
   df = d_get();
   set_box_colors();
   width = width + strlen(msg) + 2;
   if (width < 33) width = 33;
   d_change(WDISPLAY, DISABLE);
   box = w_open((ex_crt_columns-2-width)/2+1, (ex_crt_rows-4)/2, width, 4);
   if (box == NULL) return -1;
   w_print(box, "Util Prog");
   w_print(box, msg);

   for (idx = 0, col = 0; idx < 3; idx++) {
      button[idx] = c_add_button(box, button_names[idx], col, 2, code[idx], 0);
      col += (strlen(button_names[idx])+2);
      }
   key = 0; mstate.window = NULL;
   state = oldstate = 0;
   c_change(button[state], C_FOCUS, ENABLE, ENABLE);
   w_on(box);
   while (key != CMD_OK && key != CMD_CANCEL && key != CMD_SAVE) {
      if (key == ' ' || key == _ENTER) key = code[state];
      else if (toupper(key) == 'S') key = CMD_SAVE;
      else if (toupper(key) == 'A') key = CMD_OK;
      else if (toupper(key) == 'C' || key == _ESC) key = CMD_CANCEL;
      else if (key == MOUSE_LEFTCLK) {
         if (mstate.window != box) key = CMD_CANCEL;
         else key = s_getkey(&mstate);
         }
      else {
         if (key == _TAB || key == _RARROW || key == _DNARROW)
            { state++; if (state > 2) state = 0; }
         else if (key == SHIFT_TAB || key == _LARROW || key == _UPARROW)
            { state--; if (state < 0) state = 2; }
         if (state != oldstate) {
            c_change(button[oldstate], C_FOCUS, DISABLE, ENABLE);
            c_change(button[state], C_FOCUS, ENABLE, ENABLE);
            oldstate = state;
            }
         key = s_getkey(&mstate);
         }
      }
   d_put(df);
   w_close(box);
   if (key == CMD_SAVE) {
      e_write(diag_wd);
      if (! data->modified) key = CMD_OK; else key = CMD_CANCEL;
      }
   return key;
   }
   
/*************************************************************************
*  e_quit()
*/
void  e_quit(void)               
{
   struct diag_list *list;
   struct diag_data *data;
   Window diag_wd;
   list = diags;
   while (list != NULL) {            
      diag_wd = list->diag;
      data = w_user_pointer(diag_wd,NULL);
      if (data->modified) {         /*--- Ask whether to save changes ---*/
         if ((unsigned)ask_save(diag_wd) == CMD_CANCEL) return;
         }
      list = list->next;
      }
   w_closeall();
   s_close();
   exit(0);
}


/*************************************************************************
*  diag_help()
*/
void diag_help(void)
{
}

/*************************************************************************
*  diag_quit()
*/

void  diag_quit(void)               /*--- Close and exit program ---*/
{
   w_closeall();
   s_close();
   exit(0);
}



/*************************************************************************
*  ViewCurrentCfgWd()
*/
void ViewCurrentCfgWd(void)
{
	unsigned	key = 0;
	UINT16 nicAddress[3];

	char buffer[80];
	char mediumBuf[80];
	int index = 0;
	int bus;
	int dev;
	int func;
	unsigned vendorId = 0x100B;
	unsigned deviceId = 0x20;
	unsigned short IRQ1;

	Mouse_State mstate;
	Window wd;
	wd=new_diag(&mid_xl_wd_info);
	s_open();
	sys_message(0," ");
	initNIC();

	w_umessage(wd, " Current Configuration ");
	w_poscurs(wd, 0, 0);
	w_printf(wd, "Searching for LAN adapter content ...");
	ReadMACAddress(nicAddress);
	w_poscurs(wd, 0, 0);
	w_printf(wd, "                                      ");
	
	w_printf(wd, "\nEthernet Address: ........... %02X %02X %02X %02X %02X %02X",
		(unsigned char)(nicAddress[0]),
		(unsigned char)(nicAddress[0]>>8),
		(unsigned char)(nicAddress[1]),
		(unsigned char)(nicAddress[1]>>8),
		(unsigned char)(nicAddress[2]),
		(unsigned char)(nicAddress[2]>>8)
		);

	if ( inpd(RegBase+CFG) & HAS_LINK){

		if ( inpd(RegBase+CFG) & SPEED100){
			strcpy (buffer, "100 Mbps");
			strcpy (mediumBuf, "100 Mbps");
		}
		else{
			strcpy (buffer, "10 Mbps");
			strcpy (mediumBuf, "10 Mbps");
		}
	}
	else{
		strcpy (buffer, "NO LINK");
		strcpy (mediumBuf, "NO LINK");
	}

	w_printf(wd, "\nNetwork Speed: .............. %s", buffer);

	if ( inpd(RegBase+CFG) & HAS_LINK){
		if ( inpd(RegBase+CFG) & FULL_DUPLEX){
			strcpy (buffer, "ENABLED");
			strcat (mediumBuf, " FULL Duplex");
		}
		else{
			strcpy (buffer, "DISABLED");
			strcat (mediumBuf, " HALF Duplex");
		}
	}
	else{
		strcpy (buffer, "NO LINK");
		strcpy (mediumBuf, "NO LINK");
	}

	w_printf(wd, "\nFull Duplex: ................ %s ", buffer);
	w_printf(wd, "\nI/O Base: ................... %04X", RegBase);
	PciBiosFindDevice( vendorId, deviceId, index, &bus, &dev, &func );
	PciBiosReadConfigWord( bus, dev, func, 0x3C, &IRQ1 );
	IRQ1&=0XFF;
	w_printf(wd, "\nInterrupt: .................. %02X", IRQ1);

	w_printf(wd, "\nMedium Type: ................ %s", mediumBuf);
	w_printf(wd, "\nDevice/Slot Number: ......... %d", dev);
	w_printf(wd, "\nBus Number: ................. %d", bus);

	w_printf(wd, "\n\n\nThis is your current LAN adapter setting. If you would like "); 
	w_printf(wd, "to make a change to the setting. Please go back to the main ");
	w_printf(wd, "menu and select SETUP NEW CONFIGURATION.");

	w_printf(wd, "\n\n\n          Press ESC to return to main menu");

	key = k_getkey();
	while (key != _ESC)
	{
		key = s_getkey(&mstate);
	}
	s_close();
	w_close(wd);
	sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");
}

/*************************************************************************
*  SetupNewCfgWd()
*/
void SetupNewCfgWd(void)
{
}


/*************************************************************************
*  ChangeMacAddrWd()
*/
void ChangeMacAddrWd(void)
{
}

/*************************************************************************
*  DisplayEEpromWd()
*/
void DisplayEEpromWd(void)
{
	getEEPROM();
}

/*************************************************************************
*  RunEEpromTestWd()
*/
void RunEEpromTestWd(void)
{
	unsigned	key = 0;
	Mouse_State mstate;
	Window wd;
	wd=new_diag(&mid_wd_info);
	s_open();
	sys_message(0," ");

	w_printl(wd, "EEPROM Test is in progress...");
	w_printl(wd, "This may take several minutes ... ");
	if (doEeCheckSum()==SUCCESS)
	{	
		w_printl(wd, "\nDone!");
		w_printl(wd, "EEPROM of LAN adapter test successful!!  \n");
	}
	else
	{
		w_printl(wd, "\nEEPROM Checksum Failed!  ");
		w_print(wd, "\n\nIf problem persists, refer to manual for troubleshooting.");
		sys_message(1,"Error: EEPROM Checksum Failed!");
	}
	
	w_printf(wd, "\n\n\n\n            Press ESC to return to Main Menu");

	key = k_getkey();
	while (key != _ESC)
	{
		key = s_getkey(&mstate);
	}
	s_close();
	w_close(wd);
	sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");

	
}

/*************************************************************************
*  InitEEpromWd()
*/
void InitEEpromWd(void)
{
//writeEeDefaults();
findNIC();	
}

/*************************************************************************
*  MacLoobackWd()
*/
void MacLoopbackWd(void)
{
	cLoopback=1;
	NICLoopback(1);
}

/*************************************************************************
*  PhyLoopbackWd()
*/
void PhyLoopbackWd(void)
{
	cLoopback=1;
	NICLoopback(2);
}

/*************************************************************************
*  ExtLoopbackWd()
*/
void ExtLoopbackWd(void)
{
	cLoopback=1;
	NICLoopback(3);
}


/*************************************************************************
*  selMaster()
*/
void selMaster(void)
{
	Mouse_State mstate;
	Window wd;
	int x, numPacket;
	cReceive=1; cReceiveOnly=1;
	cRepeat=numPacket=e_inputNum();
	wd=new_diag(&mid_xl_wd_info);
	s_open();
	if (numPacket>0)
		initNIC( );


	for (x=0; x< numPacket; x++){
		cRepeat--;
		cCount++;
		if (x == (numPacket-1)){
			cQuit=1;
			sendPing();
			Last = receivePing();
		}
		else{
			sendPing();
			receivePing();
		}

	}
	s_close();
	w_close(wd);
	cRepeat=0; cCount=0, cXmitFailed=0;
	cQuit=0;cReceive=0; cReceiveOnly=0; cRepeat=0; cCount=0;//cMaster=0;
}

/*************************************************************************
*  LoopbackWd()
*/
void LoopbackWd(void)
{
	unsigned	key = 0;
	Mouse_State mstate;
	Window wd;
	wd=new_diag(&mid_wd_info);
	s_open();
	sys_message(0," ");

	w_printl(wd, "Loopback Test is in progress...");
	w_printl(wd, "This may take several minutes ... ");
	cLoopback=1;
	if (NICLoopback(1)==SUCCESS)
	{	
		cLoopback=1;
		if (NICLoopback(2)==SUCCESS)
		{
			w_printl(wd, "\nDone!");
			w_printl(wd, "Loopback Successful!  \n");
		}
		else
		{
		w_printl(wd, "\nLoopback Failed!  ");
		w_print(wd, "Please verify the ethernet cable is disconnected\n");
		w_print(wd, "from the LAN adapter before running the test");
		w_print(wd, "\n\nIf problem persists, refer to manual for troubleshooting.");
		sys_message(1,"Error: Loopback Failed!");
		}
	}
	else
	{
		w_printl(wd, "\nLoopback Failed!  ");
		w_print(wd, "Please verify the ethernet cable is disconnected\n");
		w_print(wd, "from the LAN adapter before running the test");
		w_print(wd, "\n\nIf problem persists, refer to manual for troubleshooting.");
		sys_message(1,"Error: Loopback Failed!");
	}
	
	w_printf(wd, "\n\n\n\n            Press ESC to return to Main Menu");

	key = k_getkey();
	while (key != _ESC)
	{
		key = s_getkey(&mstate);
	}
	s_close();
	w_close(wd);
	cLoopback=0;
	sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");

}

/*************************************************************************
*  WakeOnLanWd()
*/
void WakeOnLanWd(void)
{

	unsigned	key = 0;
	Mouse_State mstate;
	Window wd;
	wd=new_diag(&mid_xl_wd_info);
	s_open();
	sys_message(0," ");

	if (pwrMgtMode(0x200))
	{
		w_printf(wd, "Ready for Wake Event");
		w_printf(wd, "\n\n\n          Press ESC to return to main menu");
	}
	else
	{
		w_printf(wd, "Failed to clear PMCSR activitiies!  WakeOnLan Failed");
		w_printf(wd, "\n\n\n          Press ESC to return to main menu");
	}

	key = k_getkey();
	while (key != _ESC)
	{
		key = s_getkey(&mstate);
	}
	s_close();
	w_close(wd);
	sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");
}

/*************************************************************************
*  WakeOnLinkWd()
*/
void WakeOnLinkWd(void)
{

	unsigned	key = 0;
	Mouse_State mstate;
	Window wd;
	wd=new_diag(&mid_xl_wd_info);
	s_open();
	sys_message(0," ");

	if (pwrMgtMode(0x1))
	{
		w_printf(wd, "Ready for Wake Event");
		w_printf(wd, "\n\n\n          Press ESC to return to main menu");
	}
	else
	{
		w_printf(wd, "Failed to clear PMCSR activitiies!  WakeOnLink Failed");
		w_printf(wd, "\n\n\n          Press ESC to return to main menu");
	}

	key = k_getkey();
	while (key != _ESC)
	{
		key = s_getkey(&mstate);
	}
	s_close();
	w_close(wd);
	sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");
}

/*************************************************************************
*  MagicPacketWd()
*/
void MagicPacketWd(void)
{
	e_input(destAddr);
	sendMgkPkt(destAddr);
}

/*************************************************************************
*  DisplayNicWd()
*/
void DisplayNicWd(void)
{
}


/*************************************************************************
*  DisplayPCIWd()
*/
void DisplayPCIWd(void)
{
	DisplayPciRegs();
}


/*************************************************************************
*  RecTestWd()
*/
void RecTestWd(void)
{
	Mouse_State mstate;
	Window wd;
	unsigned key =0;

	int x; //int  numPacket;
	cCount=0; cKeyHit=0; cQuit=0;
	s_open();
	wd=new_diag(&mid_xl_wd_info);
	cRepeat=1;
	cReceive=1; cReceiveOnly=1;
		initNIC( );

while(!cKeyHit){

	while(!cQuit)
	{
		if ( inpd(RegBase+CFG) & HAS_LINK){
			Last = receivePing();}
		else{
				w_poscurs(wd,0,0);
				w_printl(wd,"\nNo Link!  Make sure the ethernet cable is attached");
				w_printl(wd,"to the LAN adapter, and the medium type is set properly");
				w_printl(wd,"with respect to the transmitter.");
				w_printl(wd,"\n\nPress any key to continue, or Press ESC to exit. ");
				key = k_getkey();
				if (key == _ESC)
				{
					cQuit=1; cKeyHit=1;
				}
		} // else
	}// while
	if (!cKeyHit)
		cQuit=0;
}

s_close();
w_close(wd);
	cQuit=0;cReceive=0; cReceiveOnly=0; cRepeat=0; cCount=0;//cMaster=0;
}


/*************************************************************************
*  selSlave()
*/
void selSlave(void)
{
	Window box,wd;
	int x, numPacket;
	wd=new_diag(&mid_xl_wd_info);
	box=new_diag(&bot_wd_info);
	w_poscurs(box, 0,0);
	w_printf(box, "Network diagnostic--Slave Mode Testing In Progress ....");
	initNIC( );
	cMaster=1;
	
	cRepeat=cCount=0;
	while(!cQuit)
	{
		Last = receivePing();
	}
	if (Last ==0x07){
		w_poscurs(box, 0,0);
		w_printf(box, "Network diagnostic--Slave Mode Testing Successful!!");
		delay(1);
		w_close(box);
	}
	cQuit=0;cReceive=0; cReceiveOnly=0; cRepeat=0; cCount=0;cMaster=0;
	w_close(wd);
}


/*************************************************************************
*  XmitTestWd()
*/
void XmitTestWd(void)
{
	Mouse_State mstate;
	Window wd;
	unsigned key =0;
	int x, numPacket;
	cRepeat=numPacket=e_inputNum();
	wd=new_diag(&mid_xl_wd_info);
	s_open();
	if (numPacket>0)
		initNIC( );


	for (x=0; x< numPacket; x++){
		if (kbhit())
		{
			key=k_getkey();
			if (key == _ESC)
			{
				w_printl(wd,"\n\nUser Interrupted!");
				w_printl(wd,"\nTransmit Test Packet Procedure Aborted!!");
				w_printl(wd,"\n\n\nPress any key to exit ...");
				while (s_getkey(&mstate) == MOUSE_MOVE);      
				x=numPacket;
			}
		}
		else{
		if ( inpd(RegBase+CFG) & HAS_LINK){
			cRepeat--;
			cCount++;
			sendPing();
		}
		else
		{
			w_poscurs(wd,0,0);
			w_printl(wd,"\nNo Link!  Make sure the ethernet cable is attached");
			w_printl(wd,"to the LAN adapter, and the medium type is set properly");
			w_printl(wd,"with respect to the transmitter.");
			w_printl(wd,"\n\nPress any key to continue, or Press ESC to exit. ");
			key = k_getkey();
			if (key != _ESC)
			{
			w_poscurs(wd,0,0);
			w_printl(wd,"\n                                                  ");
			w_printl(wd,"                                                       ");
			w_printl(wd,"                                ");
			w_printl(wd,"\n\n                                                 ");
			}
			else
			{
				x=numPacket;
			}

		
		}
		}
	}
	s_close();
	w_close(wd);
	cRepeat=0; cCount=0, cXmitFailed=0;
}



/*************************************************************************
*  diag_sel()
*/
void diag_sel(void)
{
}

/*************************************************************************
*  diag_cursup()
*/
void diag_cursup(void)
{
}

/*************************************************************************
*  diag_cursdown()
*/
void diag_cursdown(void)
{
}

/*************************************************************************
*  SetMedia()
*/
void setMedia( unsigned long mediaSpeed )
{
#define PHY_RST 0x00000400
unsigned long currentCfg, newCfg;

	// Read the media speed:
	currentCfg = inpd( RegBase + CFG );

	// Update the Cfg register with the desired seting and PHY reset:
	newCfg = currentCfg & 0xFFFF1FFF;
        newCfg |= mediaSpeed;
	outpd( RegBase + CFG, newCfg | PHY_RST );

	// Wait for Phy Reset and then clear the PHY_RST bit:
	delay( 1 );
	outpd( RegBase + CFG, newCfg );

}


/*************************************************************************
*  writeEeCfg()
*/
int writeEeCfg( unsigned long newCfg )
{
}


/*************************************************************************
*  SelAutoDetect()
*/
void SelAutoDetect()
{
	Window box;
	box=new_diag(&bot_wd_info);
	w_print(box,"Auto Detect Mode Selected!");
	delay(1);
	setMedia(M_AUTO);
	w_close(box);
}

/*************************************************************************
*  SelHighFull()
*/
void SelHighFull(void)
{
	Window box;
	box=new_diag(&bot_wd_info);
	w_print(box,"100Mbps Full Duplex Mode Selected!");
	delay(1);
	setMedia(M100_FULL);
	cM100_Full=1;
	w_close(box);
}

/*************************************************************************
*  SelHighHalf()
*/
void SelHighHalf(void)
{
	Window box;
	box=new_diag(&bot_wd_info);
	w_print(box,"100Mbps Half Duplex Mode Selected!");
	delay(1);
	setMedia(M100_HALF);
	cM100_Half=1;
	w_close(box);
}

/*************************************************************************
*  SelLowFull()
*/
void SelLowFull(void)
{
	Window box;
	box=new_diag(&bot_wd_info);
	w_print(box,"10Mbps Full Duplex Mode Selected!");
	delay(1);
	setMedia(M10_FULL);
	cM10_Full=1;
	w_close(box);
}

/*************************************************************************
*  SelLowHalf()
*/
void SelLowHalf(void)
{
	Window box;
	box=new_diag(&bot_wd_info);
	w_print(box,"10Mbps Half Duplex Mode Selected!");
	delay(1);
	setMedia(M10_HALF);
	cM10_Half=1;
	w_close(box);
}

/*************************************************************************
*  selBootRomEnable()
*/
void selBootRomEnable(void)
{
}

/*************************************************************************
*  selBootRomDisabled()
*/
void selBootRomDisabled(void)
{
}

/*************************************************************************
*  selFlowCtrl_NONE()
*/
void selFlowCtrl_NONE(void)
{

}

/*************************************************************************
*  selFlowCtrl_RX_ONLY()
*/
void selFlowCtrl_RX_ONLY(void)
{

}

/*************************************************************************
*  selFlowCtrl_TX_ONLY()
*/
void selFlowCtrl_TX_ONLY(void)
{
}

/*************************************************************************
*  selFlowCtrl_TX_RX()
*/
void selFlowCtrl_TX_RX(void)
{
}

/*************************************************************************
*  selWakeOnLanEnabled()
*/
void selWakeOnLanEnabled(void)
{
	Window box;
	pwrMgtMode(0x200);
	box=new_diag(&bot_wd_info);
	w_print(box,"Wake ON LAN Enabled!");
	delay(2);
	w_close(box);
}

/*************************************************************************
*  selWakeOnLanDisabled()
*/
void selWakeOnLanDisabled(void)
{
	Window box;
	pwrMgtMode(0x0);
	box=new_diag(&bot_wd_info);
	w_print(box,"Wake ON LAN Disabled!");
	delay(2);
	w_close(box);
}

/*************************************************************************
*  selWakeOnLinkEnabled()
*/
void selWakeOnLinkEnabled(void)
{
	Window box;
	pwrMgtMode(0x1);
	box=new_diag(&bot_wd_info);
	w_print(box,"Wake ON Link Enabled!");
	delay(2);
	w_close(box);
}

/*************************************************************************
*  selWakeOnLinkDisable()
*/
void selWakeOnLinkDisabled(void)
{
	Window box;
	pwrMgtMode(0x0);
	box=new_diag(&bot_wd_info);
	w_print(box,"Wake ON Link Enabled!");
	delay(2);
	w_close(box);
}



/*************************************************************************
*  main()
*/
main() 
{
	char      buffer[80];
	int hasNIC=0;

	void		sys_help();		/* Check to see if help file can be created */
	unsigned	key = 0;
	Mouse_State	mstate;
	Window		wd;
	
	d_change(SYSBORDER, DISABLE);         /* Draw intuitive window borders */
	d_change(SYSBORDCOLOR, ENABLE);       /* Extend title color on border */
	d_change(SYSBACKGROUND, 0xb0);         /* Screen background character */
	d_change(SYSBLINK, DISABLE);              /* Bright background colors */
	help_status = h_loadfile("diag.hlp");            /* Load help text */

	hasNIC = findNIC();					/* check to see if NIC is installed */
	init_test=0;						/* done with initial test */

	h_call(sys_help);     /* Designate sys_help function as help function */
	d_change(WDISPLAY, DISABLE);
	wd = w_open(1, 1, 1, 1);         /* Determine display characteristics */
	d_change(WDISPLAY, ENABLE);
	if (display_type == 1 || display_type == 6 || display_type == 7
       || display_type == 8) mono_display = 1;
	else {
      if (ex_colordisplay == 1) color_display = 1;
	}								/*--- Check for black and white display ---*/

	sys_color_change();                                     /* Set colors */
	d_change(SYSBACKGROUND, 0xb0);         /* Screen background character */
	sys_message(1, "         Space Bar = Continue    ESC = Exit to DOS       ");
	w_close(wd);



	diag=new_diag(&top_wd_info);
	w_shadowchange(diag, SHADOW_BOTTOM+SHADOW_RIGHT, COLOR(WHITE,BLACK),SHADOW_FG+SHADOW_BG, 0,0,0);
	w_print(diag, " DP83815--LAN Adapter Setup And Diagnostics Version 2.01, 2000");
	diag=new_diag(&mid_wd_info);
	w_print(diag, "                 DP83815  Version 2.01       \n");
	w_print(diag, "      Ethernet PCI LAN Adaptor Setup and Diagnostics \n\n");
	w_print(diag, "                 National Semiconductor            \n");
	w_print(diag, "                  (c) Copyright, 2000              \n\n\n\n");
	w_print(diag, "    If any network driver installed, press ESC to exit !! \n\n");
	w_print(diag, "               < Press Space Bar to Continue >           \n\n\n\n");	

	key = k_getkey();

	while (key != ' ')
	{

		if (key == _ESC)
			e_quit();
		if (key == ' ')
			w_close(diag);
	key = s_getkey(&mstate);
	}

	s_open();
	if (!hasNIC)
	{
	diag=new_diag(&error_info);
	w_print(diag, "\n\n             DP83815--LAN adapter NOT FOUND !!\n\n\n");
	w_print(diag, "   Please press any key to exit and be sure to have the\n");
	w_print(diag, " Correct LAN adapter installed before running the program.\n");
	w_print(diag, "             ");
	sys_error("Error:  LAN adapter not found!      ");
	while (s_getkey(&mstate) == MOUSE_MOVE);      /* Pause until keypress */
		e_quit();
	}
	else
	{

	sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");
	w_close(diag);

	make_menu();
	if (!display_main_menu)
	{
		e_quit();
	}

	
   while (key != ALT_F4) 
   {
      if (diag == NULL)
	  {
         key = s_getkey(&mstate);
         if (key == ALT_F4) e_quit();
      }
      else 
	  {
      if (mouse_on) 
	  {
         key = s_getkey(&mstate);                      /* Get keypress */
         if (ISKEYBOARD(key)) sys_mouseoff();
      }
      else 
	  {
         key = s_getclick(&mstate);
         if (ISMOUSE(key)) 
		 {
            sys_mouseon();
            s_ungetkey(&mstate);
            key = s_getkey(&mstate);
          }
		 switch (key){
		 case _F1	: 
			 e_help();		
			 break;
		 case ALT_F4: 
			 sys_mouseon();
			 e_quit();
			 key=0;
			 break;
		 }
			
      }
	  }

	}
	}

	s_close();
	w_closeall();
	return 0;
}

