/***********************************************************************
*									
* EEPROM.C                                                     
*									
* This is a collection of routines to interface to the EEPROM
*  for the MacPhyter Chip
*
***********************************************************************/

// Include files

#include <dos.h>
#include <time.h>
#include <sys\timeb.h>
#include <stdio.h>
#include <c:\my\msvc15\include\conio.h>
#include <memory.h>
#include <time.h>
#include "GLOBAL.H"
#include "NICSTUFF.H"
#include "PCIBIOS.H"

#include "winlib.h"
#include "wgraphic.h"

// Function prototypes

#include "FUNCTION.H"

// Globals
extern unsigned short RegBase;

extern Window  new_diag(struct info *np);            /* Create a new diag window */
extern struct    info  mid_wd_info;

/************************************************
*	Function:   DisplayEEPROM
*	Purpose:	display EEPROM contents
*	Returns:    void
*************************************************/
void getEEPROM( )
{
	int i=0;
	int row=0;
	WORD value;
	WORD next_value;

	unsigned	key = 0;
	Mouse_State mstate;
	Window wd;
	w_close(wd);
	wd=new_diag(&mid_wd_info);
	s_open();
	sys_message(0,"");


	
	w_printl(wd, "     Display EEPROM Contents   \n");
	for( i=0; i<0xC; i++ )
	{
		ReadEEWord( RegBase + MEAR, i, &value );
//		ReadEEWord( RegBase + MEAR, i+1, &next_value );
        w_printf(wd, "%01X= %04X        ", i, value);
		
//        w_printf(wd, "\n%01X= %04X                  %01X= %04X  ", i, value, i+1, next_value);
    }
	
	w_printl(wd, "\n\n\n\n            Press ESC to return to Main Menu");
	key = k_getkey();
	while (key != _ESC)
	{
		key = s_getkey(&mstate);
	}
	s_close();
	w_close(wd);
	sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");

}

/************************************************
*       Function:   ReadMACAddress
*       Purpose:    load readable form of the MAC address
*	Returns:    void
*************************************************/
void ReadMACAddress(  UINT16 *nicAddress )
{
UINT32 i;
UINT16 mask;
UINT16 word1 = 0;
UINT16 word2 = 0;
UINT16 word3 = 0;
UINT16 word4 = 0;

    //
    // Read 16 bit words 6 - 9 from the EEProm.  They contain the hardwares MAC
    // address in a rather cryptic format.
    //
    ReadEEWord( RegBase + MEAR, 0x06, &word1 );
    ReadEEWord( RegBase + MEAR, 0x07, &word2 );
    ReadEEWord( RegBase + MEAR, 0x08, &word3 );
    ReadEEWord( RegBase + MEAR, 0x09, &word4 );

    //
    // Decode the cryptic format into what we can use a word at a time.
    //
    nicAddress[ 0 ] = word1 & 1;
    nicAddress[ 1 ] = word2 & 1;
    nicAddress[ 2 ] = word3 & 1;

    i = 15;
    mask = 0x2;
    while ( i-- ) {
       if ( word2 & 0x8000 ) {
          nicAddress[ 0 ] |= mask;
       }
       word2 = word2 << 1;
       mask = mask << 1;
    }

    i = 15;
    mask = 0x2;
    while ( i-- ) {
       if ( word3 & 0x8000 ) {
          nicAddress[ 1 ] |= mask;
       }
       word3 = word3 << 1;
       mask = mask << 1;
    }

    i = 15;
    mask = 0x2;
    while ( i-- ) {
       if ( word4 & 0x8000 ) {
          nicAddress[ 2 ] |= mask;
       }
       word4 = word4 << 1;
       mask = mask << 1;
    }
 }
 
/************************************************
*       Function:   WriteMACAddress
*       Purpose:    Write readable form of the MAC address to EEPROM
*	Returns:    void
*************************************************/
int writeMacAddress( unsigned *nicAddress )
{
unsigned char csLow, csHigh, csSum, csVals[ 11 ];
UINT32      mask;
UINT32      l;
int         nbits = 9;
unsigned    i, value, sixOff, nineOff, checkSum = 0, word1, word2, word3;
unsigned    sword1, sword2, sword3, sword4;


    ReadEEWord( RegBase + MEAR, 0x06, &sixOff );
	sixOff &= ~1;

    ReadEEWord( RegBase + MEAR, 0x09, &nineOff );
	nineOff &= 1;

    word1 = nicAddress[ 0 ];
    word2 = nicAddress[ 1 ];
    word3 = nicAddress[ 2 ];
    
    writeEeEnable();
	
	value = sixOff;
	if( word1 & 1 ) value |= 1;
    sword1 = value;
    WriteEEWord( RegBase + MEAR, 0x06, value );
    
	value = 0;
    l = 15;
    mask = 0x2;
	while( l-- )
		{
		if( word1 & 0x8000 ) value |= mask;
		mask <<= 1;
		word1 <<= 1;
    	}
    
    if( word2 & 1 ) value |= 1;
    sword2 = value;
    WriteEEWord( RegBase + MEAR, 0x07, value );

	value = 0;
    l = 15;
    mask = 0x2;
	while( l-- )
		{
		if( word2 & 0x8000 ) value |= mask;
		mask <<= 1;
		word2 <<= 1;
    	}
    
    if( word3 & 1 ) value |= 1;
    sword3 = value;
    WriteEEWord( RegBase + MEAR, 0x08, value );

    value = nineOff;
    l = 15;
    mask = 0x2;
	while( l-- )
		{
		if( word3 & 0x8000 ) value |= mask;
		mask <<= 1;
		word3 <<= 1;
    	}
    
    sword4 = value;
    WriteEEWord( RegBase + MEAR, 0x09, value );
    
    writeEeDisable();
	
    // Calculate Checksums:
    for( l=0; l<11; l++ )
    {
    	ReadEEWord( RegBase + MEAR, l, &value );
    	    
    	csLow = value & 0xff;
		value >>= 8;
    	csHigh = value & 0xff;
    	
    	csVals[ l ] = csLow + csHigh;
    }
    
    // Calculate Checksums:
    csSum =  0;
    for( l=0; l<11; l++ )
    	{
        csSum += csVals[ l ];
        }
	
    csSum += 0x55;
    checkSum = (~csSum) + 1;
    checkSum = ( checkSum << 8 ) + 0x55;

    delay( 10 );

    writeEeEnable();

    WriteEEWord( RegBase + MEAR, 0x0B, checkSum );

    writeEeDisable();
	
    ReadEEWord( RegBase + MEAR, 0xB, &value );
   	
    if( value != checkSum ) 
       {
       // printf( "\n!!WARNING Checksum not programmed correctly!\n" );
       // printf( "\n  checkSUM: %04X  actual: %04X", checkSum, value );
	   return( ERROR );
       }
        
    return( checkSumCheck(  ) );
}

void writeID( int ssId, BYTE venDev )
{
int idOffset = 0;
unsigned char csLow, csHigh, csSum, csVals[ 11 ];
UINT32      l;
unsigned    value, checkSum = 0;

	// Calculate the offset:
	if( venDev == 'D' ) idOffset = 1;

    writeEeEnable();
	
    WriteEEWord( RegBase + MEAR, idOffset, ssId );

    writeEeDisable();

    // Calculate Checksums:
    for( l=0; l<11; l++ )
    {
    	ReadEEWord( RegBase + MEAR, l, &value );
    	    
    	csLow = value & 0xff;
		value >>= 8;
    	csHigh = value & 0xff;
    	
    	csVals[ l ] = csLow + csHigh;
    }
    
    // Calculate Checksums:
    csSum =  0;
    for( l=0; l<11; l++ )
    	{
        csSum += csVals[ l ];
        }
	
    csSum += 0x55;
    checkSum = (~csSum) + 1;
    checkSum = ( checkSum << 8 ) + 0x55;

    delay( 10 );

    writeEeEnable();

    WriteEEWord( RegBase + MEAR, 0x0B, checkSum );

    writeEeDisable();
	
    ReadEEWord( RegBase + MEAR, 0xB, &value );
   	
    if( value != checkSum ) 
       {
       printf( "\n!!WARNING Checksum not programmed correctly!\n" );
       printf( "\n  checkSUM: %04X  actual: %04X", checkSum, value );
       }
        
    checkSumCheck(  );
}

/************************************************
*       Function:   WriteEEWord
*       Purpose:    write a 16-bit value into the specified eeprom location
*	Returns:    void
*************************************************/
void WriteEEWord( unsigned short pMear, UINT32 wordOffset, UINT16 value )
{
UINT32      mask;
UINT32      l;
int			nbits = 9;
UINT32 invalue = (UINT32) value;

    outpd( RegBase + MEAR, 0 );           /* clock out CS low */
    mdelay( 5 );
 
    outpd( RegBase + MEAR, EECLK );
    mdelay( 5 );

    invalue = ((UINT32)( 0x0140|wordOffset) << 16 ) | invalue;
    
    nbits = 25;

    mask = ((UINT32)1) << (nbits-1);

    while( nbits-- ) {
        /* assert chip select, and setup data in */
        l = ( invalue & mask ) ? EECS | EEDI : EECS;

        outpd( RegBase + MEAR, l );
	    mdelay( 5 );
        
        outpd( RegBase + MEAR, l | EECLK );
        mdelay( 5 );
    
        mask >>= 1;
    }

    // IOW32( mear, 0 );           /* terminate write  */
    outpd( RegBase + MEAR, 0 );               // terminate operation
    mdelay( 5 );
 
    /* wait for operation to complete */
    outpd( RegBase + MEAR, EECS );        /* assert CS */
    mdelay( 5 );
 
    outpd( RegBase + MEAR, EECS | EECLK );
    mdelay( 5 );
 
    for( l = 0; l < 10; l++ ) {
        mdelay( 5 );
        if( inpd( RegBase + MEAR ) & EEDO )
            break;
    }
    
    outpd( RegBase + MEAR, 0 );           /* clock out CS low */
    mdelay( 5 );
 
    outpd( RegBase + MEAR, EECLK );
    mdelay( 5 );

}

/************************************************
*       Function:   checkSumCheck
*       Purpose:    Use the EEBIST bit on MacPhyter to
*                   validate the EEPROM checksum
*	Returns:    void
*************************************************/
int checkSumCheck(  )
{
unsigned long pciTestReg = EEBIST_EN;
	
	outpd( RegBase + PCITEST_CNTRL, EEBIST_EN );
	
	while( pciTestReg & EEBIST_EN )
	{
	pciTestReg = inpd( RegBase + PCITEST_CNTRL );
	}
	
	if( inpd( RegBase + PCITEST_CNTRL ) & EEBIST_FAIL )
	{
		// printf( "\n!!WARNING:  Checksum FAILED!\n" );
		return( ERROR );
	}
		// else printf( "\nEEBIST Test Passes!\n" );
		
	return( SUCCESS );
}

/************************************************
*       Function:   ReadEEWord
*       Purpose:    Read 16-bit value from the specified location
*	Returns:    void
*************************************************/
void ReadEEWord( unsigned short pMear, UINT32 offset, UINT16* pEeData )
{
UINT16      value;

    outpd( RegBase + MEAR, 0 );
    mdelay( 5 );
 
    outpd( RegBase + MEAR, EECLK );       // clock out no chipselect
    mdelay( 5 );
 
    
    eeput( (UINT32)(EEread|offset), 9 );
    
    value = eeget( 16 );
    
    outpd( RegBase + MEAR, 0 );           // terminate read 
    mdelay( 5 );
 
    outpd( RegBase + MEAR, EECLK );       // clock out no chipselect
    mdelay( 5 );
 
    *pEeData = ( UINT16 )value;

    outpd( RegBase + MEAR, 0 );           // terminate read 
    mdelay( 5 );
 
//    return( value );
}

/************************************************
*       Function:   EEPUT
*       Purpose:    Clock OUT data to EEPROM
*	Returns:    void
*************************************************/
void eeput( UINT32 value, int nbits )
{
UINT32      mask = ((UINT32)1) << (nbits-1);
UINT32      l;

    while( nbits-- ) {
        // assert chip select, and setup data in
        l = ( value & mask ) ? EECS | EEDI : EECS;
        outpd( RegBase + MEAR, l );
        mdelay( 5 );
 
        outpd( RegBase + MEAR, l | EECLK );
        mdelay( 5 );
 
        mask >>= 1;
    }
}

/************************************************
*       Function:   EEGET
*       Purpose:    Clock IN data from EEPROM
*	Returns:    void
*************************************************/
UINT16 eeget( int nbits )
{
UINT16      mask = 1 << (nbits-1);
UINT16      value = 0;

    while( nbits-- ) {
        // assert chip select, and clock in data
        outpd( RegBase + MEAR, EECS );
        mdelay( 5 );
 
        outpd( RegBase + MEAR, EECS | EECLK );
	    mdelay( 5 );
 
        if( inpd( RegBase + MEAR ) & EEDO )
                value |= mask;
	    mdelay( 5 );
    
        mask >>= 1;
    }
    
    return( value );
}

/************************************************
*       Function:   WriteEeEnable
*       Purpose:    ENABLE EEPROM Write Mode
*	Returns:    void
*************************************************/
void writeEeEnable()
{
UINT32      mask;
UINT32      l;
int			nbits = 9;
    //
    // Start the sequence with Chip Select 0 for a 4 microsecond cycle.
    //
    outpd( RegBase + MEAR, 0 );
    mdelay( 5 );

    outpd( RegBase + MEAR, EECLK );
    mdelay( 5 );

    mask = ((UINT32)1) << (nbits-1);

    while( nbits-- ) {
        /* assert chip select, and setup data in */
        l = ( 0x0130 & mask ) ? EECS | EEDI : EECS;
        outpd( RegBase + MEAR, l );
        outpd( RegBase + MEAR, l | EECLK );
        mask >>= 1;
    }

    outpd( RegBase + MEAR, 0 );               // terminate operation
    mdelay( 5 );

    //IOW32( mear, 0 );           /* clock out CS low */
    outpd( RegBase + MEAR, EECLK );
    mdelay( 5 );
}

/************************************************
*       Function:   WriteEEDisable
*       Purpose:    Disable EEPROM write Mode
*	Returns:    void
*************************************************/
void writeEeDisable()
{
UINT32      mask;
UINT32      l;
int			nbits = 9;

    //ed->eeput( (uint32)EEwriteDisable, 9 );
    nbits = 9;
    mask = ((UINT32)1) << (nbits-1);

    while( nbits-- ) {
        /* assert chip select, and setup data in */
        l = ( 0x0100 & mask ) ? EECS | EEDI : EECS;
        outpd( RegBase + MEAR, l );
        outpd( RegBase + MEAR, l | EECLK );
        mask >>= 1;
    }

    outpd( RegBase + MEAR, 0 );               // terminate operation
}

/************************************************
*       Function:   doEeCheckSum()
*       Purpose:    Read EEPROM contents, calculate checksum,
*                   write it, verify the write and validate it.
*	Returns:    void
*************************************************/
int doEeCheckSum()
{
int l;
WORD value, checkSum;
unsigned char csLow, csHigh, csSum, csVals[ 11 ];

    // Calculate Checksums:
    for( l=0; l<11; l++ )
    {
    	ReadEEWord( RegBase + MEAR, l, &value );
    	    
    	csLow = value & 0xff;
		value >>= 8;
    	csHigh = value & 0xff;
    	
    	csVals[ l ] = csLow + csHigh;
    }
    
    // Calculate Checksums:
    csSum =  0;
    for( l=0; l<11; l++ )
    	{
        csSum += csVals[ l ];
        }
	
    csSum += 0x55;
    checkSum = (~csSum) + 1;
    checkSum = ( checkSum << 8 ) + 0x55;

    delay( 10 );

    writeEeEnable();

    WriteEEWord( RegBase + MEAR, 0x0B, checkSum );

    writeEeDisable();
	
    ReadEEWord( RegBase + MEAR, 0xB, &value );
   	
    if( value != checkSum ) 
        {
        // printf( "\n!!WARNING Checksum not programmed correctly!\n" );
        // printf( "\n  checkSUM: %04X  actual: %04X", checkSum, value );
		return( ERROR );
        }
        
    return( checkSumCheck(  ) );
}

/*****************************************************
*       Function:   WriteEeDefaults:
*       Purpose:    Initialize the EEPROM without touching the 
*                   MAC address.
*
*	Returns:    void
*       
*       The Following was taken from EPMON Programming script:
*       
*       eeprom  0       d008            # subsystem vendor
*       eeprom  1       0400            # subsys id
*       eeprom  2       2cd0            # min gnt, max lat
*       eeprom  3       cf82            # config stuff
*       eeprom  4       0000            # secure on pw stuff
*       eeprom  5       0000            # secure on pw stuff
*       eeprom  6       0000            # LSb is part of Ethernet ID 
*       eeprom  7       2001            # Ethernet ID
*       eeprom  8       d1a1            # Ethernet ID
*       eeprom  9       $Loc9           # Ethernet ID
*       eeprom  a       a098
*       eeprom  b       $value          # checksum
********************************************************************/
int writeEeDefaults()
{
UINT16 Addressbit;

    // initNIC will display the current MAC address:
    if( initNIC( ) == ERROR ) return( ERROR );

    writeEeEnable();

    // The following values taken from the above script:
    WriteEEWord( RegBase + MEAR, 0x0, 0xd008 );
    WriteEEWord( RegBase + MEAR, 0x1, 0x0400 );
    WriteEEWord( RegBase + MEAR, 0x2, 0x2cd0 );
    WriteEEWord( RegBase + MEAR, 0x3, 0xcf82 );
    WriteEEWord( RegBase + MEAR, 0x4, 0x0000 );
    WriteEEWord( RegBase + MEAR, 0x5, 0x0000 );
    
    // Save the least significant bit for the MACaddress:
    ReadEEWord( RegBase + MEAR, 0x6, &Addressbit );
    Addressbit &= 1;
    WriteEEWord( RegBase + MEAR, 0x6, Addressbit );

    // Clear the least significant bit for the MACaddress:
    ReadEEWord( RegBase + MEAR, 0x9, &Addressbit );
    Addressbit &= 0xfffe;
    WriteEEWord( RegBase + MEAR, 0x9, Addressbit );

    WriteEEWord( RegBase + MEAR, 0xA, 0xa098 );

    writeEeDisable();
    
    return( doEeCheckSum() );
}
