/***********************************************************************
*									
* NICSTUFF.C								
*									
* This is a collection of routines that are intended to make it easy	
* to communicate with any piece of hardware that is based on a
* National Semiconductor Ethernet chip with the MacPhyter core. 
*
* This includes all routines necessary to setup the card, send	
* packets, receive packets, perform loopback test and
* others.
*
***********************************************************************/
// Include files

#include <dos.h>
#include <time.h>
#include <sys\timeb.h>
#include <stdio.h>
#include <c:\my\msvc15\include\conio.h>
#include <memory.h>
#include <time.h>
#include "GLOBAL.H"
#include "NICSTUFF.H"
#include "PCIBIOS.H"
#include "FUNCTION.H"

#include "winlib.h"
#include "wgraphic.h"

// External Function Prototypes

// Global Variables
#define M10_HALF			0x00000000
#define M100_HALF			0x00004000
#define M10_FULL			0x00008000
#define M100_FULL			0x0000C000
#define M_AUTO				0x0000E000

#define PMESTS				0x00008000
#define PMEEN				0x00000100

extern WORD  SharedAddress;
extern unsigned short RegBase;
extern unsigned short IRQ;
extern char cRepeat, cCount, cXmitFailed, cKeyHit;
extern unsigned long PciCfgSpace[ PCI_TABLE_SIZE ]; 
extern BYTE  MemMode;
extern BYTE  DataWidth;
extern BYTE  NextPacket;
extern BYTE  PacketSize;
extern char  	cReceiveOnly,cQuit, cTransmit, cLoopback, cReturn, cReceive, cMaster;
extern char		cM100_Full, cM100_Half, cM10_Full, cM10_Half;
extern Window  new_diag(struct info *np);            /* Create a new diag window */
extern struct    info  mid_wd_info;
extern struct    info  mid_xl_wd_info;
extern struct    info  bot_wd_info;
extern int init_test;
extern void setMedia( unsigned long mediaSpeed );

unsigned long IsrRegister = 0;
BYTE Source[]={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
BYTE Destination[]={0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
BYTE LoopbackPacket[128],PingPacket[128];
BYTE Header[4];

#define DESCRIPTOR_POOL_SIZE 10

HALSpace_t RxDescriptorPool[ DESCRIPTOR_POOL_SIZE ];
int RxDescriptorPoolIdx = 0;

HALSpace_t tempDesc;

etherPacket_t RxDataPool[ DESCRIPTOR_POOL_SIZE ];

#define REGISTER_TABLE_SIZE 39
RegisterTable_t RegisterTable[ REGISTER_TABLE_SIZE ] =
        {
        "CR     ", 0,    "CFG    ", 4,    "MEAR   ", 8,    "PTSCR  ", 0xC,
        "ISR    ", 0x10, "IMR    ", 0x14, "IER    ", 0x18, "TxDP   ", 0x20,
        "TxCFG  ", 0x24, "RxDP   ", 0x30, "RxCFG  ", 0x34, "WCSR   ", 0x40,
        "PCR    ", 0x44, "RFCR   ", 0x48, "RFDR   ", 0x4C, "BRAR   ", 0x50,
        "BRDR   ", 0x54, "SRR    ", 0x58, "MIBC   ", 0x5C, "RxErPkt", 0x60,
        "RxFcsEr", 0x64, "RxMsdEr", 0x68, "RxFaErr", 0x6C, "RxSymEr", 0x70,
        "RxFrLng", 0x74, "TxSqeEr", 0x78,

        "BMCR   ", 0x80, "BMSR   ", 0x84, "PHYIDR1", 0x88, "PHYIDR2", 0x8C,
        "ANAR   ", 0x90, "ANLPAR ", 0x94, "ANER   ", 0x98, "ANNPTR ", 0x9C,
        "PHYSTS ", 0xC0, "FCSCR  ", 0xD0, "RECR   ", 0xD4, "PHYCR  ", 0xE4,
        "10BTSCR", 0xE8 
        };

RegisterTable_t PciTable[ PCI_TABLE_SIZE ] =
        {
        "CFGID  ", 0x00, "CFGCS  ", 0x04, "CFGRID ", 0x08, "CFGLAT ", 0x0C,
        "CFGIOA ", 0x10, "CFGMA  ", 0x14, "CFGSID ", 0x2C, "CFGROM ", 0x30,
        "CAPPTR ", 0x34, "CFGINT ", 0x3C, "PMCAP  ", 0x40, "PMCS   ", 0x44
        };

BYTE Packet[ MAX_ETHER_PACKET_SIZE ];

MACAddr_t MACAddr = { '0', '0', '0', '0', '0', '0' };

/************************************************
*       Function:   findNIC
*       Purpose:    Find an instance of MacPhyter in the system.
*       Returns:    1 if FAIL, 0 if Succees
*************************************************/
int findNIC()
{
char inputString[ 25 ], ctemp1;
UINT16 nicAddress[ 3 ];
unsigned vendorId = 0x100B, value, pwrCaps, EEpwr; 
unsigned long dvalue;
unsigned deviceId = 0x20;
int i, index = 0;
int bus;
int dev;
int func;

	// Find the card on the PCI Bus:
	if( PciBiosFindDevice( vendorId, deviceId, index, &bus, &dev, &func ) )
		{
		return( ERROR );
		}

    PciBiosReadConfigWord( bus, dev, func, PciCfgBaseAddr0, &RegBase );

    if( RegBase & 0x01 )
             {
             RegBase = RegBase & ~0x03;
             }

    for( i=0x0; i<PCI_TABLE_SIZE; i++ )
        {
        PciCfgSpace[ i ] = 0;
	    PciBiosReadConfigWord( bus, dev, func, PciTable[ i ].regOffset, &(WORD)PciCfgSpace[ i ]  );
	    PciBiosReadConfigWord( bus, dev, func, PciTable[ i ].regOffset + 2, &value );
	    dvalue = (UINT32)value;
	    dvalue <<= 16;
	    PciCfgSpace[ i ] |= dvalue;
	    }

    // Read configuration vitals:
    ReadMACAddress(  nicAddress );
    PciBiosReadConfigWord( bus, dev, func, 0x4, &value );
	PciBiosReadConfigWord( bus, dev, func, 0x46, &pwrCaps );
    ReadEEWord( RegBase + MEAR, 0x03, &EEpwr );
    return( SUCCESS );
}

/************************************************
*       Function:       DisplayNICRegs
* 	Purpose:	Displays the contents of the ST-NIC registers
*	Input:		none
*   _output:		none
*************************************************/
void DisplayNICRegs()
{
int i=0;

        for( i=0; i < REGISTER_TABLE_SIZE; i++ )
                {
                printf
                        (
                        "\n%s: %08lX",
                        RegisterTable[ i ].regName,
                        inpd( RegBase + RegisterTable[ i ].regOffset )
                        );
                i++;

                if( i < REGISTER_TABLE_SIZE )
                printf
                        (
                        "   %s: %08lX",
                        RegisterTable[ i ].regName,
                        inpd( RegBase + RegisterTable[ i ].regOffset )
                        );
                i++;

                if( i < REGISTER_TABLE_SIZE )
                printf
                        (
                        "   %s: %08lX",
                        RegisterTable[ i ].regName,
                        inpd( RegBase + RegisterTable[ i ].regOffset )
                        );
                }

}
	
/************************************************
*	Function:   resetNIC
*	Purpose:	Performs a hardware reset of the NIC.
*	Arguments:  Port - the RESET port
*	Returns:    nothing
*************************************************/
#define mibACLR         0x00000004      /* clear all counters */

void resetNIC( unsigned long value )
{
    outpd( RegBase + INT_ENABLE, 0 );
    
    outpd( RegBase + INT_MASK, 0 );
    
    outpd( RegBase + RX_FILTER_CTRL, 0 );
    
    outpd( RegBase + MIB_CTRL, mibACLR  );
    
    outpd( RegBase, value );
    
	outpd( RegBase + CFG, 0x8 ); //PESEL bit.
    
			delay( 1 );
}

/************************************************
*	Function:   initNIC
*	Purpose:	Performs preliminary initialization
*				of NIC chip.  Performs hardware reset
*				and then sets up registers, based
*				on given arguments
*	Arguments:  UseSend - set if program to use Send Packet
*				RCRMask - sets packet types to receive
*	Returns:    nothing
*************************************************/
int initNIC( )
{
int i;

	Window wd;
	wd=new_diag(&mid_xl_wd_info);
	sys_message(0,"");
	w_print(wd, "LAN Adapter Is Initializing ...");
	
	if (init_test)
		if( findNIC() == ERROR ) return ERROR;
	w_print(wd, ".");

	resetNIC( RESET );							// Reset the NIC

    outpd( RegBase + PCITEST_CNTRL, 0x04 );     // Download eeprom defaults
	delay( 1 );
	if (cM100_Full){
		cM100_Full=0;
		setMedia(M100_FULL);
	}
	if (cM100_Half){
		cM100_Half=0;
		setMedia(M100_HALF);
	}
	if (cM10_Full){
		cM10_Full=0;
		setMedia(M10_FULL);
	}
	if (cM10_Half){
		cM10_Half=0;
		setMedia(M10_HALF);
	}



	w_print(wd, ".");
    outpd( RegBase + PHY_BMCR, BMCR_RESET );    // Reset the PHY
	delay( 1 );
	w_print(wd, ".");

    // Read the MAC Address and display the contents:
    outpd( RegBase + TX_CONFIG, 0x10C00230 );   // Config the transmit register. 
	delay( 1 );
	w_print(wd, ".");

    /* Receive Drain Threshold NSC recomended value */
    outpd( RegBase + RX_CONFIG, 0x00500008 );
	delay( 1 );

	// Receive Filter:
    outpd( RegBase + RX_FILTER_CTRL, RX_FILTER_CTRL_RFEN |\
                           RX_FILTER_CTRL_AAB  |\
                           RX_FILTER_CTRL_AAM  |\
                           RX_FILTER_CTRL_APM );

	/* Phy register initialization:*/
    outpd( ( RegBase + PHY_PAGESEL  ),  0x0001 );
    outpd( ( RegBase + PHY_PMDCSR   ),  0x189c );
    outpd( ( RegBase + PHY_TSTDAT   ),  0x0000 );
    outpd( ( RegBase + PHY_CCDSPCFG ),  0x5040 );
    outpd( ( RegBase + PHY_CCSDCFG  ),  0x008c );
    outpd( ( RegBase + PHY_PAGESEL  ),       0 );
    RxDescriptorPoolIdx = 0;

	// Initialize the memory block:
    for( i=0; i<DESCRIPTOR_POOL_SIZE; i++ )
	{
    /* Clear the ScratchSpace before starting to use it */
    memset( &RxDescriptorPool[ i ], '\0', sizeof( HALSpace_t ) );

    //Physical dataBuffer Pointer:
    RxDescriptorPool[ i ].physPointer = 
    	DosLogicalToPhysical( (void *)RxDataPool[ i ] );

    //Physical nextDescriptor Pointer:
    if( i+1 < DESCRIPTOR_POOL_SIZE ) 
    	RxDescriptorPool[ i ].linkPhyAddrDesc = 
    		DosLogicalToPhysical( &RxDescriptorPool[ i+1 ] );

    RxDescriptorPool[ i ].cmdStatus.rxStatus.descriptorSize = sizeof( etherPacket_t );
    }
	
	// Clear the MIB counters:
    outpd( RegBase + MIB_CTRL, MIB_CTRL_ACLR );

	// sET THE INTERRUPT mASK REGISTER:
    outpd( RegBase + INT_MASK,  INT_DPERR |
                		INT_SSERR         |
                		INT_RMABT         |
                		INT_RTABT         |
                		INT_RXSOVR        |
                		INT_TXERR         |
                		INT_TXOK          |
                		INT_RXERR         |
                		INT_RXOK          |
                		INT_RXORN );


    // Write RX DP:
    outpd( RegBase + RX_DP, DosLogicalToPhysical( &RxDescriptorPool[ 0 ] ) );

    // Enable Rcv:
    outpd( RegBase, COMMAND_RXENABLE );
	w_close(wd);

return SUCCESS;
}


/************************************************
*	Function:NICLoopback
*	Purpose:	Performs loopback testing on the NIC
*				controller.  The mode is passed to 
*				determine what type of loopback is to 
*				be performed:
*					1:	Internal MAC loopback
*					2:	ENDEC loopback
*					3:	External Transceiver loopback
*	Arguments:  Mode - the mode of loopback to perform
*	Returns:    ERROR/SUCCESS
*************************************************/
int NICLoopback( int Mode )
{
	WORD  Count=64;
	int   y=0;
    unsigned long tempReg = 0;
	unsigned	key = 0;
	Mouse_State mstate;
	Window wd;
	w_close(wd);
	wd=new_diag(&mid_xl_wd_info);
	s_open();
	
	sys_message(0,"");

	initNIC( );

    for( y=0; y<6; y++ ) Destination[ y ] = 0xFF;

	// If in the MAC loopback mode:
	if( Mode == 1 )
		{
		w_printl(wd, "MAC Loopback Testing In Progress ...   ");
		w_printl(wd, "This may take several minutes ... ");
		// To Enable loopback: 1st disable Xmit/Rcv:
                outpd( RegBase + CR, 0x08);
					delay( 1 );
                outpd( RegBase + CR, 0x02);
					delay( 1 );
	
		// Next: set the MLB bit in the TxCFG register:
                tempReg = inpd( RegBase + TX_CONFIG );
                tempReg |= 0x20000000;
                outpd( RegBase + TX_CONFIG, tempReg );
                }
	    
        // If in the PHY loopback mode:
        if( Mode == 2 )
		{
		w_printl(wd, "PHY Loopback Testing In Progress ...   ");
		w_printl(wd, "This may take several minutes ... ");
		// To Enable loopback: 1st disable Xmit/Rcv:
                outpd( RegBase + CR, 0x08);
					delay( 1 );
                outpd( RegBase + CR, 0x02);
					delay( 1 );
	
                // Next: set the Loopback bit in the PHY_BMCR register:
                tempReg = inpd( RegBase + PHY_BMCR );
                tempReg |= 0x04000;
                outpd( RegBase + PHY_BMCR, tempReg );
					delay( 1 );

                // Next set the ATX bit in RxCFG
                tempReg = inpd( RegBase + RX_CONFIG );
                tempReg |= 0x10000000;
                outpd( RegBase + RX_CONFIG, tempReg );
                }

        // If in the External loopback mode:
        if( Mode == 3 )
		{
		w_printl(wd, "External Loopback Testing ...   \n");
		// To Enable loopback: 1st disable Xmit/Rcv:
                outpd( RegBase + CR, 0x08);
					delay( 1 );
                outpd( RegBase + CR, 0x02);
					delay( 1 );
	
                // Next set the ATX bit in RxCFG, so we can receive our data.
                tempReg = inpd( RegBase + RX_CONFIG );
                tempReg |= 0x10000000;
                outpd( RegBase + RX_CONFIG, tempReg );
                }

        // rewrite the Rx descriptor:
        outpd( RegBase + RX_DP, DosLogicalToPhysical( &RxDescriptorPool[ 0 ] ) );

        // Enable Rcv:
        outpd( RegBase, COMMAND_RXENABLE );

        if( !sendPing() )
                {
				w_printl(wd, "\nLoopback--Send Ping failed  \n");
				sys_message(1, "Warning:  Loopback failed!  Press any key to continue ...");
				while (s_getkey(&mstate) == MOUSE_MOVE);      
				sys_message(1, NULL);
				s_close;
				w_close(wd);

				cLoopback=0;

                return ERROR;
                }
                                
                else if( !receivePing() )
                        {
						w_printl(wd, "\nLoopback--Receive Ping Failed  \n");
						sys_message(1, "Warning:  Loopback failed!  Press any key to continue ...");
						while (s_getkey(&mstate) == MOUSE_MOVE);      
						sys_message(1, NULL);
						s_close;
						w_close(wd);

						cLoopback=0;
                        return ERROR;
                        }
                

	for( y=0; y<60; y++ )
	{
	if( PingPacket[ y ] != Packet[ y ]) {
		w_printl(wd, "\nLoopback failed!--unequal ping packet size failed  \n");
		sys_message(1, "Warning:  Loopback failed!  Press any key to continue ...");
		while (s_getkey(&mstate) == MOUSE_MOVE);      
		sys_message(1, NULL);
		s_close;
		w_close(wd);

		cLoopback=0;
		return ERROR;
		}
	}

	w_printl(wd, "\nLoopback Successful!  \n");
	
	w_printl(wd, "\n          Press any key to continue ...");
	while (s_getkey(&mstate) == MOUSE_MOVE);      /* Pause until keypress */

	s_close();
	w_close(wd);
	cLoopback=0;
	sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");

	return SUCCESS;

}

/************************************************
*	Function:   displayPacket
*	Purpose:	Displays the received or transmitted
*				packet
*	Arguments:  none
*	Returns:    The value of the last byte in the packet
*************************************************/
BYTE displayPacket( unsigned long Length, int Transfer)
{   
int   x;
BYTE  temp;
	
	unsigned	key = 0;

	Mouse_State mstate;
	Window wd;
	if (Transfer==TRANSMIT) {
		for(x=0; x<(int)Length; x++)
			Packet[x] = PingPacket[x];
	}
		
	if (!cLoopback)
	{
		if (kbhit())
		{
			key=k_getkey();
		}

		wd=new_diag(&mid_xl_wd_info);
		s_open();
		sys_message(0,"");
		w_printf(wd, "Source:         ");
		for (x=0; x<6; x++) {
			w_printf(wd, "%02X ",Packet[x+6]);
		}	    
		if (cCount > 99) 
			cCount = 0;

		if (Transfer==TRANSMIT)
	        w_printf(wd, "       Transmitted:  %02d",cCount);	

		w_printf(wd, "\nDestination:    ");
		for (x=0; x<6; x++) {
			w_printf(wd, "%02X ",Packet[x]);
		}

		if (cXmitFailed > 99)
			cXmitFailed = 0;

		if (Transfer==TRANSMIT)
			w_printf(wd, "       Tx Failed:    %02d",cXmitFailed);

		w_printf(wd, "\nLength/Type:    ");
		for (x=0; x<2; x++) {
			w_printf(wd, "%02X ",Packet[x+12]);
		}

		if (Transfer == RECEIVE){ 
			w_printf(wd, "\nPacket Number:  %02d  ", cCount);
			w_printf(wd,"\n\nReceived Packet:\n");}
		else
			w_printf(wd, "\n\nTransmitted Packet:\n");
	
		for (x=14; x<(int)(Length-4); x++) {
    		w_printf(wd, "%02X   ",Packet[x]);	
    		temp=Packet[x];
		}
		
		if (cCount)
			delay(1);

		if (key == _ESC || cCount >= 99)
		{
			w_poscurs(wd, 0,13);
			w_print(wd,"\n                                                          ");
			w_print(wd,"\n                                                          ");
			w_print(wd,"\n                                                          ");
			w_poscurs(wd, 0,13);
			if (cCount != 99)
			w_print(wd,"\nUser Interrupted!");
			w_printf(wd,"\n%d Test Packets Received Successfully!!", cCount);
			w_print(wd,"\nPress any key to exit ...");
			cQuit=1; cKeyHit=1;
			while (s_getkey(&mstate) == MOUSE_MOVE);      
		}
		else
		{
			if (cRepeat==0)
			{
			w_printf(wd,"\n\n%d Test Packets Received Successfully!!", cCount);
			w_printf(wd, "\n          Press any key to continue ...");
			cQuit=1; cKeyHit=1;
 			while (s_getkey(&mstate) == MOUSE_MOVE);      
			sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");
			}
		}
		s_close();
		w_close(wd);
	}

	if (Transfer == RECEIVE){ 
		for (x=14; x<(int)(Length-4); x++) {
   			temp=Packet[x];}
	}
	return(temp);
}	


/************************************************
*	Function:   getHeader
*	Purpose:	Reads the Header from the received
*				packet.
*	Arguments:  Address - Start of packet in receive buffer
*	Returns:    nothing
*************************************************/
WORD getHeader(etherPacket_t * Address)
{   
	int   x;
	WORD  Length;

    // Determine length of received packet    
	Length = Header[3];    
    Length << 8;
    Length |= Header[2];
	printf("...a total of %d data bytes received.\n",Length);
    
    printf("\nHeader:        ");
		for (x=0; x<4; x++) {
		printf("%02X ",Header[x]);
	}

    return (Length);

}

/************************************************
*	Function:   waitForPacket
*	Purpose:	output while waiting for packets
*				
*	Arguments:  
*	Returns:    ERROR or SUCCESS
*************************************************/
int waitForPacket()
{
unsigned long isrReg;
int x=0, i=0;

	unsigned	key = 0;
	Mouse_State mstate;
	Window wd;
	wd=new_diag(&mid_xl_wd_info);
	sys_message(0,"");

	s_open();

	w_printf(wd, "Waiting to receive a packet... /\n");

	isrReg = inpd( RegBase + INT_STATUS );

	while( !( isrReg & INT_RXINTR ) ) 
	{

		if( kbhit() ){ 
			getch(); cQuit = 1;
			w_printf(wd, "\n\n\nUser Interrupted!");
			w_printf(wd,"\nReceive Test Packet Procedure Aborted!!");
			w_printf(wd, "\n\n\n          Press any key to continue ...");

			while (s_getkey(&mstate) == MOUSE_MOVE);      /* Pause until keypress */
			cKeyHit=1;
			s_close();
			w_close(wd);
			return( ERROR ); 
		}

	    isrReg = inpd( RegBase + INT_STATUS );
	    if( x++ > 999 )
	    {
	                mdelay( 300 );
					w_poscurs(wd,0,0);
	                w_printf(wd, "Waiting to receive a packet... |" );
	                mdelay( 300 );
					w_poscurs(wd,0,0);
	                w_printf( wd, "Waiting to receive a packet... \\" );
	                mdelay( 300 );
					w_poscurs(wd,0,0);
	                w_printf( wd, "Waiting to receive a packet... /" );

	                x = 0;
	                i++;
	                }

	        if( i>25)
	                {
					w_poscurs(wd,0,2);
	                w_printf(wd, "Not Receiving any data!" );
					delay(1);
					w_poscurs(wd,0,2);
	                w_printf(wd, "                       " );
					i=0;
	                }
	        }

	w_close(wd);
	return( SUCCESS );
}


/************************************************
*	Function:   MacConfig
*	Purpose:	output Mac Configuration
*				
*	Arguments:  
*	Returns:    ERROR or SUCCESS
*************************************************/

int MacConfig( UINT16 nicAddress[ 3 ], unsigned pciConfig, unsigned pwrCaps, unsigned EEpwr  )
{


	int i;
	unsigned	key = 0;

	Mouse_State mstate;
	Window wd;
	wd=new_diag(&mid_wd_info);
	s_open();
	sys_message(0,"");

    w_printf( wd, "\nMacPhyter Demo Card Found!!\n\n");
	w_printf(wd, "I/O Base:.................. %04X\n", RegBase );

    w_printf( wd,"MAC Address:............... " );

    for( i=0; i<3; i++ )
        {
        MACAddr[ i*2+1 ] = ( unsigned char )( nicAddress[ i ] >> 8 );
        MACAddr[ i*2 ] = ( unsigned char )( nicAddress[ i ] );
        w_printf( wd, "%02X %02X ", MACAddr[ i*2 ], MACAddr[ i*2+1 ] );
        }

    w_printf( wd, "\n" );
    
        // WARN for Odd Settings:
        if( (pciConfig & 7) < 7 )
                {
			if( (pciConfig & 1) == 0 ){
                       w_printf(wd, "\n !!WARNING: I/O SPACE NOT GRANTED!" );
			}
                if( (pciConfig & 2) == 0 ){
                       w_printf(wd, "\n !!WARNING: MEMORY SPACE NOT GRANTED!" );
			}
                       
                if( (pciConfig & 4) == 0 ){
                       w_printf(wd, "\n !!WARNING: BUS MASTERING NOT GRANTED!" );
			}
                }

        if( pwrCaps & 1 ) 
        	{
        	w_printf(wd, "\n WARNING: PWR MGT ENABLED!" );
            
            // Read the EEPROM contents to see if it's enabled by default:
		    if( EEpwr & 0x1000 )
				{
				w_printf(wd, "\n WARNING: PWR MGT ENABLE BIT EEPROM DEFAULT!" );
        	    }
        	}
	w_printl(wd, "\n          Press ESC to return to Main Menu");
	key = k_getkey();
	while (key != _ESC)
	{
		key = s_getkey(&mstate);
	}
	s_close();
	w_close(wd);
	sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");
return SUCCESS;
}

/************************************************
*	Function:   DisplayPciRegs
*	Purpose:	dump PCI Cfg registers
*	Returns:    void
*************************************************/
void DisplayPciRegs()
{
int i=0;

	unsigned	key = 0;
	Mouse_State mstate;
	Window wd;
	wd=new_diag(&mid_wd_info);
	s_open();
	sys_message(0,"");

	w_printl(wd, "     Display PCI Register Contents");

	for( i=0; i<PCI_TABLE_SIZE; i++ )
        	{
            w_printf
            	(wd, 
            	"\n    %02X:%s= %08lX  ",
                PciTable[ i ].regOffset,
                PciTable[ i ].regName,
        		PciCfgSpace[ i ] 
        		);

            i++;

            if( i < PCI_TABLE_SIZE )
                w_printf
                	(wd,
            		"      %02X:%s= %08lX  ",
                	PciTable[ i ].regOffset,
                	PciTable[ i ].regName,
        			PciCfgSpace[ i ] 
                    );
	}       	
	w_printl(wd, "\n\n\n            Press ESC to return to Main Menu");
	key = k_getkey();
	while (key != _ESC)
	{
		key = s_getkey(&mstate);
	}
	s_close();
	w_close(wd);
	sys_message(1, "HELP = F1     EXIT = Esc    Select = Up/Dn Arrow    Accept = Enter   ");


}

/************************************************
*	Function:   createPacket
*	Purpose:	Creates a packet to be used to test
*				the loopback features of the NIC.
*				If operating in Word-wide mode, every 
*				other byte is loaded.
*	Arguments:  none
*	Returns:    nothing
*************************************************/
void createPacket(void)
{
	int   x;

	// Setup Ping Packet	
	for( x=0; x<6; x++ )
		PingPacket[ x ] = Destination[ x ];
	
	for( x=0; x<6; x++ )
		PingPacket[ x+6 ] = Source[ x ];

	PingPacket[ 12 ] = 0x0F;
	PingPacket[ 13 ] = 0xF0;
	
	for( x=1; x<24; x++ ){
		PingPacket[ x+13 ] = x;
		PingPacket[ 60-x ] = x;
	}

	if( cLoopback )
		for( x=37; x<PacketSize; x++ )
			PingPacket[ x ] = x-13;
}

int sendMgkPkt( BYTE *destAddr )
{
	int   x, y;

				  
        // Setup Magic Packet header:
	for( x=0; x<6; x++ )
		PingPacket[ x ] = (BYTE)destAddr[ x ];
	
	for( x=0; x<6; x++ )
		PingPacket[ x+6 ] = Source[ x ];

	PingPacket[ 12 ] = 0x0F;
	PingPacket[ 13 ] = 0xF0;

        for( y=0; y<6; y++ ) PingPacket[ 14+y ] = 0xFF;

	// data for packet consists of 16 destAddr:
        for( y=0; y<=15; y++){
		for( x=0; x<6; x++ ){
                        PingPacket[ (6*y)+20+x ] = (BYTE)destAddr[ x ];
		}
	}

    if( MacSend( PingPacket, 120 ) == ERROR )
            {
            return( ERROR );
            }

	// Now display a copy of the Magic Packet Sent:
	cCount=1;
    displayPacket( 127, TRANSMIT );
	cCount=0;
	return SUCCESS;
}


/************************************************
*	Function:   sendPing
*	Purpose:	Creates a packet to be used for 
*				the Ping test.  Once the packet
*				is assembled, it is transmitted.
*				If operating in Word-wide mode, every 
*				other byte is loaded.
*	Arguments:  none
*	Returns:    value of last data byte transmitted.
*************************************************/
BYTE sendPing(void)
{
	int   x;
    
	createPacket();							// Setup Ping Packet

	if( cReturn ) {
		for (x=12; x<60; x++)
			PingPacket[x]=Packet[x];
	}
	
	if( cTransmit )
		PingPacket[59] = 0xFF;
	
	if (cQuit)
		PingPacket[59] = 0x07;
		
    if( !MacSend( PingPacket, PINGSIZE-4 ) ){
		cXmitFailed++;
		displayPacket( PINGSIZE, TRANSMIT );
        return( ERROR );
    }

	displayPacket( PINGSIZE, TRANSMIT );
	return SUCCESS;
}

/************************************************
*	Function:   receivePing
*	Purpose:	Waits for receive interrupt to be
*				set, and then processes the incoming
*				packet.  During the first iteration,
*				the destination address is setup.
*	Arguments:  none
*	Returns:    value of last data byte received
*************************************************/
BYTE receivePing(void)
{	
int   x = 0, k;
unsigned i=0;
BYTE  temp;
unsigned long Length=0;
unsigned long isrReg;
	
        // Check for Received data and check the buffer:
        if(
          !( IsrRegister & INT_RXINTR )
          &&
          ( !RxDescriptorPool[ RxDescriptorPoolIdx ].cmdStatus.rxStatus.ReceiveOwn )
          )
		if( !waitForPacket() ) return( ERROR );

while( RxDescriptorPoolIdx < DESCRIPTOR_POOL_SIZE )
{
        if( !RxDescriptorPool[ RxDescriptorPoolIdx ].cmdStatus.rxStatus.ReceiveOwn )
             break;   

        Length = RxDescriptorPool[ RxDescriptorPoolIdx ].cmdStatus.rxStatus.descriptorSize;

		if (Length <=0)
		{
			cQuit=1; cKeyHit=1;
		}

	// In case we get a bad length:
	if( Length > MAX_ETHER_PACKET_SIZE ) Length = MAX_ETHER_PACKET_SIZE;

	//Place the Packet into Packet array
        for( i=0; i<Length; i++ ) Packet[ i ] = RxDataPool[ RxDescriptorPoolIdx ][ i ];
				    
	cCount++;
	temp = displayPacket( Length, RECEIVE );
 
        for (x=0; x<6; x++)
         Destination[ x ] = Packet[ x+6 ];
    				  
        cReturn = 1;             // Indicate that packet was received

	    if( !cReceiveOnly) sendPing();
   		
    // Do the clean up on the descriptor:
        RxDescriptorPool[ RxDescriptorPoolIdx ].cmdStatus.cmdStatus = 0;
        RxDescriptorPool[ RxDescriptorPoolIdx ].cmdStatus.rxStatus.descriptorSize
                = sizeof( etherPacket_t );
    
        if (temp == 0x07){ cRepeat=0; cQuit = 1; cKeyHit=1; break; }

		if (cReceiveOnly){
			RxDescriptorPoolIdx=DESCRIPTOR_POOL_SIZE-1;
			cQuit=1;
		}

        RxDescriptorPoolIdx++;
		
}

    // reset the descriptor head and receive enable:
    if( RxDescriptorPoolIdx == DESCRIPTOR_POOL_SIZE )
        {
        RxDescriptorPoolIdx = 0;
		outpd (RegBase + CR, 0x0);				// Reset Receiver
		delay(1);
        outpd( RegBase + RX_DP, DosLogicalToPhysical( &RxDescriptorPool[ 0 ] ) );

        // Enable Rcv:
        outpd( RegBase, COMMAND_RXENABLE );
        }
	return( temp );
}


/*************************************************
*       Function:   MacSend
*	Purpose:	Transfers a packet from the NIC onto
*				the network.  The Buffer is first DMA'd
*				to the NIC using PCtoNIC.  The appropriate
*				registers are then set and a transmit 
*				command is issued.
*	Arguments:  Buffer - Pointer to the data to transfer
*				Length - The length of the packet
*	Returns:    nothing
*************************************************/
int MacSend(BYTE *Buffer, WORD Length)
{
int i = 100;
int retVal = SUCCESS;
unsigned long cmdReg;
    
    // zero out the memory. 
    memset( &tempDesc, '\0', sizeof( HALSpace_t ) );

    _disable();                                                     // Disable interrupts


    // Initialize the descriptor:
    tempDesc.physPointer = DosLogicalToPhysical( Buffer );
    tempDesc.cmdStatus.txStatus.descriptorSize = Length;
    tempDesc.cmdStatus.txStatus.TransmitOwn = 1;

	// Wait for the MacPhyter to finish transmitting:
	_enable();
        while( i-- )
		{
                if( !inpd( RegBase+TX_DP ) ) break;
		}

	// If MacPhyter never frees up then quit and exit:
	if( !i )
                {
                return( ERROR );
                }

	// Otherwise post the descriptor to the chip:
        outpd( RegBase+TX_DP, DosLogicalToPhysical( &tempDesc ) );

	// Set TX ENABLE bit:
                cmdReg = inpd( RegBase );
                cmdReg |= COMMAND_TXENABLE;
                outpd( RegBase, cmdReg );

	// Wait for processing:
					delay( 1 );

        IsrRegister  = inpd( RegBase + INT_STATUS );

        if( IsrRegister  & ( INT_TXOK  | INT_TXERR) ) 
                {
                if( IsrRegister & INT_TXOK )
						retVal = SUCCESS;
                else
                 {
                 retVal = ERROR;
                 }
                }
           else
                {
                retVal = ERROR;
                }

        return( retVal );
}               

void interrupt far mcPhyISR( )
{
unsigned long isrReg;

	isrReg = inpd( RegBase + INT_STATUS );	
}

/* This Function places the MacPhyter into a variety of test modes: */
int pwrMgtMode( int wakeOption )
{
unsigned vendorId = 0x100B, value; 
unsigned long dvalue;
unsigned deviceId = 0x20;
int index = 0;
int bus;
int dev;
int func;

	initNIC();
    // Find the card on the PCI Bus:
    if( PciBiosFindDevice( vendorId, deviceId, index, &bus, &dev, &func ) )
            {
            return( ERROR );
            }

	// Read WCSR then clear (if any) internal Wake Event & Reset WCSR
	if (inpd( RegBase + WCSR) & 0x1)		// if PHY interrupt
	{
		if (inpd (RegBase + PHY_MISR) & 0x4000)	//if there was Link change
			outpd (RegBase + PHY_MISR, 0x0);	//Clear and reset MISR

	// Reset WSCR
		outpd( RegBase + WCSR, 0x00000000);
		delay(1);
	}


	// Read WCSR to see if it contains any Wake Packet
	if (inpd (RegBase +WCSR) & 0x200)			// if there was Magic Packet
	{
		outpd (RegBase + CR, 0x0);				// Reset Receiver
		delay(1);
		outpd (RegBase +RX_DP, DosLogicalToPhysical( &RxDescriptorPool[ 0 ] ) );
												// Give RXDP a valid receiv descriptor pointer
		outpd (RegBase + CR, 0x4);				// Receiver Enable;
	}


    // Clear any PMCSR activity and ready for Wake Event:
    PciBiosWriteConfigWord( bus, dev, func, 0x44, 0x8100 );
    PciBiosWriteConfigWord( bus, dev, func, 0x46, 0x0000 );

    // Enable Promiscuous Reception:
    outpd( RegBase + RX_FILTER_CTRL, 0xF0000000 );

    // Set the WCSR with the Wake Option:
    outpd( RegBase + WCSR, (unsigned long)wakeOption );

    // If Waking on link Change: setup micr and misr:
    if( wakeOption == 1 )
        {
        // First Set the IMR to enable Phy AND PME Interrupts:
        outpd( RegBase + INT_MASK, 0x6000 );
    
        // Then Setup the PHY to generate ints on link change
        outpd( RegBase + PHY_MICR, 0x2 );
        outpd( RegBase + PHY_MISR, 0x4000 );
        }


    // Clear any PMCSR activity and ready for Wake Event:
    PciBiosWriteConfigWord( bus, dev, func, 0x44, 0x0103 );

    // Return All's Well
    return( SUCCESS );
}
